@@ -18,7 +18,11 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: about.c 1318 2007-02-23 12:14:14Z eht16 $
+ * $Id: about.c 1540 2007-05-16 17:08:22Z eht16 $
+ */
+
+/*
+ * About dialog and credits.
  */
 
 #include "about.h"
@@ -35,7 +39,7 @@ static GtkWidget *gb_window = NULL;
 #define INFO "<span size=\"larger\" weight=\"bold\">%s</span>"
 #define CODENAME "<span weight=\"bold\">\"" GEANY_CODENAME "\"</span>"
 #define BUILDDATE "<span size=\"smaller\">%s</span>"
-#define COPYRIGHT "Copyright (c)  2005-2007 \n Enrico Tröger \nAll rights reserved."
+#define COPYRIGHT "Copyright (c)  2005-2007\nEnrico Tröger\nNick Treleaven\nFrank Lanitz\nAll rights reserved."
 #define CREDITS \
 "<span size=\"larger\" weight=\"bold\">%s</span>\n\t\
 Enrico Tröger - %s\n\t\
@@ -48,6 +52,8 @@ Frank Lanitz - %s\n\t\
 \
 Yura Semashko - <language> be_BY\n\t\
 &lt;yurand2@gmail.com&gt;\n\n\t\
+Dilyan Rusev - <language> bg\n\t\
+&lt;dilyanrusev@gmail.com&gt;\n\n\t\
 Toni Garcia-Navarro - <language> ca_ES\n\t\
 &lt;topi@elpiset.net&gt;\n\n\t\
 Petr Messner - <language> cs_CZ \n\t\
@@ -59,7 +65,7 @@ Damián Viano - <language> es\n\t\
 Harri Koskinen  - <language> fi_FI\n\t\
 &lt;harri@fastmonkey.org&gt;\n\n\t\
 Jean-Philippe Moal - <language> fr\n\t\
-&lt;skateinmars@momo.servegame.org&gt;\n\n\t\
+&lt;skateinmars@skateinmars.net&gt;\n\n\t\
 Gabor Kmetyko - <language> hu\n\t\
 &lt;kg_kilo@freemail.hu&gt;\n\n\t\
 M. Baldinelli - <language> it\n\t\
@@ -120,6 +126,7 @@ static GtkWidget *create_dialog(void)
 	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(app->window));
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_window_set_title(GTK_WINDOW(dialog), _("About Geany"));
+	gtk_widget_set_name(dialog, "GeanyDialog");
 	button = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_CLOSE);
 	gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
