@@ -1,7 +1,8 @@
 /*
  *      build.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: build.h 1068 2006-12-08 15:50:10Z ntrel $
+ * $Id: build.h 1418 2007-03-24 12:40:20Z ntrel $
  */
 
 
@@ -62,21 +63,11 @@ typedef struct
 
 void build_finalize();
 
-GPid build_make_file(gint idx, gint build_opts);
-
-GPid build_compile_file(gint idx);
-
-GPid build_link_file(gint idx);
-
-GPid build_compile_tex_file(gint idx, gint mode);
-
-GPid build_view_tex_file(gint idx, gint mode);
-
-GPid build_run_cmd(gint idx);
+gboolean build_parse_make_dir(const gchar *string, gchar **prefix);
 
 void build_menu_update(gint idx);
 
-BuildMenuItems *build_get_menu_items(gint filetype_id);
+BuildMenuItems *build_get_menu_items(gint filetype_idx);
 
 
 void
@@ -99,11 +90,8 @@ void
 on_build_execute_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_build_arguments_activate            (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_build_next_error                    (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+
 #endif
