@@ -18,9 +18,13 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: callbacks.c 1270 2007-02-12 18:42:15Z eht16 $
+ * $Id: callbacks.c 1534 2007-05-15 13:02:23Z eht16 $
  */
 
+/*
+ * Callbacks used by Glade. These are mainly in response to menu item and button events in the
+ * main window. Callbacks not used by Glade should go elsewhere.
+ */
 
 #include "geany.h"
 
@@ -53,6 +57,7 @@
 #include "main.h"
 #include "symbols.h"
 #include "tools.h"
+#include "project.h"
 
 #ifdef HAVE_VTE
 # include "vte.h"
@@ -79,6 +84,8 @@ static gboolean insert_callback_from_menu = FALSE;
 // the selection-changed signal from tv.tree_openfiles
 //static gboolean switch_tv_notebook_page = FALSE;
 
+CallbacksData callbacks_data = {-1};
+
 
 // real exit function
 gint destroyapp(GtkWidget *widget, gpointer gdata)
@@ -190,19 +197,10 @@ void
 on_new1_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	document_new_file(NULL);
+	document_new_file(NULL, NULL);
 }
 
 
-void
-on_new_with_template                   (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
-{
-	document_new_file((filetype*) user_data);
-}
-
-
-
 void
 on_save1_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -243,7 +241,7 @@ on_save_all1_activate                  (GtkMenuItem     *menuitem,
 		else
 			document_save_file(idx, FALSE);
 	}
-	ui_update_tag_list(cur_idx, TRUE);
+	treeviews_update_tag_list(cur_idx, TRUE);
 	ui_set_window_title(cur_idx);
 }
 
@@ -428,7 +426,7 @@ void
 on_preferences1_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	dialogs_show_prefs_dialog();
+	prefs_show_dialog();
 }
 
 
@@ -545,7 +543,7 @@ void
 on_toolbutton8_clicked                 (GtkToolButton   *toolbutton,
                                         gpointer         user_data)
 {
-	document_new_file(NULL);
+	document_new_file(NULL, NULL);
 }
 
 // open file
@@ -566,11 +564,11 @@ on_toolbutton10_clicked                (GtkToolButton   *toolbutton,
 }
 
 
+// store text, clear search flags so we can use Search->Find Next/Previous
 static void setup_find_next(GtkEditable *editable)
 {
 	g_free(search_data.text);
-	search_data.text = g_strdup(gtk_editable_get_chars(editable, 0, -1));
-	// clear search flags so can later use Search->Find Next/Previous
+	search_data.text = gtk_editable_get_chars(editable, 0, -1);
 	search_data.flags = 0;
 	search_data.backwards = FALSE;
 }
@@ -700,38 +698,17 @@ on_toolbutton15_clicked                (GtkToolButton   *toolbutton,
 }
 
 
-// changes window-title on switching tabs and lots of other things
 void
 on_notebook1_switch_page               (GtkNotebook     *notebook,
                                         GtkNotebookPage *page,
                                         guint            page_num,
                                         gpointer         user_data)
 {
-	gint idx;
-
-	if (closing_all) return;
-
-	// guint == -1 seems useless, but it isn't!
-	if (page_num == (guint) -1 && page != NULL)
-		idx = document_find_by_sci(SCINTILLA(page));
-	else
-		idx = document_get_n_idx(page_num);
-
-	if (idx >= 0 && app->opening_session_files == FALSE)
-	{
-		gtk_tree_model_foreach(GTK_TREE_MODEL(tv.store_openfiles), treeviews_find_node, GINT_TO_POINTER(idx));
-
-		document_set_text_changed(idx);
-		ui_update_popup_reundo_items(idx);
-		ui_document_show_hide(idx); // update the document menu
-		build_menu_update(idx);
-		ui_update_statusbar(idx, -1);
-		ui_set_window_title(idx);
-		ui_update_tag_list(idx, FALSE);
-	}
+	callbacks_data.last_doc_idx = document_get_cur_idx();
 }
 
 
+// changes window-title on switching tabs and lots of other things
 void
 on_notebook1_switch_page_after         (GtkNotebook     *notebook,
                                         GtkNotebookPage *page,
@@ -750,10 +727,18 @@ on_notebook1_switch_page_after         (GtkNotebook     *notebook,
 
 	if (idx >= 0 && app->opening_session_files == FALSE)
 	{
+		gtk_tree_model_foreach(GTK_TREE_MODEL(tv.store_openfiles), treeviews_find_node, GINT_TO_POINTER(idx));
+
+		document_set_text_changed(idx);	// also sets window title and status bar
+		ui_update_popup_reundo_items(idx);
+		ui_document_show_hide(idx); // update the document menu
+		build_menu_update(idx);
+		treeviews_update_tag_list(idx, FALSE);
+
 		utils_check_disk_status(idx, FALSE);
 
 #ifdef HAVE_VTE
-	vte_cwd(doc_list[idx].file_name, FALSE);
+		vte_cwd(doc_list[idx].file_name, FALSE);
 #endif
 	}
 }
@@ -784,7 +769,6 @@ on_file_open_dialog_response           (GtkDialog *dialog,
 	if (response == GTK_RESPONSE_ACCEPT || response == GTK_RESPONSE_APPLY)
 	{
 		GSList *filelist;
-		GSList *flist;
 		gint filetype_idx = gtk_combo_box_get_active(GTK_COMBO_BOX(
 						lookup_widget(GTK_WIDGET(dialog), "filetype_combo")));
 		gint encoding_idx = gtk_combo_box_get_active(GTK_COMBO_BOX(
@@ -798,15 +782,10 @@ on_file_open_dialog_response           (GtkDialog *dialog,
 			charset = encodings[encoding_idx].charset;
 
 		filelist = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(app->open_filesel));
-		flist = filelist;
-		while (flist != NULL)
+		if (filelist != NULL)
 		{
-			if (g_file_test((gchar*) flist->data, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
-			{
-				document_open_file(-1, (gchar*) flist->data, 0, ro, ft, charset);
-			}
-			g_free(flist->data);
-			flist = flist->next;
+			document_open_files(filelist, ro, ft, charset);
+			g_slist_foreach(filelist, (GFunc) g_free, NULL);	// free filenames
 		}
 		g_slist_free(filelist);
 	}
@@ -1038,17 +1017,17 @@ toolbar_popup_menu                     (GtkWidget *widget,
 {
 	if (event->button == 3)
 	{
-		GtkWidget *widget;
+		GtkWidget *w;
 
 		ignore_toolbar_toggle = TRUE;
 
 		switch (app->toolbar_icon_style)
 		{
-			case 0: widget = lookup_widget(app->toolbar_menu, "images_only2"); break;
-			case 1: widget = lookup_widget(app->toolbar_menu, "text_only2"); break;
-			default: widget = lookup_widget(app->toolbar_menu, "images_and_text2"); break;
+			case 0: w = lookup_widget(app->toolbar_menu, "images_only2"); break;
+			case 1: w = lookup_widget(app->toolbar_menu, "text_only2"); break;
+			default: w = lookup_widget(app->toolbar_menu, "images_and_text2"); break;
 		}
-		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(widget), TRUE);
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), TRUE);
 
 		switch (app->toolbar_icon_size)
 		{
@@ -1056,7 +1035,7 @@ toolbar_popup_menu                     (GtkWidget *widget,
 					widget = lookup_widget(app->toolbar_menu, "large_icons1"); break;
 			default: widget = lookup_widget(app->toolbar_menu, "small_icons1"); break;
 		}
-		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(widget), TRUE);
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), TRUE);
 
 		ignore_toolbar_toggle = FALSE;
 
@@ -1068,17 +1047,6 @@ toolbar_popup_menu                     (GtkWidget *widget,
 }
 
 
-void
-on_filetype_change                     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
-{
-	gint idx = document_get_cur_idx();
-	if (app->ignore_callback || idx < 0 || ! doc_list[idx].is_valid) return;
-
-	document_set_filetype(idx, (filetype*)user_data);
-}
-
-
 void
 on_to_lower_case1_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -1226,13 +1194,15 @@ void
 on_goto_tag_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+	const gint forward_types = tm_tag_prototype_t | tm_tag_externvar_t;
 	gint type;
 	TMTag *tmtag;
 
+	// goto tag definition: all except prototypes / forward declarations / externs
 	if (menuitem == GTK_MENU_ITEM(lookup_widget(app->popup_menu, "goto_tag_definition1")))
-		type = tm_tag_function_t;
+		type = tm_tag_max_t - forward_types;
 	else
-		type = tm_tag_prototype_t;
+		type = forward_types;
 
 	tmtag = symbols_find_in_workspace(editor_info.current_word, type);
 	if (tmtag != NULL)
@@ -1244,10 +1214,10 @@ on_goto_tag_activate                   (GtkMenuItem     *menuitem,
 	}
 	// if we are here, there was no match and we are beeping ;-)
 	utils_beep();
-	if (type == tm_tag_prototype_t)
-		ui_set_statusbar(_("Declaration of \"%s()\" not found"), editor_info.current_word);
+	if (type == forward_types)
+		ui_set_statusbar(_("Forward declaration \"%s\" not found."), editor_info.current_word);
 	else
-		ui_set_statusbar(_("Definition of \"%s()\" not found"), editor_info.current_word);
+		ui_set_statusbar(_("Definition of \"%s\" not found."), editor_info.current_word);
 }
 
 
@@ -1270,7 +1240,7 @@ on_show_color_chooser1_activate        (GtkMenuItem     *menuitem,
 	if (idx == -1 || ! doc_list[idx].is_valid)
 		return;
 
-	sci_cb_find_current_word(doc_list[idx].sci, pos, colour, sizeof colour);
+	sci_cb_find_current_word(doc_list[idx].sci, pos, colour, sizeof colour, NULL);
 	dialogs_show_color(colour);
 }
 
@@ -1323,6 +1293,22 @@ on_find_previous1_activate             (GtkMenuItem     *menuitem,
 }
 
 
+void
+on_find_nextsel1_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	search_find_selection(document_get_cur_idx(), FALSE);
+}
+
+
+void
+on_find_prevsel1_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	search_find_selection(document_get_cur_idx(), TRUE);
+}
+
+
 void
 on_replace1_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -1343,7 +1329,7 @@ void
 on_toolbutton_new_clicked              (GtkToolButton   *toolbutton,
                                         gpointer         user_data)
 {
-	document_new_file(NULL);
+	document_new_file(NULL, NULL);
 }
 
 
@@ -1429,7 +1415,7 @@ void
 on_help_shortcuts1_activate            (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	dialogs_show_keyboard_shortcuts();
+	keybindings_show_shortcuts();
 }
 
 
@@ -1461,27 +1447,7 @@ on_comments_function_activate          (GtkMenuItem     *menuitem,
 	line = utils_get_current_function(idx, &cur_tag);
 	pos = sci_get_position_from_line(doc_list[idx].sci, line - 1);
 
-	switch (doc_list[idx].file_type->id)
-	{
-		case GEANY_FILETYPES_PASCAL:
-		{
-			text = templates_get_template_function(GEANY_TEMPLATE_FUNCTION_PASCAL, cur_tag);
-			break;
-		}
-		case GEANY_FILETYPES_PYTHON:
-		case GEANY_FILETYPES_RUBY:
-		case GEANY_FILETYPES_SH:
-		case GEANY_FILETYPES_MAKE:
-		case GEANY_FILETYPES_PERL:
-		{
-			text = templates_get_template_function(GEANY_TEMPLATE_FUNCTION_ROUTE, cur_tag);
-			break;
-		}
-		default:
-		{
-			text = templates_get_template_function(GEANY_TEMPLATE_FUNCTION, cur_tag);
-		}
-	}
+	text = templates_get_template_function(doc_list[idx].file_type->id, cur_tag);
 
 	sci_insert_text(doc_list[idx].sci, pos, text);
 	g_free(text);
@@ -1493,40 +1459,16 @@ on_comments_multiline_activate         (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	gint idx = document_get_cur_idx();
-	gchar *text;
 
-	if (doc_list[idx].file_type == NULL)
+	if (! DOC_IDX_VALID(idx) || doc_list[idx].file_type == NULL)
 	{
 		ui_set_statusbar(_("Please set the filetype for the current file before using this function."));
 		return;
 	}
 
-	switch (doc_list[idx].file_type->id)
-	{
-		case GEANY_FILETYPES_PASCAL:
-		{
-			text = templates_get_template_generic(GEANY_TEMPLATE_MULTILINE_PASCAL);
-			break;
-		}
-		case GEANY_FILETYPES_PYTHON:
-		case GEANY_FILETYPES_RUBY:
-		case GEANY_FILETYPES_SH:
-		case GEANY_FILETYPES_MAKE:
-		case GEANY_FILETYPES_PERL:
-		{
-			text = templates_get_template_generic(GEANY_TEMPLATE_MULTILINE_ROUTE);
-			break;
-		}
-		default:
-		{
-			text = templates_get_template_generic(GEANY_TEMPLATE_MULTILINE);
-		}
-	}
-
 	verify_click_pos(idx); // make sure that the click_pos is valid
 
-	sci_insert_text(doc_list[idx].sci, editor_info.click_pos, text);
-	g_free(text);
+	sci_cb_insert_multiline_comment(idx);
 }
 
 
@@ -1537,7 +1479,7 @@ on_comments_gpl_activate               (GtkMenuItem     *menuitem,
 	gint idx = document_get_cur_idx();
 	gchar *text;
 
-	text = templates_get_template_gpl(FILETYPE_ID(doc_list[idx].file_type));
+	text = templates_get_template_licence(FILETYPE_ID(doc_list[idx].file_type), GEANY_TEMPLATE_GPL);
 
 	verify_click_pos(idx); // make sure that the click_pos is valid
 
@@ -1546,6 +1488,24 @@ on_comments_gpl_activate               (GtkMenuItem     *menuitem,
 }
 
 
+void
+on_comments_bsd_activate               (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+
+	gint idx = document_get_cur_idx();
+	gchar *text;
+
+	text = templates_get_template_licence(FILETYPE_ID(doc_list[idx].file_type), GEANY_TEMPLATE_BSD);
+
+	verify_click_pos(idx); // make sure that the click_pos is valid
+
+	sci_insert_text(doc_list[idx].sci, editor_info.click_pos, text);
+	g_free(text);
+
+}
+
+
 void
 on_comments_changelog_activate         (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -1691,106 +1651,6 @@ on_insert_include_activate             (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_includes_arguments_dialog_response  (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data)
-{
-	filetype *ft = user_data;
-
-	g_return_if_fail(ft != NULL);
-
-	if (response == GTK_RESPONSE_ACCEPT)
-	{
-		const gchar *newstr;
-		struct build_programs *programs = ft->programs;
-
-		if (ft->actions->can_compile)
-		{
-			newstr = gtk_entry_get_text(
-					GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "includes_entry1")));
-			if (! utils_str_equal(newstr, programs->compiler))
-			{
-				if (programs->compiler) g_free(programs->compiler);
-				programs->compiler = g_strdup(newstr);
-				programs->modified = TRUE;
-			}
-		}
-		if (ft->actions->can_link)
-		{
-			newstr = gtk_entry_get_text(
-					GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "includes_entry2")));
-			if (! utils_str_equal(newstr, programs->linker))
-			{
-				if (programs->linker) g_free(programs->linker);
-				programs->linker = g_strdup(newstr);
-				programs->modified = TRUE;
-			}
-		}
-		if (ft->actions->can_exec)
-		{
-			newstr = gtk_entry_get_text(
-					GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "includes_entry3")));
-			if (! utils_str_equal(newstr, programs->run_cmd))
-			{
-				if (programs->run_cmd) g_free(programs->run_cmd);
-				programs->run_cmd = g_strdup(newstr);
-				programs->modified = TRUE;
-			}
-		}
-	}
-}
-
-
-void
-on_includes_arguments_tex_dialog_response  (GtkDialog *dialog,
-                                            gint response,
-                                            gpointer user_data)
-{
-	filetype *ft = user_data;
-	g_return_if_fail(ft != NULL);
-
-	if (response == GTK_RESPONSE_ACCEPT)
-	{
-		const gchar *newstr;
-		struct build_programs *programs = ft->programs;
-
-		newstr = gtk_entry_get_text(
-				GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "tex_entry1")));
-		if (! utils_str_equal(newstr, programs->compiler))
-		{
-			if (programs->compiler) g_free(programs->compiler);
-			programs->compiler = g_strdup(newstr);
-			programs->modified = TRUE;
-		}
-		newstr = gtk_entry_get_text(
-				GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "tex_entry2")));
-		if (! utils_str_equal(newstr, programs->linker))
-		{
-			if (programs->linker) g_free(programs->linker);
-			programs->linker = g_strdup(newstr);
-			programs->modified = TRUE;
-		}
-		newstr = gtk_entry_get_text(
-				GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "tex_entry3")));
-		if (! utils_str_equal(newstr, programs->run_cmd))
-		{
-			if (programs->run_cmd) g_free(programs->run_cmd);
-			programs->run_cmd = g_strdup(newstr);
-			programs->modified = TRUE;
-		}
-		newstr = gtk_entry_get_text(
-				GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "tex_entry4")));
-		if (! utils_str_equal(newstr, programs->run_cmd2))
-		{
-			if (programs->run_cmd2) g_free(programs->run_cmd2);
-			programs->run_cmd2 = g_strdup(newstr);
-			programs->modified = TRUE;
-		}
-	}
-}
-
-
 void
 on_file_open_check_hidden_toggled      (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
@@ -1852,6 +1712,9 @@ on_menu_remove_indicators1_activate    (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	gint idx = document_get_cur_idx();
+
+	if (! DOC_IDX_VALID(idx))
+		return;
 	document_clear_indicators(idx);
 }
 
@@ -1903,15 +1766,27 @@ void
 on_menu_show_sidebar1_toggled          (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data)
 {
+	static gint active_page = -1;
+
 	if (app->ignore_callback) return;
 
-	app->sidebar_visible = ! app->sidebar_visible;
-	if (! app->sidebar_openfiles_visible && ! app->sidebar_symbol_visible)
+	if (app->sidebar_visible)
 	{
-		app->sidebar_symbol_visible = TRUE;
-		app->sidebar_openfiles_visible = TRUE;
+		// to remember the active page because GTK (e.g. 2.8.18) doesn't do it and shows always
+		// the last page (for unknown reason, with GTK 2.6.4 it works)
+		active_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(app->treeview_notebook));
 	}
+
+	app->sidebar_visible = ! app->sidebar_visible;
+
+	if ((! app->sidebar_openfiles_visible && ! app->sidebar_symbol_visible))
+	{
+		app->sidebar_openfiles_visible = TRUE;
+		app->sidebar_symbol_visible = TRUE;
+	}
+
 	ui_treeviews_show_hide(TRUE);
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(app->treeview_notebook), active_page);
 }
 
 
@@ -1945,7 +1820,7 @@ on_menu_comment_line1_activate         (GtkMenuItem     *menuitem,
 {
 	gint idx = document_get_cur_idx();
 	if (idx == -1 || ! doc_list[idx].is_valid) return;
-	sci_cb_do_comment(idx, -1);
+	sci_cb_do_comment(idx, -1, FALSE);
 }
 
 
@@ -1988,7 +1863,7 @@ on_menu_increase_indent1_activate      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
+	if (! DOC_IDX_VALID(idx)) return;
 
 	if (sci_get_lines_selected(doc_list[idx].sci) > 1)
 	{
@@ -1996,16 +1871,18 @@ on_menu_increase_indent1_activate      (GtkMenuItem     *menuitem,
 	}
 	else
 	{
-		gint line, ind_pos, old_pos;
+		gint line, ind_pos, old_pos, new_pos, step;
 
 		old_pos = sci_get_current_position(doc_list[idx].sci);
 		line = sci_get_current_line(doc_list[idx].sci, old_pos);
 		ind_pos = sci_get_line_indent_position(doc_list[idx].sci, line);
+		// when using tabs increase cur pos by 1, when using space increase it by tab_width
+		step = (app->pref_editor_use_tabs) ? 1 : app->pref_editor_tab_width;
+		new_pos = (old_pos > ind_pos) ? old_pos + step : old_pos;
 
 		sci_set_current_position(doc_list[idx].sci, ind_pos, TRUE);
 		sci_cmd(doc_list[idx].sci, SCI_TAB);
-		sci_set_current_position(doc_list[idx].sci,
-			(old_pos > ind_pos) ? old_pos + 1 : old_pos, TRUE);
+		sci_set_current_position(doc_list[idx].sci, new_pos, TRUE);
 	}
 }
 
@@ -2015,7 +1892,7 @@ on_menu_decrease_indent1_activate      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
+	if (! DOC_IDX_VALID(idx)) return;
 
 	if (sci_get_lines_selected(doc_list[idx].sci) > 1)
 	{
@@ -2023,18 +1900,25 @@ on_menu_decrease_indent1_activate      (GtkMenuItem     *menuitem,
 	}
 	else
 	{
-		gint line, ind_pos, old_pos;
+		gint line, ind_pos, old_pos, new_pos, step, indent;
 
 		old_pos = sci_get_current_position(doc_list[idx].sci);
 		line = sci_get_current_line(doc_list[idx].sci, old_pos);
 		ind_pos = sci_get_line_indent_position(doc_list[idx].sci, line);
+		step = (app->pref_editor_use_tabs) ? 1 : app->pref_editor_tab_width;
+		new_pos = (old_pos >= ind_pos) ? old_pos - step : old_pos;
 
 		if (ind_pos == sci_get_position_from_line(doc_list[idx].sci, line))
 			return;
+
 		sci_set_current_position(doc_list[idx].sci, ind_pos, TRUE);
-		sci_cmd(doc_list[idx].sci, SCI_BACKTAB);
-		sci_set_current_position(doc_list[idx].sci,
-			(old_pos >= ind_pos) ? old_pos - 1 : old_pos, TRUE);
+		indent = sci_get_line_indentation(doc_list[idx].sci, line);
+		indent -= app->pref_editor_tab_width;
+		if (indent < 0)
+			indent = 0;
+		sci_set_line_indentation(doc_list[idx].sci, line, indent);
+
+		sci_set_current_position(doc_list[idx].sci, new_pos, TRUE);
 	}
 }
 
@@ -2053,6 +1937,10 @@ void
 on_menu_insert_special_chars1_activate (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+	// refuse opening the dialog if we don't have an active tab
+	gint idx = document_get_cur_idx();
+	if (idx == -1 || ! doc_list[idx].is_valid) return;
+
 	tools_show_dialog_insert_special_chars();
 }
 
@@ -2075,6 +1963,15 @@ on_menu_comments_gpl_activate          (GtkMenuItem     *menuitem,
 }
 
 
+void
+on_menu_comments_bsd_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	insert_callback_from_menu = TRUE;
+	on_comments_bsd_activate(menuitem, user_data);
+}
+
+
 void
 on_menu_insert_include_activate        (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -2091,3 +1988,231 @@ on_menu_insert_date_activate           (GtkMenuItem     *menuitem,
 	insert_callback_from_menu = TRUE;
 	on_insert_date_activate(menuitem, user_data);
 }
+
+
+void
+on_project_new1_activate               (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	project_new();
+}
+
+
+void
+on_project_open1_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	project_open();
+}
+
+
+void
+on_project_close1_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	project_close();
+}
+
+
+void
+on_project_properties1_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	project_properties();
+}
+
+
+void
+on_menu_project1_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	static GtkWidget *item_close = NULL;
+	static GtkWidget *item_properties = NULL;
+
+	if (item_close == NULL)
+	{
+		item_close = lookup_widget(app->window, "project_close1");
+		item_properties = lookup_widget(app->window, "project_properties1");
+	}
+
+	gtk_widget_set_sensitive(item_close, (app->project != NULL));
+	gtk_widget_set_sensitive(item_properties, (app->project != NULL));
+}
+
+
+void
+on_menu_open_selected_file1_activate   (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	gint idx = document_get_cur_idx();
+	gchar *filename = NULL;
+
+	if (idx == -1 || ! doc_list[idx].is_valid) return;
+
+	if (sci_get_lines_selected(doc_list[idx].sci) == 1)
+	{
+		filename = g_malloc(sci_get_selected_text_length(doc_list[idx].sci));
+		sci_get_selected_text(doc_list[idx].sci, filename);
+	}
+	else if (sci_get_lines_selected(doc_list[idx].sci) == 0)
+	{	// use the word at current cursor position
+		gchar word[GEANY_MAX_WORD_LENGTH];
+
+		sci_cb_find_current_word(doc_list[idx].sci, -1, word, sizeof(word), GEANY_WORDCHARS"./");
+		if (word[0] != '\0')
+			filename = g_strdup(word);
+	}
+
+	if (filename != NULL)
+	{
+		gchar *locale_filename;
+
+		if (! g_path_is_absolute(filename))
+		{	// relative filename, add the path of the current file
+			gchar *path;
+			gchar *tmp = filename;
+
+			// use the projects base path if we have an open project (useful?)
+			if (app->project != NULL && app->project->base_path != NULL)
+				path = g_strdup(app->project->base_path);
+			else
+				path = g_path_get_dirname(doc_list[idx].file_name);
+
+			filename = g_strconcat(path, G_DIR_SEPARATOR_S, filename, NULL);
+			g_free(tmp);
+			g_free(path);
+		}
+
+		locale_filename = utils_get_locale_from_utf8(filename);
+		document_open_file(-1, locale_filename, 0, FALSE, NULL, NULL);
+
+		g_free(filename);
+		g_free(locale_filename);
+	}
+}
+
+
+void
+on_remove_markers1_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	gint idx = document_get_cur_idx();
+
+	if (! DOC_IDX_VALID(idx))
+		return;
+
+	sci_marker_delete_all(doc_list[idx].sci, 0);	// delete the yellow tag marker
+	sci_marker_delete_all(doc_list[idx].sci, 1);	// delete user markers
+}
+
+
+void
+on_load_tags1_activate                 (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	symbols_show_load_tags_dialog();
+}
+
+
+void
+on_context_action1_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	gint idx;
+	gchar *word, *command;
+	GError *error = NULL;
+
+	idx = document_get_cur_idx();
+	if (! DOC_IDX_VALID(idx)) return;
+
+	if (sci_can_copy(doc_list[idx].sci))
+	{	// take selected text if there is a selection
+		word = g_malloc(sci_get_selected_text_length(doc_list[idx].sci) + 1);
+		sci_get_selected_text(doc_list[idx].sci, word);
+	}
+	else
+	{
+		word = g_strdup(editor_info.current_word);
+	}
+
+	// use the filetype specific command if available, fallback to global command otherwise
+	if (doc_list[idx].file_type != NULL &&
+		doc_list[idx].file_type->context_action_cmd != NULL &&
+		*doc_list[idx].file_type->context_action_cmd != '\0')
+	{
+		command = g_strdup(doc_list[idx].file_type->context_action_cmd);
+	}
+	else
+	{
+		command = g_strdup(app->context_action_cmd);
+	}
+
+	// substitute the wildcard %s and run the command if it is non empty
+	if (command != NULL && *command != '\0')
+	{
+		command = utils_str_replace(command, "%s", word);
+
+		if (! g_spawn_command_line_async(command, &error))
+		{
+			msgwin_status_add("Context action command failed: %s", error->message);
+			g_error_free(error);
+		}
+	}
+	g_free(word);
+	g_free(command);
+}
+
+
+void
+on_menu_toggle_all_additional_widgets1_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	static gint hide_all = -1;
+	GtkCheckMenuItem *msgw = GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_show_messages_window1"));
+	GtkCheckMenuItem *toolbari = GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_show_toolbar1"));
+
+	// get the initial state (necessary if Geany was closed with hide_all = TRUE)
+	if (hide_all == -1)
+	{
+		if (! gtk_check_menu_item_get_active(msgw) &&
+			! app->show_notebook_tabs &&
+			! gtk_check_menu_item_get_active(toolbari))
+		{
+			hide_all = TRUE;
+		}
+		else
+			hide_all = FALSE;
+	}
+
+	hide_all = ! hide_all; // toggle
+
+	if (hide_all)
+	{
+		if (gtk_check_menu_item_get_active(msgw))
+			gtk_check_menu_item_set_active(msgw, ! gtk_check_menu_item_get_active(msgw));
+
+		app->show_notebook_tabs = FALSE;
+		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(app->notebook), app->show_notebook_tabs);
+
+		ui_statusbar_showhide(FALSE);
+
+		if (gtk_check_menu_item_get_active(toolbari))
+			gtk_check_menu_item_set_active(toolbari, ! gtk_check_menu_item_get_active(toolbari));
+	}
+	else
+	{
+
+		if (! gtk_check_menu_item_get_active(msgw))
+			gtk_check_menu_item_set_active(msgw, ! gtk_check_menu_item_get_active(msgw));
+
+		app->show_notebook_tabs = TRUE;
+		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(app->notebook), app->show_notebook_tabs);
+
+		ui_statusbar_showhide(TRUE);
+
+		if (! gtk_check_menu_item_get_active(toolbari))
+			gtk_check_menu_item_set_active(toolbari, ! gtk_check_menu_item_get_active(toolbari));
+	}
+}
+
