@@ -1,7 +1,8 @@
 /*
  *      callbacks.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,9 +18,17 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: callbacks.h 1112 2006-12-17 19:31:32Z eht16 $
+ * $Id: callbacks.h 1511 2007-05-06 14:49:51Z eht16 $
  */
 
+typedef struct
+{
+	gint last_doc_idx;
+} CallbacksData;
+
+extern CallbacksData callbacks_data;
+
+
 gint
 destroyapp                             (GtkWidget *widget, gpointer gdata);
 
@@ -219,10 +228,6 @@ void
 on_entry1_changed                      (GtkEditable     *editable,
                                         gpointer         user_data);
 
-void
-on_filetype_change                     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_to_lower_case1_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -307,6 +312,12 @@ on_find_next1_activate                 (GtkMenuItem     *menuitem,
 void
 on_find_previous1_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+void
+on_find_nextsel1_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+void
+on_find_prevsel1_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
 
 void
 on_replace1_activate                   (GtkMenuItem     *menuitem,
@@ -316,10 +327,6 @@ void
 on_find_in_files1_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_new_with_template                   (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_toolbutton_new_clicked              (GtkToolButton   *toolbutton,
                                         gpointer         user_data);
@@ -328,16 +335,6 @@ void
 on_go_to_line_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_includes_arguments_dialog_response  (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data);
-
-void
-on_includes_arguments_tex_dialog_response  (GtkDialog *dialog,
-                                            gint response,
-                                            gpointer user_data);
-
 void
 on_goto_line_dialog_response           (GtkDialog *dialog,
                                         gint response,
@@ -562,3 +559,52 @@ on_menu_insert_include_activate        (GtkMenuItem     *menuitem,
 void
 on_menu_insert_date_activate           (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+
+void
+on_menu_comments_bsd_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_comments_bsd_activate               (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_project_new1_activate               (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_project_open1_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_project_close1_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_project_properties1_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_project1_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_open_selected_file1_activate   (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_remove_markers1_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_load_tags1_activate                 (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_context_action1_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_toggle_all_additional_widgets1_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
