@@ -1,7 +1,8 @@
 /*
  *      dialogs.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,11 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: dialogs.c 1316 2007-02-22 12:41:48Z ntrel $
+ * $Id: dialogs.c 1535 2007-05-15 15:16:16Z eht16 $
+ */
+
+/*
+ * File related dialogs, miscellaneous dialogs.
  */
 
 #include "geany.h"
@@ -34,6 +39,7 @@
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
 #endif
+#include <glib/gstdio.h>
 
 #include "dialogs.h"
 
@@ -78,6 +84,7 @@ void dialogs_show_open_file ()
 		gtk_tooltips_set_tip(tooltips, viewbtn,
 			_("Opens the file in read-only mode. If you choose more than one file to open, all files will be opened read-only."), NULL);
 		gtk_widget_show(viewbtn);
+		gtk_widget_set_name(app->open_filesel, "GeanyDialog");
 		gtk_dialog_add_action_widget(GTK_DIALOG(app->open_filesel),
 			viewbtn, GTK_RESPONSE_APPLY);
 		gtk_dialog_add_buttons(GTK_DIALOG(app->open_filesel),
@@ -103,6 +110,9 @@ void dialogs_show_open_file ()
 		// add FileFilters(start with "All Files")
 		gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(app->open_filesel),
 					filetypes_create_file_filter(filetypes[GEANY_FILETYPES_ALL]));
+		// now create meta filter "All Source"
+		gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(app->open_filesel),
+					filetypes_create_file_filter_all_source());
 		for (i = 0; i < GEANY_MAX_FILE_TYPES - 1; i++)
 		{
 			gtk_combo_box_append_text(GTK_COMBO_BOX(filetype_combo), filetypes[i]->title);
@@ -265,6 +275,7 @@ gboolean dialogs_show_save_as()
 		gtk_window_set_skip_taskbar_hint(GTK_WINDOW(app->save_filesel), TRUE);
 		gtk_window_set_type_hint(GTK_WINDOW(app->save_filesel), GDK_WINDOW_TYPE_HINT_DIALOG);
 		gtk_dialog_set_default_response(GTK_DIALOG(app->save_filesel), GTK_RESPONSE_ACCEPT);
+		gtk_widget_set_name(app->save_filesel, "GeanyDialog");
 
 		g_signal_connect((gpointer) app->save_filesel, "delete_event",
 			G_CALLBACK(gtk_widget_hide_on_delete), NULL);
@@ -325,6 +336,7 @@ void dialogs_show_msgbox(gint type, const gchar *text, ...)
 #else
 	dialog = gtk_message_dialog_new(GTK_WINDOW(app->window), GTK_DIALOG_DESTROY_WITH_PARENT,
                                   type, GTK_BUTTONS_OK, "%s", string);
+	gtk_widget_set_name(dialog, "GeanyDialog");
 	gtk_dialog_run(GTK_DIALOG(dialog));
 	gtk_widget_destroy(dialog);
 #endif
@@ -412,6 +424,7 @@ void dialogs_show_open_font()
 		gtk_window_set_destroy_with_parent(GTK_WINDOW(app->open_fontsel), TRUE);
 		gtk_window_set_skip_taskbar_hint(GTK_WINDOW(app->open_fontsel), TRUE);
 		gtk_window_set_type_hint(GTK_WINDOW(app->open_fontsel), GDK_WINDOW_TYPE_HINT_DIALOG);
+		gtk_widget_set_name(app->open_fontsel, "GeanyDialog");
 
 		gtk_widget_show(GTK_FONT_SELECTION_DIALOG(app->open_fontsel)->apply_button);
 
@@ -448,6 +461,7 @@ void dialogs_show_word_count()
 										 GTK_DIALOG_DESTROY_WITH_PARENT,
 										 GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL, NULL);
 	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
+	gtk_widget_set_name(dialog, "GeanyDialog");
 
 	if (sci_can_copy(doc_list[idx].sci))
 	{
@@ -541,6 +555,7 @@ void dialogs_show_color(gchar *colour)
 	if (app->open_colorsel == NULL)
 	{
 		app->open_colorsel = gtk_color_selection_dialog_new(_("Color Chooser"));
+		gtk_widget_set_name(app->open_colorsel, "GeanyDialog");
 		gtk_window_set_transient_for(GTK_WINDOW(app->open_colorsel), GTK_WINDOW(app->window));
 
 		g_signal_connect(GTK_COLOR_SELECTION_DIALOG(app->open_colorsel)->cancel_button, "clicked",
@@ -580,6 +595,7 @@ void dialogs_show_input(const gchar *title, const gchar *label_text, const gchar
 						GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 						GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
 	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
+	gtk_widget_set_name(dialog, "GeanyDialog");
 	gtk_box_set_spacing(GTK_BOX(vbox), 6);
 
 	label = gtk_label_new(label_text);
@@ -612,6 +628,7 @@ void dialogs_show_goto_line()
 										GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 										GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
 	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
+	gtk_widget_set_name(dialog, "GeanyDialog");
 
 	label = gtk_label_new(_("Enter the line you want to go to:"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
@@ -628,236 +645,6 @@ void dialogs_show_goto_line()
 }
 
 
-void dialogs_show_includes_arguments_tex()
-{
-	GtkWidget *dialog, *label, *entries[4], *vbox, *table;
-	gint idx = document_get_cur_idx();
-	gint response;
-	filetype *ft = NULL;
-
-	if (DOC_IDX_VALID(idx)) ft = doc_list[idx].file_type;
-	g_return_if_fail(ft != NULL);
-
-	dialog = gtk_dialog_new_with_buttons(_("Set Arguments"), GTK_WINDOW(app->window),
-										GTK_DIALOG_DESTROY_WITH_PARENT,
-										GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-										GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
-	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
-
-	label = gtk_label_new(_("Set programs and options for compiling and viewing (La)TeX files."));
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_container_add(GTK_CONTAINER(vbox), label);
-
-	table = gtk_table_new(4, 2, FALSE);
-	gtk_table_set_row_spacings(GTK_TABLE(table), 6);
-	gtk_container_add(GTK_CONTAINER(vbox), table);
-
-	// LaTeX -> DVI args
-	if (ft->programs->compiler != NULL)
-	{
-		label = gtk_label_new(_("DVI creation:"));
-		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-		gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
-			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
-
-		entries[0] = gtk_entry_new();
-		gtk_entry_set_width_chars(GTK_ENTRY(entries[0]), 30);
-		if (ft->programs->compiler)
-		{
-			gtk_entry_set_text(GTK_ENTRY(entries[0]), ft->programs->compiler);
-		}
-		gtk_table_attach_defaults(GTK_TABLE(table), entries[0], 1, 2, 0, 1);
-		g_object_set_data_full(G_OBJECT(dialog), "tex_entry1",
-					gtk_widget_ref(entries[0]), (GDestroyNotify)gtk_widget_unref);
-	}
-
-	// LaTeX -> PDF args
-	if (ft->programs->linker != NULL)
-	{
-		label = gtk_label_new(_("PDF creation:"));
-		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-		gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
-			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
-
-		entries[1] = gtk_entry_new();
-		gtk_entry_set_width_chars(GTK_ENTRY(entries[1]), 30);
-		if (ft->programs->linker)
-		{
-			gtk_entry_set_text(GTK_ENTRY(entries[1]), ft->programs->linker);
-		}
-		gtk_table_attach_defaults(GTK_TABLE(table), entries[1], 1, 2, 1, 2);
-		g_object_set_data_full(G_OBJECT(dialog), "tex_entry2",
-					gtk_widget_ref(entries[1]), (GDestroyNotify)gtk_widget_unref);
-	}
-
-	// View LaTeX -> DVI args
-	if (ft->programs->run_cmd != NULL)
-	{
-		label = gtk_label_new(_("DVI preview:"));
-		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-		gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
-			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
-
-		entries[2] = gtk_entry_new();
-		gtk_entry_set_width_chars(GTK_ENTRY(entries[2]), 30);
-		if (ft->programs->run_cmd)
-		{
-			gtk_entry_set_text(GTK_ENTRY(entries[2]), ft->programs->run_cmd);
-		}
-		gtk_table_attach_defaults(GTK_TABLE(table), entries[2], 1, 2, 2, 3);
-		g_object_set_data_full(G_OBJECT(dialog), "tex_entry3",
-					gtk_widget_ref(entries[2]), (GDestroyNotify)gtk_widget_unref);
-	}
-
-	// View LaTeX -> PDF args
-	if (ft->programs->run_cmd2 != NULL)
-	{
-		label = gtk_label_new(_("PDF preview:"));
-		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-		gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
-			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
-
-		entries[3] = gtk_entry_new();
-		gtk_entry_set_width_chars(GTK_ENTRY(entries[3]), 30);
-		if (ft->programs->run_cmd2)
-		{
-			gtk_entry_set_text(GTK_ENTRY(entries[3]), ft->programs->run_cmd2);
-		}
-		gtk_table_attach_defaults(GTK_TABLE(table), entries[3], 1, 2, 3, 4);
-		g_object_set_data_full(G_OBJECT(dialog), "tex_entry4",
-					gtk_widget_ref(entries[3]), (GDestroyNotify)gtk_widget_unref);
-	}
-
-	label = gtk_label_new(_("%f will be replaced by the current filename, e.g. test_file.c\n"
-							"%e will be replaced by the filename without extension, e.g. test_file"));
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_container_add(GTK_CONTAINER(vbox), label);
-
-	gtk_widget_show_all(dialog);
-	// run modally to prevent user changing idx filetype
-	response = gtk_dialog_run(GTK_DIALOG(dialog));
-	// call the callback manually
-	on_includes_arguments_tex_dialog_response(GTK_DIALOG(dialog), response, ft);
-
-	gtk_widget_destroy(dialog);
-}
-
-
-void dialogs_show_includes_arguments_gen()
-{
-	GtkWidget *dialog, *label, *entries[3], *vbox;
-	GtkWidget *ft_table = NULL;
-	gint row = 0;
-	gint idx = document_get_cur_idx();
-	gint response;
-	filetype *ft = NULL;
-
-	if (DOC_IDX_VALID(idx)) ft = doc_list[idx].file_type;
-	g_return_if_fail(ft != NULL);
-
-	dialog = gtk_dialog_new_with_buttons(_("Set Includes and Arguments"), GTK_WINDOW(app->window),
-										GTK_DIALOG_DESTROY_WITH_PARENT,
-										GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-										GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
-	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
-
-	label = gtk_label_new(_("Set the commands for building and running programs."));
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_container_add(GTK_CONTAINER(vbox), label);
-
-	if (ft->actions->can_compile || ft->actions->can_link || ft->actions->can_exec)
-	{
-		GtkWidget *align, *frame;
-		gchar *frame_title = g_strconcat(ft->title, _(" commands"), NULL);
-
-		frame = ui_frame_new_with_alignment(frame_title, &align);
-		gtk_container_add(GTK_CONTAINER(vbox), frame);
-		g_free(frame_title);
-
-		ft_table = gtk_table_new(3, 2, FALSE);
-		gtk_table_set_row_spacings(GTK_TABLE(ft_table), 6);
-		gtk_container_add(GTK_CONTAINER(align), ft_table);
-		row = 0;
-	}
-
-	// include-args
-	if (ft->actions->can_compile)
-	{
-		label = gtk_label_new(_("Compile:"));
-		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-		gtk_table_attach(GTK_TABLE(ft_table), label, 0, 1, row, row + 1,
-			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
-
-		entries[0] = gtk_entry_new();
-		gtk_entry_set_width_chars(GTK_ENTRY(entries[0]), 30);
-		if (ft->programs->compiler)
-		{
-			gtk_entry_set_text(GTK_ENTRY(entries[0]), ft->programs->compiler);
-		}
-		gtk_table_attach_defaults(GTK_TABLE(ft_table), entries[0], 1, 2, row, row + 1);
-		row++;
-
-		g_object_set_data_full(G_OBJECT(dialog), "includes_entry1",
-					gtk_widget_ref(entries[0]), (GDestroyNotify)gtk_widget_unref);
-	}
-
-	// lib-args
-	if (ft->actions->can_link)
-	{
-		label = gtk_label_new(_("Build:"));
-		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-		gtk_table_attach(GTK_TABLE(ft_table), label, 0, 1, row, row + 1,
-			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
-
-		entries[1] = gtk_entry_new();
-		gtk_entry_set_width_chars(GTK_ENTRY(entries[1]), 30);
-		if (ft->programs->linker)
-		{
-			gtk_entry_set_text(GTK_ENTRY(entries[1]), ft->programs->linker);
-		}
-		gtk_table_attach_defaults(GTK_TABLE(ft_table), entries[1], 1, 2, row, row + 1);
-		row++;
-
-		g_object_set_data_full(G_OBJECT(dialog), "includes_entry2",
-					gtk_widget_ref(entries[1]), (GDestroyNotify)gtk_widget_unref);
-	}
-
-	// program-args
-	if (ft->actions->can_exec)
-	{
-		label = gtk_label_new(_("Execute:"));
-		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-		gtk_table_attach(GTK_TABLE(ft_table), label, 0, 1, row, row + 1,
-			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
-
-		entries[2] = gtk_entry_new();
-		gtk_entry_set_width_chars(GTK_ENTRY(entries[2]), 30);
-		if (ft->programs->run_cmd)
-		{
-			gtk_entry_set_text(GTK_ENTRY(entries[2]), ft->programs->run_cmd);
-		}
-		gtk_table_attach_defaults(GTK_TABLE(ft_table), entries[2], 1, 2, row, row + 1);
-		row++;
-
-		g_object_set_data_full(G_OBJECT(dialog), "includes_entry3",
-						gtk_widget_ref(entries[2]), (GDestroyNotify)gtk_widget_unref);
-	}
-
-	label = gtk_label_new(_("%f will be replaced by the current filename, e.g. test_file.c\n"
-							"%e will be replaced by the filename without extension, e.g. test_file"));
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_container_add(GTK_CONTAINER(vbox), label);
-
-	gtk_widget_show_all(dialog);
-	// run modally to prevent user changing idx filetype
-	response = gtk_dialog_run(GTK_DIALOG(dialog));
-	// call the callback manually
-	on_includes_arguments_dialog_response(GTK_DIALOG(dialog), response, ft);
-
-	gtk_widget_destroy(dialog);
-}
-
-
 void dialogs_show_file_properties(gint idx)
 {
 	GtkWidget *dialog, *label, *table, *hbox, *image, *perm_table, *check, *vbox;
@@ -896,8 +683,13 @@ void dialogs_show_file_properties(gint idx)
 
 
 #if defined(HAVE_SYS_STAT_H) && defined(TIME_WITH_SYS_TIME) && defined(HAVE_SYS_TYPES_H)
+#ifdef G_OS_WIN32
+	// don't try to convert the filename on Windows, it should be already in UTF8
+	locale_filename = g_strdup(doc_list[idx].file_name);
+#else
 	locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
-	if (stat(locale_filename, &st) == 0)
+#endif
+	if (g_stat(locale_filename, &st) == 0)
 	{
 		// first copy the returned string and the trim it, to not modify the static glibc string
 		// g_strchomp() is used to remove trailing EOL chars, which are there for whatever reason
@@ -928,6 +720,7 @@ void dialogs_show_file_properties(gint idx)
 										 GTK_DIALOG_DESTROY_WITH_PARENT,
 										 GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL, NULL);
 	g_free(title);
+	gtk_widget_set_name(dialog, "GeanyDialog");
 	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
 
 	g_signal_connect(dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);
@@ -1239,12 +1032,13 @@ show_question(const gchar *yes_btn, const gchar *no_btn, const gchar *question_t
 	dialog = gtk_message_dialog_new(GTK_WINDOW(app->window),
 		GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION,
 		GTK_BUTTONS_NONE, "%s", question_text);
+	gtk_widget_set_name(dialog, "GeanyDialog");
 	// question_text will be in bold if optional extra_text used
 	if (extra_text != NULL)
 		gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
 			"%s", extra_text);
 
-	// For a cancel button, use cancel reponse so user can press escape to cancel
+	// For a cancel button, use cancel response so user can press escape to cancel
 	gtk_dialog_add_button(GTK_DIALOG(dialog), no_btn,
 		utils_str_equal(no_btn, GTK_STOCK_CANCEL) ? GTK_RESPONSE_CANCEL : GTK_RESPONSE_NO);
 	gtk_dialog_add_button(GTK_DIALOG(dialog), yes_btn, GTK_RESPONSE_YES);
@@ -1289,62 +1083,3 @@ gboolean dialogs_show_question_full(const gchar *yes_btn, const gchar *no_btn,
 }
 
 
-void dialogs_show_keyboard_shortcuts()
-{
-	GtkWidget *dialog, *hbox, *label1, *label2, *label3, *swin, *vbox;
-	GString *text_names = g_string_sized_new(600);
-	GString *text_keys = g_string_sized_new(600);
-	gchar *shortcut;
-	guint i;
-	gint height;
-
-	dialog = gtk_dialog_new_with_buttons(_("Keyboard shortcuts"), GTK_WINDOW(app->window),
-				GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL, NULL);
-	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
-	gtk_box_set_spacing(GTK_BOX(vbox), 6);
-
-	height = GEANY_WINDOW_MINIMAL_HEIGHT;
-	gtk_window_set_default_size(GTK_WINDOW(dialog), height * 0.8, height);
-	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_CANCEL);
-
-	label3 = gtk_label_new(_("The following keyboard shortcuts are defined:"));
-	gtk_misc_set_alignment(GTK_MISC(label3), 0, 0.5);
-
-	hbox = gtk_hbox_new(FALSE, 6);
-
-	label1 = gtk_label_new(NULL);
-
-	label2 = gtk_label_new(NULL);
-
-	for (i = 0; i < GEANY_MAX_KEYS; i++)
-	{
-		shortcut = gtk_accelerator_get_label(keys[i]->key, keys[i]->mods);
-		g_string_append(text_names, keys[i]->label);
-		g_string_append(text_names, "\n");
-		g_string_append(text_keys, shortcut);
-		g_string_append(text_keys, "\n");
-		g_free(shortcut);
-	}
-
-	gtk_label_set_text(GTK_LABEL(label1), text_names->str);
-	gtk_label_set_text(GTK_LABEL(label2), text_keys->str);
-
-	gtk_container_add(GTK_CONTAINER(hbox), label1);
-	gtk_container_add(GTK_CONTAINER(hbox), label2);
-
-	swin = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin), GTK_POLICY_AUTOMATIC,
-		GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swin), hbox);
-
-	gtk_box_pack_start(GTK_BOX(vbox), label3, FALSE, FALSE, 6);
-	gtk_box_pack_start(GTK_BOX(vbox), swin, TRUE, TRUE, 0);
-
-	g_signal_connect((gpointer) dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);
-
-	gtk_widget_show_all(dialog);
-
-	g_string_free(text_names, TRUE);
-	g_string_free(text_keys, TRUE);
-}
-
