@@ -1,7 +1,8 @@
 /*
  *      dialogs.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: dialogs.h 807 2006-09-09 14:36:15Z ntrel $
+ * $Id: dialogs.h 1472 2007-04-23 12:30:27Z ntrel $
  */
 
 
@@ -28,11 +29,7 @@
 void dialogs_show_open_file();
 
 /* This shows the file selection dialog to save a file. */
-void dialogs_show_save_as();
-
-void dialogs_show_info(const gchar *text, ...);
-
-void dialogs_show_error(const gchar *text, ...);
+gboolean dialogs_show_save_as();
 
 gboolean dialogs_show_unsaved_file(gint idx);
 
@@ -48,18 +45,14 @@ void dialogs_show_input(const gchar *title, const gchar *label_text, const gchar
 
 void dialogs_show_goto_line();
 
-void dialogs_show_includes_arguments_gen();
-
-void dialogs_show_includes_arguments_tex();
-
 void dialogs_show_file_properties(gint idx);
 
-gboolean dialogs_show_question(const gchar *text, ...);
+gboolean dialogs_show_question(const gchar *text, ...) G_GNUC_PRINTF (1, 2);
 
 /* extra_text can be NULL; otherwise it is displayed below main_text. */
 gboolean dialogs_show_question_full(const gchar *yes_btn, const gchar *no_btn,
-	const gchar *extra_text, const gchar *main_text, ...);
+	const gchar *extra_text, const gchar *main_text, ...) G_GNUC_PRINTF (4, 5);
 
-void dialogs_show_keyboard_shortcuts();
+void dialogs_show_msgbox(gint type, const gchar *text, ...) G_GNUC_PRINTF (2, 3);
 
 #endif
