@@ -18,7 +18,12 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: document.c 1316 2007-02-22 12:41:48Z ntrel $
+ * $Id: document.c 1531 2007-05-14 13:06:18Z ntrel $
+ */
+
+/*
+ * Document related actions: new, save, open, etc.
+ * Also Scintilla search actions.
  */
 
 #include "geany.h"
@@ -43,6 +48,8 @@
 #include <ctype.h>
 #include <stdlib.h>
 
+#include <glib/gstdio.h>
+
 #include "document.h"
 #include "support.h"
 #include "sciwrappers.h"
@@ -64,15 +71,15 @@
 /* dynamic array of document elements to hold all information of the notebook tabs */
 GArray *doc_array;
 
+/* Whether to colourise the document straight after styling settings are changed.
+ * (e.g. when filetype is set or typenames are updated) */
+static gboolean delay_colourise = FALSE;
 
-/* Returns -1 if no text found or the new range endpoint after replacing. */
-static gint
-document_replace_range(gint idx, const gchar *find_text, const gchar *replace_text,
-	gint flags, gint start, gint end, gboolean escaped_chars);
 
 static void document_undo_clear(gint idx);
 static void document_redo_add(gint idx, guint type, gpointer data);
 
+static gboolean update_type_keywords(ScintillaObject *sci);
 
 
 /* returns the document index which has the given filename.
@@ -184,11 +191,17 @@ void document_set_text_changed(gint idx)
 
 
 // Apply just the prefs that can change in the Preferences dialog
-void document_apply_update_prefs(ScintillaObject *sci)
+void document_apply_update_prefs(gint idx)
 {
+	ScintillaObject *sci = doc_list[idx].sci;
 	sci_set_mark_long_lines(sci, app->long_line_type, app->long_line_column, app->long_line_color);
 
 	sci_set_tab_width(sci, app->pref_editor_tab_width);
+
+	sci_set_use_tabs(sci, app->pref_editor_use_tabs);
+	// remove indent spaces on backspace, if using spaces to indent
+	SSM(sci, SCI_SETBACKSPACEUNINDENTS, ! app->pref_editor_use_tabs, 0);
+
 	sci_set_autoc_max_height(sci, app->autocompletion_max_height);
 
 	sci_set_indentionguides(sci, app->pref_editor_show_indent_guide);
@@ -196,6 +209,8 @@ void document_apply_update_prefs(ScintillaObject *sci)
 	sci_set_visible_eols(sci, app->pref_editor_show_line_endings);
 
 	sci_set_folding_margin_visible(sci, app->pref_editor_folding);
+
+	doc_list[idx].use_auto_indention = (app->pref_editor_indention_mode != INDENT_NONE);
 }
 
 
@@ -205,7 +220,7 @@ static void init_doc_struct(document *new_doc)
 {
 	new_doc->is_valid = FALSE;
 	new_doc->has_tags = FALSE;
-	new_doc->use_auto_indention = app->pref_editor_use_auto_indention;
+	new_doc->use_auto_indention = (app->pref_editor_indention_mode != INDENT_NONE);
 	new_doc->line_breaking = app->pref_editor_line_breaking;
 	new_doc->readonly = FALSE;
 	new_doc->tag_store = NULL;
@@ -284,17 +299,18 @@ static gint document_create_new_sci(const gchar *filename)
 	// disable select all to be able to redefine it
 	sci_clear_cmdkey(sci, 'A' | (SCMOD_CTRL << 16));
 
-	document_apply_update_prefs(sci);
+	document_apply_update_prefs(new_idx);
 
 	sci_set_tab_indents(sci, app->use_tab_to_indent);
 	sci_set_symbol_margin(sci, app->show_markers_margin);
 	sci_set_line_numbers(sci, app->show_linenumber_margin, 0);
 	sci_set_lines_wrapped(sci, app->pref_editor_line_breaking);
+	sci_set_scrollbar_mode(sci, app->pref_editor_show_scrollbars);
 
 	// signal for insert-key(works without too, but to update the right status bar)
-/*	g_signal_connect((GtkWidget*) sci, "key-press-event",
-					G_CALLBACK(keybindings_got_event), GINT_TO_POINTER(new_idx));
-*/	// signal for the popup menu
+	//g_signal_connect((GtkWidget*) sci, "key-press-event",
+					//G_CALLBACK(keybindings_got_event), GINT_TO_POINTER(new_idx));
+	// signal for the popup menu
 	g_signal_connect((GtkWidget*) sci, "button-press-event",
 					G_CALLBACK(on_editor_button_press_event), GINT_TO_POINTER(new_idx));
 
@@ -319,7 +335,7 @@ static gint document_create_new_sci(const gchar *filename)
 	this->last_check = time(NULL);
 	this->readonly = FALSE;
 	this->line_breaking = app->pref_editor_line_breaking;
-	this->use_auto_indention = app->pref_editor_use_auto_indention;
+	this->use_auto_indention = (app->pref_editor_indention_mode != INDENT_NONE);
 	this->has_tags = FALSE;
 
 	treeviews_openfiles_add(new_idx);	// sets this->iter
@@ -327,6 +343,14 @@ static gint document_create_new_sci(const gchar *filename)
 	tabnum = notebook_new_tab(new_idx);
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(app->notebook), tabnum);
 
+	// select document in sidebar
+	{
+		GtkTreeSelection *sel;
+
+		sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv.tree_openfiles));
+		gtk_tree_selection_select_iter(sel, &this->iter);
+	}
+
 	ui_close_buttons_toggle();
 
 	this->is_valid = TRUE;	// do this last to prevent UI updating with NULL items.
@@ -353,6 +377,7 @@ gboolean document_remove(guint page_num)
 		g_free(doc_list[idx].saved_encoding.encoding);
 		g_free(doc_list[idx].file_name);
 		tm_workspace_remove_object(doc_list[idx].tm_file, TRUE);
+
 		doc_list[idx].is_valid = FALSE;
 		doc_list[idx].sci = NULL;
 		doc_list[idx].file_name = NULL;
@@ -364,7 +389,7 @@ gboolean document_remove(guint page_num)
 		document_undo_clear(idx);
 		if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)) == 0)
 		{
-			ui_update_tag_list(-1, FALSE);
+			treeviews_update_tag_list(-1, FALSE);
 			//on_notebook1_switch_page(GTK_NOTEBOOK(app->notebook), NULL, 0, NULL);
 			ui_set_window_title(-1);
 			ui_save_buttons_toggle(FALSE);
@@ -388,10 +413,12 @@ static void store_saved_encoding(gint idx)
 
 
 /* This creates a new document, by clearing the text widget and setting the
-   current filename to NULL. */
-void document_new_file(filetype *ft)
+   current filename to filename or NULL. If ft is NULL and filename is not NULL, then the filetype
+   will be guessed from the given filename.
+   filename is expected in UTF-8 encoding. */
+gint document_new_file(const gchar *filename, filetype *ft)
 {
-	gint idx = document_create_new_sci(NULL);
+	gint idx = document_create_new_sci(filename);
 	gchar *template = templates_get_template_new_file(ft);
 
 	g_assert(idx != -1);
@@ -414,6 +441,9 @@ void document_new_file(filetype *ft)
 	store_saved_encoding(idx);
 
 	//document_set_filetype(idx, (ft == NULL) ? filetypes[GEANY_FILETYPES_ALL] : ft);
+	if (ft == NULL && filename != NULL) // guess the filetype from the filename if one is given
+		ft = filetypes_detect_from_file(idx);
+
 	document_set_filetype(idx, ft);	// also clears taglist
 	if (ft == NULL) filetypes[GEANY_FILETYPES_ALL]->style_func_ptr(doc_list[idx].sci);
 	ui_set_window_title(idx);
@@ -424,7 +454,7 @@ void document_new_file(filetype *ft)
 
 	document_update_tag_list(idx, FALSE);
 	document_set_text_changed(idx);
-	ui_document_show_hide(idx); //update the document menu
+	ui_document_show_hide(idx); // update the document menu
 
 	sci_set_line_numbers(doc_list[idx].sci, app->show_linenumber_margin, 0);
 	sci_goto_pos(doc_list[idx].sci, 0, TRUE);
@@ -433,7 +463,10 @@ void document_new_file(filetype *ft)
 	g_signal_connect((GtkWidget*) doc_list[idx].sci, "sci-notify",
 				G_CALLBACK(on_editor_notification), GINT_TO_POINTER(idx));
 
-	msgwin_status_add(_("New file opened."));
+	msgwin_status_add(_("New file \"%s\" opened."),
+		(doc_list[idx].file_name != NULL) ? doc_list[idx].file_name : GEANY_STRING_UNTITLED);
+
+	return idx;
 }
 
 
@@ -581,7 +614,7 @@ static gboolean load_text_file(const gchar *locale_filename, const gchar *utf8_f
 	filedata->bom = FALSE;
 	filedata->readonly = FALSE;
 
-	if (stat(locale_filename, &st) != 0)
+	if (g_stat(locale_filename, &st) != 0)
 	{
 		msgwin_status_add(_("Could not open file %s (%s)"), utf8_filename, g_strerror(errno));
 		return FALSE;
@@ -589,11 +622,7 @@ static gboolean load_text_file(const gchar *locale_filename, const gchar *utf8_f
 
 	filedata->mtime = st.st_mtime;
 
-#ifdef G_OS_WIN32
-	if (! g_file_get_contents(utf8_filename, &filedata->data, NULL, &err))
-#else
 	if (! g_file_get_contents(locale_filename, &filedata->data, NULL, &err))
-#endif
 	{
 		msgwin_status_add(err->message);
 		g_error_free(err);
@@ -653,7 +682,8 @@ static gboolean load_text_file(const gchar *locale_filename, const gchar *utf8_f
 
 
 /* Sets the cursor position on opening a file. First it sets the line when cl_options.goto_line
- * is set, otherwise it sets the line when pos is greater than zero. */
+ * is set, otherwise it sets the line when pos is greater than zero and finally it sets the column
+ * if cl_options.goto_column is set. */
 static void set_cursor_position(gint idx, gint pos)
 {
 	if (cl_options.goto_line >= 0)
@@ -667,17 +697,24 @@ static void set_cursor_position(gint idx, gint pos)
 		sci_set_current_position(doc_list[idx].sci, pos, FALSE);
 		doc_list[idx].scroll_percent = 0.5F;
 	}
+
+	if (cl_options.goto_column >= 0)
+	{	// goto column which was specified on command line and then undefine the column
+		gint cur_pos = sci_get_current_position(doc_list[idx].sci);
+		sci_set_current_position(doc_list[idx].sci, cur_pos + cl_options.goto_column, FALSE);
+		doc_list[idx].scroll_percent = 0.5F;
+		cl_options.goto_column = -1;
+	}
 }
 
 
-/* If idx is set to -1, it creates a new tab, opens the file from filename and
- * set the cursor to pos.
- * If idx is greater than -1, it reloads the file in the tab corresponding to
- * idx and set the cursor to position 0. In this case, filename should be NULL
- * It returns the idx of the opened file or -1 if an error occurred.
- */
-int document_open_file(gint idx, const gchar *filename, gint pos, gboolean readonly, filetype *ft,
-					   const gchar *forced_enc)
+/* To open a new file, set idx to -1; filename should be locale encoded.
+ * To reload a file, set the idx for the document to be reloaded; filename should be NULL.
+ * Returns: idx of the opened file or -1 if an error occurred.
+ * Note: If opening more than one file, document_delay_colourise() should be used before
+ * and document_colourise_new() after opening to avoid unnecessary recolourising. */
+gint document_open_file(gint idx, const gchar *filename, gint pos, gboolean readonly,
+		filetype *ft, const gchar *forced_enc)
 {
 	gint editor_mode;
 	gboolean reload = (idx == -1) ? FALSE : TRUE;
@@ -706,7 +743,11 @@ int document_open_file(gint idx, const gchar *filename, gint pos, gboolean reado
 
 		// try to get the UTF-8 equivalent for the filename, fallback to filename if error
 		locale_filename = g_strdup(filename);
+#ifdef G_OS_WIN32 // on Win32 we only use locale_filename because it is already UTF8. I hope
+		utf8_filename = g_strdup(locale_filename);
+#else
 		utf8_filename = utils_get_utf8_from_locale(locale_filename);
+#endif
 
 		// if file is already open, switch to it and go
 		idx = document_find_by_filename(utf8_filename, FALSE);
@@ -738,6 +779,7 @@ int document_open_file(gint idx, const gchar *filename, gint pos, gboolean reado
 	sci_empty_undo_buffer(doc_list[idx].sci);
 
 	// add the text to the ScintillaObject
+	sci_set_readonly(doc_list[idx].sci, FALSE);	// to allow replacing text
 	sci_set_text(doc_list[idx].sci, filedata.data);	// NULL terminated data
 
 	// detect & set line endings
@@ -769,7 +811,7 @@ int document_open_file(gint idx, const gchar *filename, gint pos, gboolean reado
 		g_signal_connect((GtkWidget*) doc_list[idx].sci, "sci-notify",
 					G_CALLBACK(on_editor_notification), GINT_TO_POINTER(idx));
 
-		use_ft = (ft != NULL) ? ft : filetypes_get_from_filename(idx);
+		use_ft = (ft != NULL) ? ft : filetypes_detect_from_file(idx);
 	}
 	else
 	{	// reloading
@@ -823,6 +865,8 @@ void document_open_file_list(const gchar *data, gssize length)
 		default: list = g_strsplit(data, "\n", 0);
 	}
 
+	document_delay_colourise();
+
 	for (i = 0; ; i++)
 	{
 		if (list[i] == NULL) break;
@@ -831,11 +875,29 @@ void document_open_file_list(const gchar *data, gssize length)
 		document_open_file(-1, filename, 0, FALSE, NULL, NULL);
 		g_free(filename);
 	}
+	document_colourise_new();
 
 	g_strfreev(list);
 }
 
 
+/* Takes a linked list of filename URIs and opens each file, ensuring the newly opened
+ * documents and existing documents (if necessary) are only colourised once. */
+void document_open_files(const GSList *filenames, gboolean readonly, filetype *ft,
+		const gchar *forced_enc)
+{
+	const GSList *item;
+
+	document_delay_colourise();
+
+	for (item = filenames; item != NULL; item = g_slist_next(item))
+	{
+		document_open_file(-1, item->data, 0, readonly, ft, forced_enc);
+	}
+	document_colourise_new();
+}
+
+
 gint document_reload_file(gint idx, const gchar *forced_enc)
 {
 	gint pos = 0;
@@ -857,9 +919,13 @@ static gboolean document_update_timestamp(gint idx)
 
 	g_return_val_if_fail(DOC_IDX_VALID(idx), FALSE);
 
+#ifdef G_OS_WIN32
+	// don't try to convert the filename on Windows, it should be already in UTF8
+	locale_filename = g_strdup(doc_list[idx].file_name);
+#else
 	locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
-
-	if (stat(locale_filename, &st) != 0)
+#endif
+	if (g_stat(locale_filename, &st) != 0)
 	{
 		msgwin_status_add(_("Could not open file %s (%s)"), doc_list[idx].file_name,
 			g_strerror(errno));
@@ -905,7 +971,9 @@ gboolean document_save_file(gint idx, gboolean force)
 
 	len = sci_get_length(doc_list[idx].sci) + 1;
 	if (doc_list[idx].has_bom && encodings_is_unicode_charset(doc_list[idx].encoding))
-	{
+	{	// always write a UTF-8 BOM because in this moment the text itself is still in UTF-8
+		// encoding, it will be converted to doc_list[idx].encoding below and this conversion
+		// also changes the BOM
 		data = (gchar*) g_malloc(len + 3);	// 3 chars for BOM
 		data[0] = 0xef;
 		data[1] = 0xbb;
@@ -994,7 +1062,7 @@ gboolean document_save_file(gint idx, gboolean force)
 
 		if (doc_list[idx].file_type == NULL || doc_list[idx].file_type->id == GEANY_FILETYPES_ALL)
 		{
-			doc_list[idx].file_type = filetypes_get_from_filename(idx);
+			doc_list[idx].file_type = filetypes_detect_from_file(idx);
 			filetypes_select_radio_item(doc_list[idx].file_type);
 		}
 		document_set_filetype(idx, doc_list[idx].file_type);
@@ -1113,15 +1181,11 @@ gint document_find_text(gint idx, const gchar *text, gint flags, gboolean search
 		{
 			gint ret;
 
-			sci_goto_pos(doc_list[idx].sci, (search_backwards) ? sci_len : 0, TRUE);
+			sci_set_current_position(doc_list[idx].sci, (search_backwards) ? sci_len : 0, FALSE);
 			ret = document_find_text(idx, text, flags, search_backwards, scroll);
 			if (ret == -1)
 			{	// return to original cursor position if not found
-				gint new_visible_line;
-
-				sci_goto_pos(doc_list[idx].sci, selection_start, FALSE);
-				new_visible_line = sci_get_first_visible_line(doc_list[idx].sci);
-				sci_scroll_lines(doc_list[idx].sci, first_visible_line - new_visible_line);
+				sci_set_current_position(doc_list[idx].sci, selection_start, FALSE);
 			}
 			return ret;
 		}
@@ -1198,33 +1262,39 @@ static void show_replace_summary(gint idx, gint count, const gchar *find_text,
 	{	// escape special characters for showing
 		escaped_find_text = g_strescape(find_text, NULL);
 		escaped_replace_text = g_strescape(replace_text, NULL);
-		msgwin_status_add(_("%s: replaced %d occurrences of \"%s\" with \"%s\"."),
+		msgwin_status_add(_("%s: replaced %d occurrence(s) of \"%s\" with \"%s\"."),
 						filename, count, escaped_find_text, escaped_replace_text);
 		g_free(escaped_find_text);
 		g_free(escaped_replace_text);
 	}
 	else
 	{
-		msgwin_status_add(_("%s: replaced %d occurrences of \"%s\" with \"%s\"."),
+		msgwin_status_add(_("%s: replaced %d occurrence(s) of \"%s\" with \"%s\"."),
 						filename, count, find_text, replace_text);
 	}
 	g_free(filename);
 }
 
 
-/* Returns -1 if no text found or the new range endpoint after replacing. */
-static gint
+/* Replace all text matches in a certain range within document idx.
+ * If not NULL, *new_range_end is set to the new range endpoint after replacing,
+ * or -1 if no text was found.
+ * scroll_to_match is whether to scroll the last replacement in view (which also
+ * clears the selection).
+ * Returns: the number of replacements made. */
+static guint
 document_replace_range(gint idx, const gchar *find_text, const gchar *replace_text,
-	gint flags, gint start, gint end, gboolean escaped_chars)
+	gint flags, gint start, gint end, gboolean scroll_to_match, gint *new_range_end)
 {
 	gint search_pos;
 	gint count = 0;
 	gint find_len = 0, replace_len = 0;
-	gboolean match_found = FALSE;
 	struct TextToFind ttf;
 
-	g_return_val_if_fail(find_text != NULL && replace_text != NULL, FALSE);
-	if (idx == -1 || ! *find_text) return FALSE;
+	if (new_range_end != NULL)
+		*new_range_end = -1;
+	g_return_val_if_fail(find_text != NULL && replace_text != NULL, 0);
+	if (idx == -1 || ! *find_text || doc_list[idx].readonly) return 0;
 
 	sci_start_undo_action(doc_list[idx].sci);
 	ttf.chrg.cpMin = start;
@@ -1234,14 +1304,14 @@ document_replace_range(gint idx, const gchar *find_text, const gchar *replace_te
 	while (TRUE)
 	{
 		search_pos = sci_find_text(doc_list[idx].sci, flags, &ttf);
-		if (search_pos == -1) break;
+		if (search_pos == -1)
+			break;
 		find_len = ttf.chrgText.cpMax - ttf.chrgText.cpMin;
 
 		if (search_pos + find_len > end)
 			break; //found text is partly out of range
 		else
 		{
-			match_found = TRUE;
 			sci_target_start(doc_list[idx].sci, search_pos);
 			sci_target_end(doc_list[idx].sci, search_pos + find_len);
 			replace_len = sci_target_replace(doc_list[idx].sci, replace_text,
@@ -1254,48 +1324,115 @@ document_replace_range(gint idx, const gchar *find_text, const gchar *replace_te
 	}
 	sci_end_undo_action(doc_list[idx].sci);
 
-	show_replace_summary(idx, count, find_text, replace_text, escaped_chars);
+	if (count > 0)
+	{	// scroll last match in view, will destroy the existing selection
+		if (scroll_to_match)
+			sci_goto_pos(doc_list[idx].sci, ttf.chrg.cpMin, TRUE);
 
-	if (match_found)
-	{
-		// scroll last match in view.
-		sci_goto_pos(doc_list[idx].sci, ttf.chrg.cpMin, TRUE);
-		return end;
+		if (new_range_end != NULL)
+			*new_range_end = end;
 	}
-	else
-		return -1; // no text was found
+	return count;
 }
 
 
 void document_replace_sel(gint idx, const gchar *find_text, const gchar *replace_text, gint flags,
 						  gboolean escaped_chars)
 {
-	gint selection_end, selection_start;
+	gint selection_end, selection_start, selection_mode, selected_lines, last_line = 0;
+	gint max_column = 0, count = 0;
+	gboolean replaced = FALSE;
 
 	g_return_if_fail(find_text != NULL && replace_text != NULL);
 	if (idx == -1 || ! *find_text) return;
 
 	selection_start = sci_get_selection_start(doc_list[idx].sci);
 	selection_end = sci_get_selection_end(doc_list[idx].sci);
+	// do we have a selection?
 	if ((selection_end - selection_start) == 0)
 	{
 		utils_beep();
 		return;
 	}
 
-	selection_end = document_replace_range(idx, find_text, replace_text, flags,
-		selection_start, selection_end, escaped_chars);
-	if (selection_end == -1)
+	selection_mode = sci_get_selection_mode(doc_list[idx].sci);
+	selected_lines = sci_get_lines_selected(doc_list[idx].sci);
+	// handle rectangle, multi line selections (it doesn't matter on a single line)
+	if (selection_mode == SC_SEL_RECTANGLE && selected_lines > 1)
 	{
-		// no replacements
+		gint first_line, line;
+
+		sci_start_undo_action(doc_list[idx].sci);
+
+		first_line = sci_get_line_from_position(doc_list[idx].sci, selection_start);
+		// Find the last line with chars selected (not EOL char)
+		last_line = sci_get_line_from_position(doc_list[idx].sci, selection_end - 1);
+		last_line = MAX(first_line, last_line);
+		for (line = first_line; line < (first_line + selected_lines); line++)
+		{
+			gint line_start = sci_get_pos_at_line_sel_start(doc_list[idx].sci, line);
+			gint line_end = sci_get_pos_at_line_sel_end(doc_list[idx].sci, line);
+
+			// skip line if there is no selection
+			if (line_start != INVALID_POSITION)
+			{
+				// don't let document_replace_range() scroll to match to keep our selection
+				gint new_sel_end;
+
+				count += document_replace_range(idx, find_text, replace_text, flags,
+								line_start, line_end, FALSE, &new_sel_end);
+				if (new_sel_end != -1)
+				{
+					replaced = TRUE;
+					// this gets the greatest column within the selection after replacing
+					max_column = MAX(max_column,
+						new_sel_end - sci_get_position_from_line(doc_list[idx].sci, line));
+				}
+			}
+		}
+		sci_end_undo_action(doc_list[idx].sci);
+	}
+	else	// handle normal line selection
+	{
+		count += document_replace_range(idx, find_text, replace_text, flags,
+						selection_start, selection_end, TRUE, &selection_end);
+		if (selection_end != -1)
+			replaced = TRUE;
+	}
+
+	if (replaced)
+	{	// update the selection for the new endpoint
+
+		if (selection_mode == SC_SEL_RECTANGLE && selected_lines > 1)
+		{
+			// now we can scroll to the selection and destroy it because we rebuild it later
+			//sci_goto_pos(doc_list[idx].sci, selection_start, FALSE);
+
+			// Note: the selection will be wrapped to last_line + 1 if max_column is greater than
+			// the highest column on the last line. The wrapped selection is completely different
+			// from the original one, so skip the selection at all
+			/// TODO is there a better way to handle the wrapped selection?
+			if ((sci_get_line_length(doc_list[idx].sci, last_line) - 1) >= max_column)
+			{
+				// for keeping and adjusting the selection in multi line rectangle selection we
+				// need the last line of the original selection and the greatest column number after
+				// replacing and set the selection end to the last line at the greatest column
+				sci_set_selection_start(doc_list[idx].sci, selection_start);
+				sci_set_selection_end(doc_list[idx].sci,
+					sci_get_position_from_line(doc_list[idx].sci, last_line) + max_column);
+				sci_set_selection_mode(doc_list[idx].sci, selection_mode);
+			}
+		}
+		else
+		{
+			sci_set_selection_start(doc_list[idx].sci, selection_start);
+			sci_set_selection_end(doc_list[idx].sci, selection_end);
+		}
+	}
+	else // no replacements
 		utils_beep();
-	}
-	else
-	{
-		// update the selection for the new endpoint
-		sci_set_selection_start(doc_list[idx].sci, selection_start);
-		sci_set_selection_end(doc_list[idx].sci, selection_end);
-	}
+
+	show_replace_summary(idx, count, find_text, replace_text, escaped_chars);
 }
 
 
@@ -1303,17 +1440,20 @@ void document_replace_sel(gint idx, const gchar *find_text, const gchar *replace
 gboolean document_replace_all(gint idx, const gchar *find_text, const gchar *replace_text,
 		gint flags, gboolean escaped_chars)
 {
-	gint len;
+	gint len, count;
 	g_return_val_if_fail(find_text != NULL && replace_text != NULL, FALSE);
 	if (idx == -1 || ! *find_text) return FALSE;
 
 	len = sci_get_length(doc_list[idx].sci);
-	if (document_replace_range(idx, find_text, replace_text, flags, 0, len, escaped_chars) == -1)
+	count = document_replace_range(
+			idx, find_text, replace_text, flags, 0, len, TRUE, NULL);
+
+	if (count == 0)
 	{
 		utils_beep();
-		return FALSE;
 	}
-	return TRUE;
+	show_replace_summary(idx, count, find_text, replace_text, escaped_chars);
+	return (count > 0);
 }
 
 
@@ -1339,7 +1479,7 @@ void document_update_tag_list(gint idx, gboolean update)
 		! doc_list[idx].file_type->has_tags || ! doc_list[idx].file_name)
 	{
 		// set the default (empty) tag list
-		ui_update_tag_list(idx, FALSE);
+		treeviews_update_tag_list(idx, FALSE);
 		return;
 	}
 
@@ -1353,13 +1493,13 @@ void document_update_tag_list(gint idx, gboolean update)
 		tm_workspace_add_object(doc_list[idx].tm_file);
 		if (update)
 			tm_source_file_update(doc_list[idx].tm_file, TRUE, FALSE, TRUE);
-		ui_update_tag_list(idx, TRUE);
+		treeviews_update_tag_list(idx, TRUE);
 	}
 	else
 	{
 		if (tm_source_file_update(doc_list[idx].tm_file, TRUE, FALSE, TRUE))
 		{
-			ui_update_tag_list(idx, TRUE);
+			treeviews_update_tag_list(idx, TRUE);
 		}
 		else
 		{
@@ -1369,8 +1509,12 @@ void document_update_tag_list(gint idx, gboolean update)
 }
 
 
-static GString *get_project_typenames()
+/* Caches the list of project typenames, as a space separated GString.
+ * Returns: TRUE if typenames have changed.
+ * (*types) is set to the list of typenames, or NULL if there are none. */
+static gboolean get_project_typenames(const GString **types)
 {
+	static GString *last_typenames = NULL;
 	GString *s = NULL;
 
 	if (app->tm_workspace)
@@ -1382,40 +1526,72 @@ static GString *get_project_typenames()
 			s = symbols_find_tags_as_string(tags_array, TM_GLOBAL_TYPE_MASK);
 		}
 	}
-	return s;
+
+	if (s && last_typenames && g_string_equal(s, last_typenames))
+	{
+		g_string_free(s, TRUE);
+		*types = last_typenames;
+		return FALSE;	// project typenames haven't changed
+	}
+	// cache typename list for next time
+	if (last_typenames)
+		g_string_free(last_typenames, TRUE);
+	last_typenames = s;
+
+	*types = s;
+	if (s == NULL) return FALSE;
+	return TRUE;
 }
 
 
-/* Returns: whether sci_colourise has been called for sci */
+/* If sci is NULL, update project typenames for all documents that support typenames,
+ * if typenames have changed.
+ * If sci is not NULL, then if sci supports typenames, project typenames are updated
+ * if necessary, and typename keywords are set for sci.
+ * Returns: TRUE if any scintilla type keywords were updated. */
 static gboolean update_type_keywords(ScintillaObject *sci)
 {
 	gboolean ret = FALSE;
+	guint n;
+	const GString *s;
 
-	if (sci_cb_lexer_get_type_keyword_idx(sci_get_lexer(sci)) != -1)
-	{
-		guint n;
-		GString *s = get_project_typenames();
+	if (sci != NULL && sci_cb_lexer_get_type_keyword_idx(sci_get_lexer(sci)) == -1)
+		return FALSE;
 
-		if (s == NULL) return FALSE;
-
-		for (n = 0; n < doc_array->len; n++)
+	if (! get_project_typenames(&s))
+	{	// typenames have not changed
+		if (s != NULL && sci != NULL)
 		{
-			ScintillaObject *wid = doc_list[n].sci;
+			gint keyword_idx = sci_cb_lexer_get_type_keyword_idx(sci_get_lexer(sci));
 
-			if (wid)
+			sci_set_keywords(sci, keyword_idx, s->str);
+			if (! delay_colourise)
 			{
-				gint keyword_idx = sci_cb_lexer_get_type_keyword_idx(sci_get_lexer(wid));
+				sci_colourise(sci, 0, -1);
+			}
+		}
+		return FALSE;
+	}
+	g_return_val_if_fail(s != NULL, FALSE);
 
-				if (keyword_idx > 0)
+	for (n = 0; n < doc_array->len; n++)
+	{
+		ScintillaObject *wid = doc_list[n].sci;
+
+		if (wid)
+		{
+			gint keyword_idx = sci_cb_lexer_get_type_keyword_idx(sci_get_lexer(wid));
+
+			if (keyword_idx > 0)
+			{
+				sci_set_keywords(wid, keyword_idx, s->str);
+				if (! delay_colourise)
 				{
-					sci_set_keywords(wid, keyword_idx, s->str);
 					sci_colourise(wid, 0, -1);
-					if (sci == wid)
-						ret = TRUE;
 				}
+				ret = TRUE;
 			}
 		}
-		g_string_free(s, TRUE);
 	}
 	return ret;
 }
@@ -1425,13 +1601,15 @@ static gboolean update_type_keywords(ScintillaObject *sci)
 void document_set_filetype(gint idx, filetype *type)
 {
 	gboolean colourise = FALSE;
+	gboolean ft_changed;
 
 	if (type == NULL || ! DOC_IDX_VALID(idx))
 		return;
 
 	geany_debug("%s : %s (%s)",	doc_list[idx].file_name, type->name, doc_list[idx].encoding);
 
-	if (doc_list[idx].file_type != type)	// filetype has changed
+	ft_changed = (doc_list[idx].file_type != type);
+	if (ft_changed)	// filetype has changed
 	{
 		doc_list[idx].file_type = type;
 
@@ -1447,9 +1625,18 @@ void document_set_filetype(gint idx, filetype *type)
 	}
 
 	document_update_tag_list(idx, TRUE);
-	colourise &= ! update_type_keywords(doc_list[idx].sci);
-	if (colourise)
-		sci_colourise(doc_list[idx].sci, 0, -1);
+	if (! delay_colourise)
+	{
+		/* Check if project typename keywords have changed.
+		 * If they haven't, we may need to colourise the document. */
+		if (! update_type_keywords(doc_list[idx].sci) && colourise)
+			sci_colourise(doc_list[idx].sci, 0, -1);
+	}
+	if (ft_changed)
+	{
+		utils_get_current_function(-1, NULL);
+		ui_update_statusbar(idx, -1);
+	}
 }
 
 
@@ -1506,12 +1693,17 @@ void document_fold_all(gint idx)
 
 void document_clear_indicators(gint idx)
 {
-	glong last_pos = sci_get_length(doc_list[idx].sci);
+	glong last_pos;
+
+	g_return_if_fail(DOC_IDX_VALID(idx));
+
+	last_pos = sci_get_length(doc_list[idx].sci);
 	if (last_pos > 0)
 	{
 		sci_start_styling(doc_list[idx].sci, 0, INDIC2_MASK);
 		sci_set_styling(doc_list[idx].sci, last_pos, 0);
 	}
+	sci_marker_delete_all(doc_list[idx].sci, 0);	// remove the yellow error line marker
 }
 
 
@@ -1553,7 +1745,8 @@ void document_print(gint idx)
 {
 	gchar *cmdline;
 
-	if (idx == -1 || ! doc_list[idx].is_valid || doc_list[idx].file_name == NULL) return;
+	if (! DOC_IDX_VALID(idx) || doc_list[idx].file_name == NULL)
+		return;
 
 	cmdline = g_strdup(app->tools_print_cmd);
 	cmdline = utils_str_replace(cmdline, "%f", doc_list[idx].file_name);
@@ -1561,19 +1754,29 @@ void document_print(gint idx)
 	if (dialogs_show_question(_("The file \"%s\" will be printed with the following command:\n\n%s"),
 								doc_list[idx].file_name, cmdline))
 	{
-		gint rc;
-		// system() is not the best way, but the only one I found to get the following working:
-		// a2ps -1 --medium=A4 -o - %f | xfprint4
-		rc = system(cmdline);
-		if (rc != 0)
+		GError *error = NULL;
+
+#ifdef G_OS_WIN32
+		gchar *tmp_cmdline = cmdline;
+#else
+		// /bin/sh -c emulates the system() call and makes complex commands possible
+		// but only needed on non-win32 systems due to the lack of win32's shell capabilities
+		gchar *tmp_cmdline = g_strconcat("/bin/sh -c \"", cmdline, "\"", NULL);
+#endif
+
+		if (! g_spawn_command_line_async(tmp_cmdline, &error))
 		{
-			dialogs_show_msgbox(GTK_MESSAGE_ERROR, _("Printing of \"%s\" failed (return code: %d)."),
-								doc_list[idx].file_name, rc);
+			dialogs_show_msgbox(GTK_MESSAGE_ERROR, _("Printing of \"%s\" failed (return code: %s)."),
+								doc_list[idx].file_name, error->message);
+			g_error_free(error);
 		}
 		else
 		{
 			msgwin_status_add(_("File %s printed."), doc_list[idx].file_name);
 		}
+#ifndef G_OS_WIN32
+		g_free(tmp_cmdline);
+#endif
 	}
 	g_free(cmdline);
 }
@@ -1621,7 +1824,7 @@ void document_strip_trailing_spaces(gint idx)
 	for (line = 0; line < max_lines; line++)
 	{
 		gint line_start = sci_get_position_from_line(doc_list[idx].sci, line);
-		gint line_end = sci_get_line_end_from_position(doc_list[idx].sci, line);
+		gint line_end = sci_get_line_end_position(doc_list[idx].sci, line);
 		gint i = line_end - 1;
 		gchar ch = sci_get_char_at(doc_list[idx].sci, i);
 
@@ -1946,3 +2149,74 @@ document *doc(gint idx)
 	return DOC_IDX_VALID(idx) ? &doc_list[idx] : NULL;
 }
 #endif
+
+
+static GArray *doc_indexes = NULL;
+
+/* Cache the current document indexes and prevent any colourising until
+ * document_colourise_new() is called. */
+void document_delay_colourise()
+{
+	gint n;
+
+	g_return_if_fail(delay_colourise == FALSE);
+	g_return_if_fail(doc_indexes == NULL);
+
+	// make an array containing all the current document indexes
+	doc_indexes = g_array_new(FALSE, FALSE, sizeof(gint));
+	for (n = 0; n < (gint) doc_array->len; n++)
+	{
+		if (DOC_IDX_VALID(n))
+			g_array_append_val(doc_indexes, n);
+	}
+	delay_colourise = TRUE;
+}
+
+
+/* Colourise only newly opened documents and existing documents whose project typenames
+ * keywords have changed.
+ * document_delay_colourise() should already have been called. */
+void document_colourise_new()
+{
+	guint n, i;
+	/* A bitset representing which docs need [re]colourising.
+	 * (use gint8 to save memory because gboolean = gint) */
+	gint8 *doc_set = g_newa(gint8, doc_array->len);
+	gboolean recolour = FALSE;	// whether to recolourise existing typenames
+
+	g_return_if_fail(delay_colourise == TRUE);
+	g_return_if_fail(doc_indexes != NULL);
+
+	// first assume recolourising all docs
+	memset(doc_set, TRUE, doc_array->len * sizeof(gint8));
+
+	// remove existing docs from the set if they don't use typenames or typenames haven't changed
+	recolour = update_type_keywords(NULL);
+	for (i = 0; i < doc_indexes->len; i++)
+	{
+		ScintillaObject *sci;
+
+		n = g_array_index(doc_indexes, gint, i);
+		sci = doc_list[n].sci;
+		if (! recolour || (sci && sci_cb_lexer_get_type_keyword_idx(sci_get_lexer(sci)) == -1))
+		{
+			doc_set[n] = FALSE;
+		}
+	}
+	// colourise all in the doc_set
+	for (n = 0; n < doc_array->len; n++)
+	{
+		if (doc_set[n] && doc_list[n].is_valid)
+			sci_colourise(doc_list[n].sci, 0, -1);
+	}
+	delay_colourise = FALSE;
+	g_array_free(doc_indexes, TRUE);
+	doc_indexes = NULL;
+
+	/* now that the current document is colourised, fold points are now accurate,
+	 * so force an update of the current function/tag. */
+	utils_get_current_function(-1, NULL);
+	ui_update_statusbar(-1, -1);
+}
+
+
