@@ -1,7 +1,8 @@
 /*
  *      geany.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: geany.h 1124 2006-12-18 22:57:04Z eht16 $
+ * $Id: geany.h 1483 2007-04-29 10:57:25Z eht16 $
  */
 
 #ifndef GEANY_H
@@ -36,8 +37,10 @@
 // listed in the documentation should not be changed ;-)
 #define GEANY_HOME_DIR					g_get_home_dir()
 #define GEANY_FILEDEFS_SUBDIR			"filedefs"
-#define GEANY_CODENAME					"Sander"
+#define GEANY_TEMPLATES_SUBDIR			"templates"
+#define GEANY_CODENAME					"Bandor"
 #define GEANY_HOMEPAGE					"http://geany.uvena.de/"
+#define GEANY_PROJECT_EXT				"geany"
 #define GEANY_USE_WIN32_DIALOG			0
 #define GEANY_CHECK_FILE_DELAY			30
 #define GEANY_WORDCHARS					"_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
@@ -57,7 +60,7 @@
 #else
 #define GEANY_DEFAULT_TOOLS_TERMINAL	"xterm"
 #endif
-#define GEANY_DEFAULT_TOOLS_BROWSER		"mozilla"
+#define GEANY_DEFAULT_TOOLS_BROWSER		"firefox"
 #define GEANY_DEFAULT_TOOLS_PRINTCMD	"lpr"
 #define GEANY_DEFAULT_TOOLS_GREP		"grep"
 #define GEANY_DEFAULT_MRU_LENGTH		10
@@ -67,6 +70,10 @@
 
 
 
+// simple forward declaration to avoid unnecessary inclusion of project.h
+typedef struct _GeanyProject GeanyProject;
+
+
 /* store some pointers and variables for frequently used widgets  */
 typedef struct MyApp
 {
@@ -84,6 +91,7 @@ typedef struct MyApp
 	gboolean			 sidebar_symbol_visible;
 	gboolean			 sidebar_openfiles_visible;
 	gboolean			 sidebar_visible;
+	gboolean			 statusbar_visible;
 	gboolean			 msgwindow_visible;
 	gboolean			 fullscreen;
 	gboolean			 beep_on_errors;
@@ -98,20 +106,24 @@ typedef struct MyApp
 	// I know, it is a bit confusing, but this line breaking is globally,
 	// to change the default value at startup, I think
 	gboolean			 pref_editor_line_breaking;
-	gboolean			 pref_editor_use_auto_indention;
+	gint			 	 pref_editor_indention_mode;
 	gboolean			 pref_editor_use_indicators;
 	gboolean			 pref_editor_show_white_space;
 	gboolean			 pref_editor_show_indent_guide;
 	gboolean			 pref_editor_show_line_endings;
+	gboolean			 pref_editor_auto_complete_symbols;
 	gboolean			 pref_editor_auto_close_xml_tags;
 	gboolean			 pref_editor_auto_complete_constructs;
 	gboolean			 pref_editor_folding;
 	gboolean			 pref_editor_unfold_all_children;
+	gboolean			 pref_editor_show_scrollbars;
 	gint				 pref_editor_tab_width;
+	gboolean			 pref_editor_use_tabs;
 	gint				 pref_editor_default_encoding;
 	gboolean			 pref_editor_new_line;
 	gboolean			 pref_editor_replace_tabs;
 	gboolean			 pref_editor_trail_space;
+	gboolean			 pref_editor_disable_dnd;
 	gboolean			 pref_main_load_session;
 	gboolean			 pref_main_save_winpos;
 	gboolean			 pref_main_confirm_exit;
@@ -132,6 +144,7 @@ typedef struct MyApp
 	gint				 long_line_type;
 	gint				 long_line_column;
 	gchar				*long_line_color;
+	gchar				*context_action_cmd;
 	gchar				*pref_template_developer;
 	gchar				*pref_template_company;
 	gchar				*pref_template_mail;
@@ -144,6 +157,7 @@ typedef struct MyApp
 	gchar				*datadir;
 	gchar				*docdir;
 	gchar				*custom_date_format;
+	gchar				**custom_commands;
 	gchar				*tools_browser_cmd;
 	gchar				*tools_make_cmd;
 	gchar				*tools_term_cmd;
@@ -171,7 +185,7 @@ typedef struct MyApp
 	GtkWidget			*redo_items[3];
 	GtkWidget			*undo_items[3];
 	GtkWidget			*save_buttons[4];
-	GtkWidget			*sensitive_buttons[37];
+	GtkWidget			*sensitive_buttons[39];
 	GtkWidget			*open_colorsel;
 	GtkWidget			*open_fontsel;
 	GtkWidget			*open_filesel;
@@ -180,20 +194,12 @@ typedef struct MyApp
 	GtkWidget			*default_tag_tree;
 	const TMWorkspace	*tm_workspace;
 	GQueue				*recent_queue;
+	GeanyProject		*project; // currently active project or NULL if none is open
 } MyApp;
 
 MyApp *app;
 
 
-// small struct to track tag name and type together
-typedef struct GeanySymbol
-{
-	gchar	*str;
-	gint	 type;
-	gint	 line;
-} GeanySymbol;
-
-
 enum
 {
 	GEANY_IMAGE_SMALL_CROSS,
@@ -211,7 +217,15 @@ enum
 	RIGHT
 };
 
-enum {
+enum
+{
+	INDENT_NONE = 0,
+	INDENT_BASIC,
+	INDENT_ADVANCED
+};
+
+enum
+{
 	KILOBYTE = 1024,
 	MEGABYTE = (KILOBYTE*1024),
 	GIGABYTE = (MEGABYTE*1024)
