@@ -1,7 +1,8 @@
 /*
  *      highlighting.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: highlighting.h 779 2006-09-01 17:07:49Z eht16 $
+ * $Id: highlighting.h 1521 2007-05-11 11:58:56Z ntrel $
  */
 
 
@@ -32,18 +33,8 @@
 #include "ScintillaWidget.h"
 
 
-typedef struct
-{
-	gint		  styling[55][4];
-	gchar		**keywords;
-	gchar		 *wordchars;
-} style_set;
-
-
 void styleset_free_styles(void);
 
-void styleset_common(ScintillaObject *sci, gint style_bits);
-
 void styleset_c(ScintillaObject *sci);
 
 void styleset_cpp(ScintillaObject *sci);
@@ -70,8 +61,6 @@ void styleset_sh(ScintillaObject *sci);
 
 void styleset_xml(ScintillaObject *sci);
 
-void styleset_markup(ScintillaObject *sci);
-
 void styleset_docbook(ScintillaObject *sci);
 
 void styleset_none(ScintillaObject *sci);
@@ -86,6 +75,8 @@ void styleset_sql(ScintillaObject *sci);
 
 void styleset_caml(ScintillaObject *sci);
 
+void styleset_haskell(ScintillaObject *sci);
+
 void styleset_oms(ScintillaObject *sci);
 
 void styleset_tcl(ScintillaObject *sci);
@@ -98,4 +89,10 @@ void styleset_diff(ScintillaObject *sci);
 
 void styleset_ferite(ScintillaObject *sci);
 
+void styleset_vhdl(ScintillaObject *sci);
+
+void styleset_js(ScintillaObject *sci);
+
+void styleset_lua(ScintillaObject *sci);
+
 #endif
