@@ -18,7 +18,11 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: keyfile.c 1270 2007-02-12 18:42:15Z eht16 $
+ * $Id: keyfile.c 1468 2007-04-21 12:25:17Z ntrel $
+ */
+
+/*
+ * geany.conf preferences file loading and saving.
  */
 
 #include <stdlib.h>
@@ -42,6 +46,8 @@
 #include "vte.h"
 #include "main.h"
 #include "msgwindow.h"
+#include "search.h"
+#include "project.h"
 
 
 static gchar *scribble_text = NULL;
@@ -50,143 +56,10 @@ static gint hpan_position;
 static gint vpan_position;
 
 
-void configuration_save()
+static void save_recent_files(GKeyFile *config)
 {
-	guint i = 0, j = 0, max;
-	gint idx;
-	gboolean config_exists;
-	gboolean have_session_files;
-	GKeyFile *config = g_key_file_new();
-	gchar *configfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "geany.conf", NULL);
-	gchar *data, *tmp;
-	gchar *entry = g_malloc(14);
 	gchar **recent_files = g_new0(gchar*, app->mru_length + 1);
-	GtkTextBuffer *buffer;
-	GtkTextIter start, end;
-
-	config_exists = g_key_file_load_from_file(config, configfile, G_KEY_FILE_KEEP_COMMENTS, NULL);
-
-	if (!config_exists)
-	{
-		gchar *start_comm = g_strdup_printf(_("%s configuration file, edit as you need"), PACKAGE);
-		g_key_file_set_comment(config, NULL, NULL, start_comm, NULL);
-		g_free(start_comm);
-	}
-
-	// gets the text from the scribble textview
-	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(lookup_widget(app->window, "textview_scribble")));
-	gtk_text_buffer_get_bounds(buffer, &start, &end);
-	scribble_text = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
-
-	// store basic settings
-	if (app->pref_main_save_winpos)
-	{
-		g_key_file_set_integer(config, PACKAGE, "treeview_position",
-				gtk_paned_get_position(GTK_PANED(lookup_widget(app->window, "hpaned1"))));
-		g_key_file_set_integer(config, PACKAGE, "msgwindow_position",
-				gtk_paned_get_position(GTK_PANED(lookup_widget(app->window, "vpaned1"))));
-	}
-
-	g_key_file_set_integer(config, PACKAGE, "mru_length", app->mru_length);
-	g_key_file_set_integer(config, PACKAGE, "long_line_type", app->long_line_type);
-	g_key_file_set_integer(config, PACKAGE, "tab_pos_editor", app->tab_pos_editor);
-	g_key_file_set_integer(config, PACKAGE, "tab_pos_msgwin", app->tab_pos_msgwin);
-	g_key_file_set_integer(config, PACKAGE, "tab_pos_sidebar", app->tab_pos_sidebar);
-	g_key_file_set_integer(config, PACKAGE, "autocompletion_max_height", app->autocompletion_max_height);
-	g_key_file_set_integer(config, PACKAGE, "long_line_column", app->long_line_column);
-	g_key_file_set_string(config, PACKAGE, "long_line_color", app->long_line_color);
-	g_key_file_set_boolean(config, PACKAGE, "beep_on_errors", app->beep_on_errors);
-	g_key_file_set_boolean(config, PACKAGE, "sidebar_symbol_visible", app->sidebar_symbol_visible);
-	g_key_file_set_boolean(config, PACKAGE, "sidebar_openfiles_visible", app->sidebar_openfiles_visible);
-	g_key_file_set_boolean(config, PACKAGE, "sidebar_visible", app->sidebar_visible);
-	g_key_file_set_boolean(config, PACKAGE, "msgwindow_visible", app->msgwindow_visible);
-	g_key_file_set_boolean(config, PACKAGE, "use_folding", app->pref_editor_folding);
-	g_key_file_set_boolean(config, PACKAGE, "unfold_all_children", app->pref_editor_unfold_all_children);
-	g_key_file_set_boolean(config, PACKAGE, "use_auto_indention", app->pref_editor_use_auto_indention);
-	g_key_file_set_boolean(config, PACKAGE, "use_tab_to_indent", app->use_tab_to_indent);
-	g_key_file_set_boolean(config, PACKAGE, "use_indicators", app->pref_editor_use_indicators);
-	g_key_file_set_boolean(config, PACKAGE, "show_indent_guide", app->pref_editor_show_indent_guide);
-	g_key_file_set_boolean(config, PACKAGE, "show_white_space", app->pref_editor_show_white_space);
-	g_key_file_set_boolean(config, PACKAGE, "show_markers_margin", app->show_markers_margin);
-	g_key_file_set_boolean(config, PACKAGE, "show_linenumber_margin", app->show_linenumber_margin);
-	g_key_file_set_boolean(config, PACKAGE, "line_breaking", app->pref_editor_line_breaking);
-	g_key_file_set_boolean(config, PACKAGE, "show_line_endings", app->pref_editor_show_line_endings);
-	g_key_file_set_boolean(config, PACKAGE, "fullscreen", app->fullscreen);
-	g_key_file_set_boolean(config, PACKAGE, "tab_order_ltr", app->tab_order_ltr);
-	g_key_file_set_boolean(config, PACKAGE, "show_notebook_tabs", app->show_notebook_tabs);
-	g_key_file_set_boolean(config, PACKAGE, "brace_match_ltgt", app->brace_match_ltgt);
-	g_key_file_set_boolean(config, PACKAGE, "switch_msgwin_pages", app->switch_msgwin_pages);
-	g_key_file_set_boolean(config, PACKAGE, "auto_close_xml_tags", app->pref_editor_auto_close_xml_tags);
-	g_key_file_set_boolean(config, PACKAGE, "auto_complete_constructs", app->pref_editor_auto_complete_constructs);
-#ifdef HAVE_VTE
-	g_key_file_set_boolean(config, "VTE", "load_vte", vte_info.load_vte);
-	if (vte_info.load_vte && vc != NULL)
-	{
-		gchar *tmp_string;
-
-		g_key_file_set_string(config, "VTE", "emulation", vc->emulation);
-		g_key_file_set_string(config, "VTE", "font", vc->font);
-		g_key_file_set_boolean(config, "VTE", "scroll_on_key", vc->scroll_on_key);
-		g_key_file_set_boolean(config, "VTE", "scroll_on_out", vc->scroll_on_out);
-		g_key_file_set_boolean(config, "VTE", "ignore_menu_bar_accel", vc->ignore_menu_bar_accel);
-		g_key_file_set_boolean(config, "VTE", "follow_path", vc->follow_path);
-		g_key_file_set_boolean(config, "VTE", "run_in_vte", vc->run_in_vte);
-		g_key_file_set_integer(config, "VTE", "scrollback_lines", vc->scrollback_lines);
-		g_key_file_set_string(config, "VTE", "font", vc->font);
-		g_key_file_set_string(config, "VTE", "shell", vc->shell);
-		tmp_string = utils_get_hex_from_color(vc->colour_fore);
-		g_key_file_set_string(config, "VTE", "colour_fore", tmp_string);
-		g_free(tmp_string);
-		tmp_string = utils_get_hex_from_color(vc->colour_back);
-		g_key_file_set_string(config, "VTE", "colour_back", tmp_string);
-		g_free(tmp_string);
-		vte_get_working_directory();	// refresh vte_info.dir
-		g_key_file_set_string(config, "VTE", "last_dir", vte_info.dir);
-	}
-#endif
-	g_key_file_set_string(config, PACKAGE, "custom_date_format", app->custom_date_format);
-	g_key_file_set_string(config, PACKAGE, "editor_font", app->editor_font);
-	g_key_file_set_string(config, PACKAGE, "tagbar_font", app->tagbar_font);
-	g_key_file_set_string(config, PACKAGE, "msgwin_font", app->msgwin_font);
-	g_key_file_set_string(config, PACKAGE, "scribble_text", scribble_text);
-	if (app->pref_main_save_winpos && ! app->fullscreen)
-	{
-		gtk_window_get_position(GTK_WINDOW(app->window), &app->geometry[0], &app->geometry[1]);
-		gtk_window_get_size(GTK_WINDOW(app->window), &app->geometry[2], &app->geometry[3]);
-		g_key_file_set_integer_list(config, PACKAGE, "geometry", app->geometry, 4);
-	}
-	g_key_file_set_integer(config, PACKAGE, "pref_editor_tab_width", app->pref_editor_tab_width);
-	g_key_file_set_boolean(config, PACKAGE, "pref_main_confirm_exit", app->pref_main_confirm_exit);
-	g_key_file_set_boolean(config, PACKAGE, "pref_main_suppress_search_dialogs", app->pref_main_suppress_search_dialogs);
-	g_key_file_set_boolean(config, PACKAGE, "pref_main_load_session", app->pref_main_load_session);
-	g_key_file_set_boolean(config, PACKAGE, "pref_main_save_winpos", app->pref_main_save_winpos);
-	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show", app->toolbar_visible);
-	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_search", app->pref_toolbar_show_search);
-	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_goto", app->pref_toolbar_show_goto);
-	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_zoom", app->pref_toolbar_show_zoom);
-	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_undo", app->pref_toolbar_show_undo);
-	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_compile", app->pref_toolbar_show_compile);
-	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_colour", app->pref_toolbar_show_colour);
-	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_fileops", app->pref_toolbar_show_fileops);
-	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_quit", app->pref_toolbar_show_quit);
-	g_key_file_set_integer(config, PACKAGE, "pref_toolbar_icon_style", app->toolbar_icon_style);
-	g_key_file_set_integer(config, PACKAGE, "pref_toolbar_icon_size", app->toolbar_icon_size);
-	g_key_file_set_boolean(config, PACKAGE, "pref_editor_new_line", app->pref_editor_new_line);
-	g_key_file_set_boolean(config, PACKAGE, "pref_editor_replace_tabs", app->pref_editor_replace_tabs);
-	g_key_file_set_boolean(config, PACKAGE, "pref_editor_trail_space", app->pref_editor_trail_space);
-	g_key_file_set_string(config, PACKAGE, "pref_editor_default_encoding", encodings[app->pref_editor_default_encoding].charset);
-	g_key_file_set_string(config, PACKAGE, "pref_template_developer", app->pref_template_developer);
-	g_key_file_set_string(config, PACKAGE, "pref_template_company", app->pref_template_company);
-	g_key_file_set_string(config, PACKAGE, "pref_template_mail", app->pref_template_mail);
-	g_key_file_set_string(config, PACKAGE, "pref_template_initial", app->pref_template_initial);
-	g_key_file_set_string(config, PACKAGE, "pref_template_version", app->pref_template_version);
-
-	// store tools settings
-	g_key_file_set_string(config, "tools", "make_cmd", app->tools_make_cmd ? app->tools_make_cmd : "");
-	g_key_file_set_string(config, "tools", "term_cmd", app->tools_term_cmd ? app->tools_term_cmd : "");
-	g_key_file_set_string(config, "tools", "browser_cmd", app->tools_browser_cmd ? app->tools_browser_cmd : "");
-	g_key_file_set_string(config, "tools", "print_cmd", app->tools_print_cmd ? app->tools_print_cmd : "");
-	g_key_file_set_string(config, "tools", "grep_cmd", app->tools_grep_cmd? app->tools_grep_cmd: "");
+	guint i;
 
 	for (i = 0; i < app->mru_length; i++)
 	{
@@ -207,6 +80,16 @@ void configuration_save()
 	g_key_file_set_string_list(config, "files", "recent_files",
 				(const gchar**)recent_files, app->mru_length);
 	g_strfreev(recent_files);
+}
+
+
+static void save_session_files(GKeyFile *config)
+{
+	gint idx;
+	gboolean have_session_files;
+	gchar *tmp;
+	gchar *entry = g_malloc(14);
+	guint i = 0, j = 0, max;
 
 	if (cl_options.load_session)
 	{
@@ -249,6 +132,160 @@ void configuration_save()
 			}
 		}
 	}
+	g_free(entry);
+}
+
+
+void configuration_save()
+{
+	gboolean config_exists;
+	GKeyFile *config = g_key_file_new();
+	gchar *configfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "geany.conf", NULL);
+	gchar *data;
+	GtkTextBuffer *buffer;
+	GtkTextIter start, end;
+
+	config_exists = g_key_file_load_from_file(config, configfile, G_KEY_FILE_KEEP_COMMENTS, NULL);
+
+	if (!config_exists)
+	{
+		gchar *start_comm = g_strdup_printf(_("%s configuration file, edit as you need"), PACKAGE);
+		g_key_file_set_comment(config, NULL, NULL, start_comm, NULL);
+		g_free(start_comm);
+	}
+
+	// gets the text from the scribble textview
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(lookup_widget(app->window, "textview_scribble")));
+	gtk_text_buffer_get_bounds(buffer, &start, &end);
+	scribble_text = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
+
+	// store basic settings
+	if (app->pref_main_save_winpos)
+	{
+		g_key_file_set_integer(config, PACKAGE, "treeview_position",
+				gtk_paned_get_position(GTK_PANED(lookup_widget(app->window, "hpaned1"))));
+		g_key_file_set_integer(config, PACKAGE, "msgwindow_position",
+				gtk_paned_get_position(GTK_PANED(lookup_widget(app->window, "vpaned1"))));
+	}
+
+	g_key_file_set_integer(config, PACKAGE, "mru_length", app->mru_length);
+	g_key_file_set_integer(config, PACKAGE, "long_line_type", app->long_line_type);
+	g_key_file_set_integer(config, PACKAGE, "tab_pos_editor", app->tab_pos_editor);
+	g_key_file_set_integer(config, PACKAGE, "tab_pos_msgwin", app->tab_pos_msgwin);
+	g_key_file_set_integer(config, PACKAGE, "tab_pos_sidebar", app->tab_pos_sidebar);
+	g_key_file_set_integer(config, PACKAGE, "autocompletion_max_height", app->autocompletion_max_height);
+	g_key_file_set_integer(config, PACKAGE, "long_line_column", app->long_line_column);
+	g_key_file_set_string(config, PACKAGE, "long_line_color", app->long_line_color);
+	g_key_file_set_boolean(config, PACKAGE, "beep_on_errors", app->beep_on_errors);
+	g_key_file_set_boolean(config, PACKAGE, "sidebar_symbol_visible", app->sidebar_symbol_visible);
+	g_key_file_set_boolean(config, PACKAGE, "sidebar_openfiles_visible", app->sidebar_openfiles_visible);
+	g_key_file_set_boolean(config, PACKAGE, "sidebar_visible", app->sidebar_visible);
+	g_key_file_set_boolean(config, PACKAGE, "statusbar_visible", app->statusbar_visible);
+	g_key_file_set_boolean(config, PACKAGE, "msgwindow_visible", app->msgwindow_visible);
+	g_key_file_set_boolean(config, PACKAGE, "use_folding", app->pref_editor_folding);
+	g_key_file_set_boolean(config, PACKAGE, "unfold_all_children", app->pref_editor_unfold_all_children);
+	g_key_file_set_boolean(config, PACKAGE, "show_editor_scrollbars", app->pref_editor_show_scrollbars);
+	g_key_file_set_integer(config, PACKAGE, "indention_mode", app->pref_editor_indention_mode);
+	g_key_file_set_boolean(config, PACKAGE, "use_tab_to_indent", app->use_tab_to_indent);
+	g_key_file_set_boolean(config, PACKAGE, "use_indicators", app->pref_editor_use_indicators);
+	g_key_file_set_boolean(config, PACKAGE, "show_indent_guide", app->pref_editor_show_indent_guide);
+	g_key_file_set_boolean(config, PACKAGE, "show_white_space", app->pref_editor_show_white_space);
+	g_key_file_set_boolean(config, PACKAGE, "show_markers_margin", app->show_markers_margin);
+	g_key_file_set_boolean(config, PACKAGE, "show_linenumber_margin", app->show_linenumber_margin);
+	g_key_file_set_boolean(config, PACKAGE, "line_breaking", app->pref_editor_line_breaking);
+	g_key_file_set_boolean(config, PACKAGE, "show_line_endings", app->pref_editor_show_line_endings);
+	g_key_file_set_boolean(config, PACKAGE, "fullscreen", app->fullscreen);
+	g_key_file_set_boolean(config, PACKAGE, "tab_order_ltr", app->tab_order_ltr);
+	g_key_file_set_boolean(config, PACKAGE, "show_notebook_tabs", app->show_notebook_tabs);
+	g_key_file_set_boolean(config, PACKAGE, "brace_match_ltgt", app->brace_match_ltgt);
+	g_key_file_set_boolean(config, PACKAGE, "switch_msgwin_pages", app->switch_msgwin_pages);
+	g_key_file_set_boolean(config, PACKAGE, "auto_close_xml_tags", app->pref_editor_auto_close_xml_tags);
+	g_key_file_set_boolean(config, PACKAGE, "auto_complete_constructs", app->pref_editor_auto_complete_constructs);
+	g_key_file_set_boolean(config, PACKAGE, "auto_complete_symbols", app->pref_editor_auto_complete_symbols);
+#ifdef HAVE_VTE
+	g_key_file_set_boolean(config, "VTE", "load_vte", vte_info.load_vte);
+	if (vte_info.load_vte && vc != NULL)
+	{
+		gchar *tmp_string;
+
+		g_key_file_set_string(config, "VTE", "emulation", vc->emulation);
+		g_key_file_set_string(config, "VTE", "font", vc->font);
+		g_key_file_set_boolean(config, "VTE", "scroll_on_key", vc->scroll_on_key);
+		g_key_file_set_boolean(config, "VTE", "scroll_on_out", vc->scroll_on_out);
+		g_key_file_set_boolean(config, "VTE", "ignore_menu_bar_accel", vc->ignore_menu_bar_accel);
+		g_key_file_set_boolean(config, "VTE", "follow_path", vc->follow_path);
+		g_key_file_set_boolean(config, "VTE", "run_in_vte", vc->run_in_vte);
+		g_key_file_set_integer(config, "VTE", "scrollback_lines", vc->scrollback_lines);
+		g_key_file_set_string(config, "VTE", "font", vc->font);
+		g_key_file_set_string(config, "VTE", "shell", vc->shell);
+		tmp_string = utils_get_hex_from_color(vc->colour_fore);
+		g_key_file_set_string(config, "VTE", "colour_fore", tmp_string);
+		g_free(tmp_string);
+		tmp_string = utils_get_hex_from_color(vc->colour_back);
+		g_key_file_set_string(config, "VTE", "colour_back", tmp_string);
+		g_free(tmp_string);
+		vte_get_working_directory();	// refresh vte_info.dir
+		g_key_file_set_string(config, "VTE", "last_dir", vte_info.dir);
+	}
+#endif
+	g_key_file_set_string(config, PACKAGE, "custom_date_format", app->custom_date_format);
+	g_key_file_set_string(config, PACKAGE, "context_action_cmd", app->context_action_cmd);
+	if (app->custom_commands != NULL)
+	{
+		g_key_file_set_string_list(config, PACKAGE, "custom_commands",
+				(const gchar**) app->custom_commands, g_strv_length(app->custom_commands));
+	}
+
+	g_key_file_set_string(config, PACKAGE, "editor_font", app->editor_font);
+	g_key_file_set_string(config, PACKAGE, "tagbar_font", app->tagbar_font);
+	g_key_file_set_string(config, PACKAGE, "msgwin_font", app->msgwin_font);
+	g_key_file_set_string(config, PACKAGE, "scribble_text", scribble_text);
+	if (app->pref_main_save_winpos && ! app->fullscreen)
+	{
+		gtk_window_get_position(GTK_WINDOW(app->window), &app->geometry[0], &app->geometry[1]);
+		gtk_window_get_size(GTK_WINDOW(app->window), &app->geometry[2], &app->geometry[3]);
+		g_key_file_set_integer_list(config, PACKAGE, "geometry", app->geometry, 4);
+	}
+	g_key_file_set_integer(config, PACKAGE, "pref_editor_tab_width", app->pref_editor_tab_width);
+	g_key_file_set_boolean(config, PACKAGE, "pref_editor_use_tabs", app->pref_editor_use_tabs);
+	g_key_file_set_boolean(config, PACKAGE, "pref_main_confirm_exit", app->pref_main_confirm_exit);
+	g_key_file_set_boolean(config, PACKAGE, "pref_main_suppress_search_dialogs", app->pref_main_suppress_search_dialogs);
+	g_key_file_set_boolean(config, PACKAGE, "pref_main_load_session", app->pref_main_load_session);
+	g_key_file_set_boolean(config, PACKAGE, "pref_main_save_winpos", app->pref_main_save_winpos);
+	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show", app->toolbar_visible);
+	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_search", app->pref_toolbar_show_search);
+	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_goto", app->pref_toolbar_show_goto);
+	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_zoom", app->pref_toolbar_show_zoom);
+	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_undo", app->pref_toolbar_show_undo);
+	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_compile", app->pref_toolbar_show_compile);
+	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_colour", app->pref_toolbar_show_colour);
+	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_fileops", app->pref_toolbar_show_fileops);
+	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_quit", app->pref_toolbar_show_quit);
+	g_key_file_set_integer(config, PACKAGE, "pref_toolbar_icon_style", app->toolbar_icon_style);
+	g_key_file_set_integer(config, PACKAGE, "pref_toolbar_icon_size", app->toolbar_icon_size);
+	g_key_file_set_boolean(config, PACKAGE, "pref_editor_new_line", app->pref_editor_new_line);
+	g_key_file_set_boolean(config, PACKAGE, "pref_editor_replace_tabs", app->pref_editor_replace_tabs);
+	g_key_file_set_boolean(config, PACKAGE, "pref_editor_trail_space", app->pref_editor_trail_space);
+	g_key_file_set_boolean(config, PACKAGE, "pref_editor_disable_dnd", app->pref_editor_disable_dnd);
+	g_key_file_set_string(config, PACKAGE, "pref_editor_default_encoding", encodings[app->pref_editor_default_encoding].charset);
+	g_key_file_set_string(config, PACKAGE, "pref_template_developer", app->pref_template_developer);
+	g_key_file_set_string(config, PACKAGE, "pref_template_company", app->pref_template_company);
+	g_key_file_set_string(config, PACKAGE, "pref_template_mail", app->pref_template_mail);
+	g_key_file_set_string(config, PACKAGE, "pref_template_initial", app->pref_template_initial);
+	g_key_file_set_string(config, PACKAGE, "pref_template_version", app->pref_template_version);
+
+	// store tools settings
+	g_key_file_set_string(config, "tools", "make_cmd", app->tools_make_cmd ? app->tools_make_cmd : "");
+	g_key_file_set_string(config, "tools", "term_cmd", app->tools_term_cmd ? app->tools_term_cmd : "");
+	g_key_file_set_string(config, "tools", "browser_cmd", app->tools_browser_cmd ? app->tools_browser_cmd : "");
+	g_key_file_set_string(config, "tools", "print_cmd", app->tools_print_cmd ? app->tools_print_cmd : "");
+	g_key_file_set_string(config, "tools", "grep_cmd", app->tools_grep_cmd ? app->tools_grep_cmd : "");
+
+	g_key_file_set_string(config, "search", "fif_extra_options", search_prefs.fif_extra_options ? search_prefs.fif_extra_options : "");
+
+	project_save_prefs(config);	// save project filename, etc.
+	save_recent_files(config);
+	save_session_files(config);
 
 	// write the file
 	data = g_key_file_to_data(config, NULL, NULL);
@@ -257,11 +294,51 @@ void configuration_save()
 
 	g_key_file_free(config);
 	g_free(configfile);
-	g_free(entry);
 	g_free(scribble_text);
 }
 
 
+static void load_file_lists(GKeyFile *config)
+{
+	gchar **recent_files;
+	guint i;
+	gsize len = 0;
+	gboolean have_session_files;
+	gchar *entry = g_malloc(14);
+	gchar *tmp_string;
+	GError *error = NULL;
+
+	recent_files = g_key_file_get_string_list(config, "files", "recent_files", &len, NULL);
+	if (recent_files != NULL)
+	{
+		for (i = 0; (i < len) && (i < app->mru_length); i++)
+		{
+			gchar *filename = g_strdup(recent_files[i]);
+			g_queue_push_tail(app->recent_queue, filename);
+		}
+	}
+	g_strfreev(recent_files);
+
+	session_files = g_ptr_array_new();
+	have_session_files = TRUE;
+	i = 0;
+	while (have_session_files)
+	{
+		g_snprintf(entry, 13, "FILE_NAME_%d", i);
+		tmp_string = g_key_file_get_string(config, "files", entry, &error);
+		if (! tmp_string || error)
+		{
+			g_error_free(error);
+			error = NULL;
+			have_session_files = FALSE;
+		}
+		g_ptr_array_add(session_files, tmp_string);
+		i++;
+	}
+	g_free(entry);
+}
+
+
 #define GEANY_GET_SETTING(propertyname, value, default_value) \
 	if (g_object_class_find_property( \
 		G_OBJECT_GET_CLASS(G_OBJECT(gtk_settings_get_default())), propertyname)) \
@@ -273,16 +350,12 @@ void configuration_save()
 gboolean configuration_load()
 {
 	gboolean config_exists;
-	gboolean have_session_files;
-	guint i, geo_len;
+	guint geo_len;
 	gint *geo;
-	gsize len = 0;
 	gchar *configfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "geany.conf", NULL);
-	gchar *entry = g_malloc(14);
 	gchar *tmp_string, *tmp_string2;
 	GKeyFile *config = g_key_file_new();
 	GError *error = NULL;
-	gchar **recent_files;
 
 	config_exists = g_key_file_load_from_file(config, configfile, G_KEY_FILE_KEEP_COMMENTS, NULL);
 
@@ -307,9 +380,10 @@ gboolean configuration_load()
 	app->sidebar_symbol_visible = utils_get_setting_boolean(config, PACKAGE, "sidebar_symbol_visible", TRUE);
 	app->sidebar_openfiles_visible = utils_get_setting_boolean(config, PACKAGE, "sidebar_openfiles_visible", TRUE);
 	app->sidebar_visible = utils_get_setting_boolean(config, PACKAGE, "sidebar_visible", TRUE);
+	app->statusbar_visible = utils_get_setting_boolean(config, PACKAGE, "statusbar_visible", TRUE);
 	app->msgwindow_visible = utils_get_setting_boolean(config, PACKAGE, "msgwindow_visible", TRUE);
 	app->pref_editor_line_breaking = utils_get_setting_boolean(config, PACKAGE, "line_breaking", FALSE); //default is off for better performance
-	app->pref_editor_use_auto_indention = utils_get_setting_boolean(config, PACKAGE, "use_auto_indention", TRUE);
+	app->pref_editor_indention_mode = utils_get_setting_integer(config, PACKAGE, "indention_mode", INDENT_ADVANCED);
 	app->use_tab_to_indent = utils_get_setting_boolean(config, PACKAGE, "use_tab_to_indent", FALSE);
 	app->pref_editor_use_indicators = utils_get_setting_boolean(config, PACKAGE, "use_indicators", TRUE);
 	app->pref_editor_show_indent_guide = utils_get_setting_boolean(config, PACKAGE, "show_indent_guide", FALSE);
@@ -317,8 +391,10 @@ gboolean configuration_load()
 	app->pref_editor_show_line_endings = utils_get_setting_boolean(config, PACKAGE, "show_line_endings", FALSE);
 	app->pref_editor_auto_close_xml_tags = utils_get_setting_boolean(config, PACKAGE, "auto_close_xml_tags", TRUE);
 	app->pref_editor_auto_complete_constructs = utils_get_setting_boolean(config, PACKAGE, "auto_complete_constructs", TRUE);
+	app->pref_editor_auto_complete_symbols = utils_get_setting_boolean(config, PACKAGE, "auto_complete_symbols", TRUE);
 	app->pref_editor_folding = utils_get_setting_boolean(config, PACKAGE, "use_folding", TRUE);
 	app->pref_editor_unfold_all_children = utils_get_setting_boolean(config, PACKAGE, "unfold_all_children", FALSE);
+	app->pref_editor_show_scrollbars = utils_get_setting_boolean(config, PACKAGE, "show_editor_scrollbars", TRUE);
 	app->show_markers_margin = utils_get_setting_boolean(config, PACKAGE, "show_markers_margin", TRUE);
 	app->show_linenumber_margin = utils_get_setting_boolean(config, PACKAGE, "show_linenumber_margin", TRUE);
 	app->fullscreen = utils_get_setting_boolean(config, PACKAGE, "fullscreen", FALSE);
@@ -327,6 +403,8 @@ gboolean configuration_load()
 	app->brace_match_ltgt = utils_get_setting_boolean(config, PACKAGE, "brace_match_ltgt", FALSE);
 	app->switch_msgwin_pages = utils_get_setting_boolean(config, PACKAGE, "switch_msgwin_pages", FALSE);
 	app->custom_date_format = utils_get_setting_string(config, PACKAGE, "custom_date_format", "");
+	app->context_action_cmd = utils_get_setting_string(config, PACKAGE, "context_action_cmd", "");
+	app->custom_commands = g_key_file_get_string_list(config, PACKAGE, "custom_commands", NULL, NULL);
 	app->editor_font = utils_get_setting_string(config, PACKAGE, "editor_font", GEANY_DEFAULT_FONT_EDITOR);
 	app->tagbar_font = utils_get_setting_string(config, PACKAGE, "tagbar_font", GEANY_DEFAULT_FONT_SYMBOL_LIST);
 	app->msgwin_font = utils_get_setting_string(config, PACKAGE, "msgwin_font", GEANY_DEFAULT_FONT_MSG_WINDOW);
@@ -354,6 +432,7 @@ gboolean configuration_load()
 
 
 	app->pref_editor_tab_width = utils_get_setting_integer(config, PACKAGE, "pref_editor_tab_width", 4);
+	app->pref_editor_use_tabs = utils_get_setting_boolean(config, PACKAGE, "pref_editor_use_tabs", TRUE);
 	tmp_string = utils_get_setting_string(config, PACKAGE, "pref_editor_default_encoding",
 											encodings[GEANY_ENCODING_UTF_8].charset);
 	if (tmp_string)
@@ -430,6 +509,7 @@ gboolean configuration_load()
 	app->pref_editor_replace_tabs = utils_get_setting_boolean(config, PACKAGE, "pref_editor_replace_tabs", FALSE);
 	app->pref_editor_new_line = utils_get_setting_boolean(config, PACKAGE, "pref_editor_new_line", TRUE);
 	app->pref_editor_trail_space = utils_get_setting_boolean(config, PACKAGE, "pref_editor_trail_space", FALSE);
+	app->pref_editor_disable_dnd = utils_get_setting_boolean(config, PACKAGE, "pref_editor_disable_dnd", FALSE);
 
 	tmp_string = g_find_program_in_path(GEANY_DEFAULT_TOOLS_MAKE);
 	app->tools_make_cmd = utils_get_setting_string(config, "tools", "make_cmd", tmp_string);
@@ -444,7 +524,12 @@ gboolean configuration_load()
 	g_free(tmp_string);
 
 	tmp_string2 = g_find_program_in_path(GEANY_DEFAULT_TOOLS_PRINTCMD);
+#ifdef G_OS_WIN32
+	// single quote paths on Win32 for g_spawn_command_line_async
+	tmp_string = g_strconcat("'", tmp_string2, "' '%f'", NULL);
+#else
 	tmp_string = g_strconcat(tmp_string2, " %f", NULL);
+#endif
 	app->tools_print_cmd = utils_get_setting_string(config, "tools", "print_cmd", tmp_string);
 	g_free(tmp_string);
 	g_free(tmp_string2);
@@ -453,37 +538,13 @@ gboolean configuration_load()
 	app->tools_grep_cmd = utils_get_setting_string(config, "tools", "grep_cmd", tmp_string);
 	g_free(tmp_string);
 
-	recent_files = g_key_file_get_string_list(config, "files", "recent_files", &len, NULL);
-	if (recent_files != NULL)
-	{
-		for (i = 0; (i < len) && (i < app->mru_length); i++)
-		{
-			gchar *filename = g_strdup(recent_files[i]);
-			g_queue_push_tail(app->recent_queue, filename);
-		}
-	}
-	g_strfreev(recent_files);
+	search_prefs.fif_extra_options = utils_get_setting_string(config, "search", "fif_extra_options", "");
 
-	session_files = g_ptr_array_new();
-	have_session_files = TRUE;
-	i = 0;
-	while (have_session_files)
-	{
-		g_snprintf(entry, 13, "FILE_NAME_%d", i);
-		tmp_string = g_key_file_get_string(config, "files", entry, &error);
-		if (! tmp_string || error)
-		{
-			g_error_free(error);
-			error = NULL;
-			have_session_files = FALSE;
-		}
-		g_ptr_array_add(session_files, tmp_string);
-		i++;
-	}
+	project_load_prefs(config);
+	load_file_lists(config);
 
 	g_key_file_free(config);
 	g_free(configfile);
-	g_free(entry);
 	g_free(geo);
 	return TRUE;
 }
@@ -495,6 +556,8 @@ gboolean configuration_open_files()
 	guint x, pos, y, len;
 	gboolean ret = FALSE, failure = FALSE;
 
+	document_delay_colourise();
+
 	i = app->tab_order_ltr ? 0 : (session_files->len - 1);
 	while (TRUE)
 	{
@@ -555,6 +618,7 @@ gboolean configuration_open_files()
 			if (i < 0) break;
 		}
 	}
+	document_colourise_new();
 
 	g_ptr_array_free(session_files, TRUE);
 	if (failure)
