@@ -18,9 +18,13 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: msgwindow.c 1269 2007-02-12 17:21:57Z eht16 $
+ * $Id: msgwindow.c 1376 2007-03-09 13:52:26Z eht16 $
  */
 
+/*
+ * Message window functions (status, compiler, messages windows).
+ * Also compiler error message parsing and grep file and line parsing.
+ */
 
 #include <time.h>
 
@@ -125,7 +129,7 @@ static void prepare_msg_tree_view(void)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
-	GtkTreeSelection *select;
+	GtkTreeSelection *selection;
 	PangoFontDescription *pfd;
 
 	msgwindow.store_msg = gtk_list_store_new(4, G_TYPE_INT, G_TYPE_INT, GDK_TYPE_COLOR, G_TYPE_STRING);
@@ -146,9 +150,9 @@ static void prepare_msg_tree_view(void)
 					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_MESSAGE));
 
 	// selection handling
-	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgwindow.tree_msg));
-	gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
-	//g_signal_connect(G_OBJECT(select), "changed", G_CALLBACK(on_msg_tree_selection_changed), NULL);
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgwindow.tree_msg));
+	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
+	//g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(on_msg_tree_selection_changed), NULL);
 }
 
 
@@ -157,7 +161,7 @@ static void prepare_compiler_tree_view(void)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
-	GtkTreeSelection *select;
+	GtkTreeSelection *selection;
 	PangoFontDescription *pfd;
 
 	msgwindow.store_compiler = gtk_list_store_new(2, GDK_TYPE_COLOR, G_TYPE_STRING);
@@ -178,9 +182,9 @@ static void prepare_compiler_tree_view(void)
 					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_COMPILER));
 
 	// selection handling
-	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgwindow.tree_compiler));
-	gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
-	//g_signal_connect(G_OBJECT(select), "changed", G_CALLBACK(on_msg_tree_selection_changed), NULL);
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgwindow.tree_compiler));
+	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
+	//g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(on_msg_tree_selection_changed), NULL);
 }
 
 
@@ -244,6 +248,19 @@ void msgwin_show_hide(gboolean show)
 }
 
 
+void msgwin_msg_add_fmt(gint line, gint idx, const gchar *format, ...)
+{
+	gchar string[512];
+	va_list args;
+
+	va_start(args, format);
+	g_vsnprintf(string, 512, format, args);
+	va_end(args);
+
+	msgwin_msg_add(line, idx, string);
+}
+
+
 // adds string to the msg treeview
 void msgwin_msg_add(gint line, gint idx, const gchar *string)
 {
@@ -423,6 +440,32 @@ void msgwin_menu_add_common_items(GtkMenu *menu)
 }
 
 
+/* look back up from the current path and find the directory we came from */
+static gboolean
+find_prev_build_dir(GtkTreePath *cur, GtkTreeModel *model, gchar **prefix)
+{
+	GtkTreeIter iter;
+	*prefix = NULL;
+
+	while (gtk_tree_path_prev(cur))
+	{
+		if (gtk_tree_model_get_iter(model, &iter, cur))
+		{
+			gchar *string;
+			gtk_tree_model_get(model, &iter, 1, &string, -1);
+			if (string != NULL && build_parse_make_dir(string, prefix))
+			{
+				g_free(string);
+				return TRUE;
+			}
+			g_free(string);
+		}
+	}
+
+	return FALSE;
+}
+
+
 gboolean msgwin_goto_compiler_file_line()
 {
 	GtkTreeIter iter;
@@ -449,8 +492,17 @@ gboolean msgwin_goto_compiler_file_line()
 		{
 			gint line;
 			gint idx;
-			gchar *filename;
-			msgwin_parse_compiler_error_line(string, &filename, &line);
+			gchar *filename, *dir;
+			GtkTreePath *path;
+
+			path = gtk_tree_model_get_path(model, &iter);
+			find_prev_build_dir(path, model, &dir);
+			gtk_tree_path_free(path);
+			msgwin_parse_compiler_error_line(string, dir, &filename, &line);
+
+			if (dir != NULL)
+				g_free(dir);
+
 			if (filename != NULL && line > -1)
 			{
 				gchar *utf8_filename = utils_get_utf8_from_locale(filename);
@@ -540,22 +592,21 @@ static void parse_file_line(ParseData *data, gchar **filename, gint *line)
  * relevant file with the error in *filename.
  * *line will be -1 if no error was found in string.
  * *filename must be freed unless it is NULL. */
-void msgwin_parse_compiler_error_line(const gchar *string, gchar **filename, gint *line)
+void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir, gchar **filename, gint *line)
 {
 	ParseData data = {string, build_info.dir, NULL, 0, 0, 0};
 
 	*filename = NULL;
 	*line = -1;
 
-	g_return_if_fail(build_info.dir != NULL);
+	if (dir != NULL)
+		data.dir = dir;
+
+	g_return_if_fail(build_info.dir != NULL || dir != NULL);
 	if (string == NULL) return;
 
 	switch (build_info.file_type_id)
 	{
-		case GEANY_FILETYPES_ALL:
-		{
-			return;
-		}
 		case GEANY_FILETYPES_PHP:
 		{
 			// Parse error: parse error, unexpected T_CASE in brace_bug.php on line 3
@@ -675,6 +726,7 @@ void msgwin_parse_compiler_error_line(const gchar *string, gchar **filename, gin
 		case GEANY_FILETYPES_LATEX:
 			// ./kommtechnik_2b.tex:18: Emergency stop.
 		case GEANY_FILETYPES_MAKE:	// Assume makefile is building with gcc
+		case GEANY_FILETYPES_ALL:
 		default:	// The default is a GNU gcc type error
 		{
 			// don't accidently find libtool versions x:y:x and think it is a file name
@@ -743,8 +795,7 @@ static void msgwin_parse_grep_line(const gchar *string, gchar **filename, gint *
 	*filename = NULL;
 	*line = -1;
 
-	g_return_if_fail(msgwindow.find_in_files_dir != NULL);
-	if (string == NULL) return;
+	if (string == NULL || msgwindow.find_in_files_dir == NULL) return;
 
 	// conflict:3:conflicting types for `foo'
 	data.string = string;
