@@ -1,8 +1,8 @@
 /*
  *      notebook.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
- *      Copyright 2006 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,11 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: notebook.c 790 2006-09-05 14:24:47Z ntrel $
+ * $Id: notebook.c 1394 2007-03-14 12:11:50Z ntrel $
+ */
+
+/*
+ * Notebook tab Drag 'n' Drop reordering and tab management.
  */
 
 #include "geany.h"
@@ -26,6 +30,7 @@
 #include "document.h"
 #include "ui_utils.h"
 #include "treeviews.h"
+#include "support.h"
 
 #define GEANY_DND_NOTEBOOK_TAB_TYPE	"geany_dnd_notebook_tab"
 
@@ -34,10 +39,17 @@ static const GtkTargetEntry drag_targets[] =
 	{GEANY_DND_NOTEBOOK_TAB_TYPE, GTK_TARGET_SAME_APP | GTK_TARGET_SAME_WIDGET, 0}
 };
 
+static GtkTargetEntry files_drop_targets[] = {
+	{ "STRING",			0, 0 },
+	{ "UTF8_STRING",	0, 0 },
+	{ "text/plain",		0, 0 },
+	{ "text/uri-list",	0, 0 }
+};
+
 
 static gboolean
 notebook_drag_motion_cb(GtkWidget *widget, GdkDragContext *dc,
-	gint x, gint y, guint time, gpointer user_data);
+	gint x, gint y, guint event_time, gpointer user_data);
 
 static void
 notebook_page_reordered_cb(GtkNotebook *notebook, GtkWidget *child, guint page_num,
@@ -49,6 +61,11 @@ notebook_motion_notify_event_cb(GtkWidget *widget, GdkEventMotion *event,
 	gpointer user_data);
 #endif
 
+static void
+on_window_drag_data_received(GtkWidget *widget, GdkDragContext *drag_context,
+                             gint x, gint y, GtkSelectionData *data, guint info,
+                             guint event_time, gpointer user_data);
+
 static gint
 notebook_find_tab_num_at_pos(GtkNotebook *notebook, gint x, gint y);
 
@@ -58,8 +75,25 @@ notebook_tab_close_clicked_cb(GtkButton *button, gpointer user_data);
 static void setup_tab_dnd();
 
 
+static void focus_sci(GtkWidget *widget, gpointer user_data)
+{
+	gint idx = document_get_cur_idx();
+
+	if (! DOC_IDX_VALID(idx)) return;
+
+	gtk_widget_grab_focus(GTK_WIDGET(doc_list[idx].sci));
+}
+
+
 void notebook_init()
 {
+	// focus the current document after clicking on a tab
+	g_signal_connect_after(G_OBJECT(app->notebook), "button-release-event",
+		G_CALLBACK(focus_sci), NULL);
+
+	g_signal_connect(G_OBJECT(app->notebook), "drag-data-received",
+		G_CALLBACK(on_window_drag_data_received), NULL);
+
 	setup_tab_dnd();
 }
 
@@ -148,35 +182,36 @@ static void
 notebook_page_reordered_cb(GtkNotebook *notebook, GtkWidget *child, guint page_num,
 	gpointer user_data)
 {
-	treeviews_openfiles_update_all();
+	/* Not necessary to update open files treeview if it's sorted.
+	 * Note: if enabled, it's best to move the item instead of recreating all items. */
+	//treeviews_openfiles_update_all();
 }
 
 
 static gboolean
 notebook_drag_motion_cb(GtkWidget *widget, GdkDragContext *dc,
-	gint x, gint y, guint time, gpointer user_data)
+	gint x, gint y, guint event_time, gpointer user_data)
 {
-	GtkNotebook *notebook = GTK_NOTEBOOK(widget);
 	static gint oldx, oldy; // for determining direction of mouse drag
+	GtkNotebook *notebook = GTK_NOTEBOOK(widget);
 	gint ndest = notebook_find_tab_num_at_pos(notebook, x, y);
 	gint ncurr = gtk_notebook_get_current_page(notebook);
 
-	if (ndest >= 0)
-	if (ndest != ncurr)
+	if (ndest >= 0 && ndest != ncurr)
 	{
 		gboolean ok = FALSE;
 		// prevent oscillation between non-homogeneous sized tabs
 		switch(gtk_notebook_get_tab_pos(notebook))
 		{
-		case GTK_POS_LEFT:
-		case GTK_POS_RIGHT:
-		ok = ((ndest > ncurr) && (y > oldy)) || ((ndest < ncurr) && (y < oldy));
-		break;
+			case GTK_POS_LEFT:
+			case GTK_POS_RIGHT:
+			ok = ((ndest > ncurr) && (y > oldy)) || ((ndest < ncurr) && (y < oldy));
+			break;
 
-		case GTK_POS_TOP:
-		case GTK_POS_BOTTOM:
-		ok = ((ndest > ncurr) && (x > oldx)) || ((ndest < ncurr) && (x < oldx));
-		break;
+			case GTK_POS_TOP:
+			case GTK_POS_BOTTOM:
+			ok = ((ndest > ncurr) && (x > oldx)) || ((ndest < ncurr) && (x < oldx));
+			break;
 		}
 
 		if (ok)
@@ -242,16 +277,44 @@ notebook_find_tab_num_at_pos(GtkNotebook *notebook, gint x, gint y)
 }
 
 
-/* Returns index of notebook page, or -1 on error */
-gint notebook_new_tab(gint doc_idx, gchar *title, GtkWidget *page)
+// call this after the number of tabs in app->notebook changes.
+static void tab_count_changed()
+{
+	switch (gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)))
+	{
+		case 0:
+		/* Enables DnD for dropping files into the empty notebook widget */
+		gtk_drag_dest_set(app->notebook, GTK_DEST_DEFAULT_ALL,
+			files_drop_targets,	G_N_ELEMENTS(files_drop_targets),
+			GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
+		break;
+
+		case 1:
+		/* Disables DnD for dropping files into the notebook widget and enables the DnD for moving file
+		 * tabs. Files can still be dropped into the notebook widget because it will be handled by the
+		 * active Scintilla Widget (only dropping to the tab bar is not possible but it should be ok) */
+		gtk_drag_dest_set(app->notebook, GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
+			drag_targets, G_N_ELEMENTS(drag_targets), GDK_ACTION_MOVE);
+		break;
+	}
+}
+
+
+/* Returns page number of notebook page, or -1 on error */
+gint notebook_new_tab(gint doc_idx)
 {
 	GtkWidget *hbox, *but;
 	GtkWidget *align;
 	gint tabnum;
+	gchar *title;
 	document *this = &(doc_list[doc_idx]);
+	GtkWidget *page;
 
 	g_return_val_if_fail(doc_idx >= 0 && this != NULL, -1);
 
+	page = GTK_WIDGET(this->sci);
+	title = g_path_get_basename(DOC_FILENAME(doc_idx));
+
 	this->tab_label = gtk_label_new(title);
 
 	hbox = gtk_hbox_new(FALSE, 0);
@@ -273,11 +336,13 @@ gint notebook_new_tab(gint doc_idx, gchar *title, GtkWidget *page)
 	gtk_misc_set_alignment(GTK_MISC(this->tabmenu_label), 0.0, 0);
 
 	if (app->tab_order_ltr)
-		tabnum = gtk_notebook_append_page_menu(GTK_NOTEBOOK(app->notebook),
-			GTK_WIDGET(page), hbox, this->tabmenu_label);
+		tabnum = gtk_notebook_append_page_menu(GTK_NOTEBOOK(app->notebook), page,
+			hbox, this->tabmenu_label);
 	else
-		tabnum = gtk_notebook_insert_page_menu(GTK_NOTEBOOK(app->notebook),
-			GTK_WIDGET(page), hbox, this->tabmenu_label, 0);
+		tabnum = gtk_notebook_insert_page_menu(GTK_NOTEBOOK(app->notebook), page,
+			hbox, this->tabmenu_label, 0);
+
+	tab_count_changed();
 
 	// signal for clicking the tab-close button
 	g_signal_connect(G_OBJECT(but), "clicked",
@@ -290,6 +355,7 @@ gint notebook_new_tab(gint doc_idx, gchar *title, GtkWidget *page)
 		gtk_notebook_set_tab_reorderable(GTK_NOTEBOOK(app->notebook), page, TRUE);
 	}
 #endif
+	g_free(title);
 	return tabnum;
 }
 
@@ -301,3 +367,35 @@ notebook_tab_close_clicked_cb(GtkButton *button, gpointer user_data)
 		GTK_WIDGET(user_data));
 	document_remove(cur_page);
 }
+
+
+// Always use this instead of gtk_notebook_remove_page().
+void notebook_remove_page(gint page_num)
+{
+	gtk_notebook_remove_page(GTK_NOTEBOOK(app->notebook), page_num);
+	tab_count_changed();
+}
+
+
+static void
+on_window_drag_data_received(GtkWidget *widget, GdkDragContext *drag_context,
+                             gint x, gint y, GtkSelectionData *data, guint target_type,
+                             guint event_time, gpointer user_data)
+{
+	gboolean success = FALSE;
+
+	if (data->length > 0 && data->format == 8)
+	{
+		if (drag_context->action == GDK_ACTION_ASK)
+		{
+			drag_context->action = GDK_ACTION_COPY;
+		}
+
+		document_open_file_list((const gchar *)data->data, data->length);
+
+		success = TRUE;
+	}
+	gtk_drag_finish(drag_context, success, FALSE, event_time);
+}
+
+
