@@ -1,7 +1,8 @@
 /*
  *      prefs.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -31,7 +32,7 @@ void on_prefs_font_choosed(GtkFontButton *widget, gpointer user_data);
 
 void on_prefs_color_choosed(GtkColorButton *widget, gpointer user_data);
 
-void dialogs_show_prefs_dialog(void);
+void prefs_show_dialog(void);
 
 void on_prefs_tools_button_clicked(GtkButton *button, gpointer user_data);
 
