@@ -18,7 +18,12 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: sciwrappers.c 1270 2007-02-12 18:42:15Z eht16 $
+ * $Id: sciwrappers.c 1514 2007-05-08 16:03:07Z eht16 $
+ */
+
+/*
+ * Wrappers for the SCI_* Scintilla messages.
+ * Originally from the cssed project (http://cssed.sf.net).
  */
 
 #include <string.h>
@@ -321,38 +326,41 @@ gboolean sci_is_marker_set_at_line(ScintillaObject* sci, gint line, gint marker)
 {
 	gint state;
 
-	state = SSM( sci, SCI_MARKERGET, line, marker );
-	return(!(state & (1 << marker)));
+	state = SSM( sci, SCI_MARKERGET, line, 0 );
+	return (state & (1 << marker));
 }
 
 
-gboolean sci_marker_next(ScintillaObject* sci, gint line, gint marker_mask)
+/* Returns the line number of the next marker that matches marker_mask, or -1.
+ * marker_mask is a bitor of 1 << marker_index. (See MarkerHandleSet::MarkValue()).
+ * Note: If there is a marker on the line, it returns the same line. */
+gint sci_marker_next(ScintillaObject* sci, gint line, gint marker_mask, gboolean wrap)
 {
 	gint marker_line;
 
 	marker_line = SSM(sci, SCI_MARKERNEXT, line, marker_mask);
-
-	if( marker_line != -1 ){
-		SSM(sci,SCI_GOTOLINE,marker_line,0);
-		return TRUE;
-	}else{
-		return FALSE;
-	}
+	if (wrap && marker_line == -1)
+		marker_line = SSM(sci, SCI_MARKERNEXT, 0, marker_mask);
+	return marker_line;
 }
 
 
-gboolean sci_marker_prev(ScintillaObject* sci, gint line, gint marker_mask)
+/* Returns the line number of the previous marker that matches marker_mask, or -1.
+ * marker_mask is a bitor of 1 << marker_index. (See MarkerHandleSet::MarkValue()).
+ * Note: If there is a marker on the line, it returns the same line. */
+gint sci_marker_previous(ScintillaObject* sci, gint line, gint marker_mask, gboolean wrap)
 {
 	gint marker_line;
 
 	marker_line = SSM(sci, SCI_MARKERPREVIOUS, line, marker_mask);
+	if (wrap && marker_line == -1)
+	{
+		gint len = sci_get_length(sci);
+		gint last_line = sci_get_line_from_position(sci, len - 1);
 
-	if( marker_line != -1 ){
-		SSM(sci,SCI_GOTOLINE,marker_line,0);
-		return TRUE;
-	}else{
-		return FALSE;
+		marker_line = SSM(sci, SCI_MARKERPREVIOUS, last_line, marker_mask);
 	}
+	return marker_line;
 }
 
 
@@ -416,9 +424,9 @@ void sci_set_selection_end(ScintillaObject* sci, gint position)
 }
 
 
-gint sci_get_line_end_from_position(ScintillaObject* sci, gint position)
+gint sci_get_line_end_position(ScintillaObject* sci, gint line)
 {
-	return SSM(sci, SCI_GETLINEENDPOSITION, position, 0);
+	return SSM(sci, SCI_GETLINEENDPOSITION, line, 0);
 }
 
 
@@ -902,3 +910,44 @@ void sci_set_tab_indents(ScintillaObject *sci, gboolean set)
 	SSM(sci, SCI_SETTABINDENTS, set, 0);
 }
 
+void sci_set_use_tabs(ScintillaObject *sci, gboolean set)
+{
+	SSM(sci, SCI_SETUSETABS, set, 0);
+}
+
+gint sci_get_pos_at_line_sel_start(ScintillaObject *sci, gint line)
+{
+	return SSM(sci, SCI_GETLINESELSTARTPOSITION, line, 0);
+}
+
+gint sci_get_pos_at_line_sel_end(ScintillaObject *sci, gint line)
+{
+	return SSM(sci, SCI_GETLINESELENDPOSITION, line, 0);
+}
+
+gint sci_get_selection_mode(ScintillaObject *sci)
+{
+	return SSM(sci, SCI_GETSELECTIONMODE, 0, 0);
+}
+
+void sci_set_selection_mode(ScintillaObject *sci, gint mode)
+{
+	SSM(sci, SCI_SETSELECTIONMODE, mode, 0);
+}
+
+void sci_set_scrollbar_mode(ScintillaObject *sci, gboolean visible)
+{
+	SSM(sci, SCI_SETHSCROLLBAR, visible, 0);
+	SSM(sci, SCI_SETVSCROLLBAR, visible, 0);
+}
+
+void sci_set_line_indentation(ScintillaObject *sci, gint line, gint indent)
+{
+	SSM(sci, SCI_SETLINEINDENTATION, line, indent);
+}
+
+int sci_get_line_indentation(ScintillaObject *sci, gint line)
+{
+	return SSM(sci, SCI_GETLINEINDENTATION, line, 0);
+}
+
