@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: sciwrappers.h 1270 2007-02-12 18:42:15Z eht16 $
+ * $Id: sciwrappers.h 1514 2007-05-08 16:03:07Z eht16 $
  */
 
 #ifndef GEANY_SCIWRAPPERS_H
@@ -62,8 +62,8 @@ void 				sci_zoom_out				(ScintillaObject* sci);
 void 				sci_zoom_off				(ScintillaObject* sci);
 void				sci_set_marker_at_line		(ScintillaObject* sci, gint line_number, gboolean set, gint marker );
 gboolean 			sci_is_marker_set_at_line	(ScintillaObject* sci, gint line, gint marker);
-gboolean 			sci_marker_next				(ScintillaObject* sci, gint line, gint marker_mask);
-gboolean 			sci_marker_prev				(ScintillaObject* sci, gint line, gint marker_mask);
+gint				sci_marker_next				(ScintillaObject* sci, gint line, gint marker_mask, gboolean wrap);
+gint				sci_marker_previous			(ScintillaObject* sci, gint line, gint marker_mask, gboolean wrap);
 
 gint 				sci_get_col_from_position	(ScintillaObject* sci, gint position);
 gint 				sci_get_line_from_position	(ScintillaObject* sci, gint position);
@@ -80,6 +80,12 @@ void 				sci_clear					(ScintillaObject* sci);
 gint				sci_get_selection_start		(ScintillaObject* sci);
 gint				sci_get_selection_end		(ScintillaObject* sci);
 void 				sci_replace_sel				(ScintillaObject* sci, gchar* text);
+gint				sci_get_selection_mode		(ScintillaObject* sci);
+void				sci_set_selection_mode		(ScintillaObject* sci, gint mode);
+gint				sci_get_pos_at_line_sel_start(ScintillaObject*sci, gint line);
+gint				sci_get_pos_at_line_sel_end	(ScintillaObject* sci, gint line);
+void 				sci_set_selection_start		(ScintillaObject* sci, gint position);
+void				sci_set_selection_end		(ScintillaObject* sci, gint position);
 
 gint				sci_get_length				(ScintillaObject* sci);
 void				sci_get_text				(ScintillaObject* sci,gint len,gchar* text);
@@ -87,16 +93,14 @@ void				sci_get_selected_text		(ScintillaObject* sci, gchar* text);
 gint				sci_get_selected_text_length(ScintillaObject* sci);
 gchar *				sci_get_line				(ScintillaObject* sci, gint line_num);
 gint 				sci_get_line_length			(ScintillaObject* sci, gint line);
-gint				sci_get_line_count			( ScintillaObject* sci );
+gint				sci_get_line_count			(ScintillaObject* sci);
 void 				sci_get_xy_from_position	(ScintillaObject* sci,gint pos, gint* x, gint* y);
 gint				sci_get_position_from_xy	(ScintillaObject* sci, gint x, gint y, gboolean nearby);
 
 void 				sci_set_undo_collection		(ScintillaObject* sci, gboolean set);
 gboolean			sci_get_undo_collection		(ScintillaObject* sci);
 
-void 				sci_set_selection_start		(ScintillaObject* sci, gint position);
-void				sci_set_selection_end		(ScintillaObject* sci, gint position);
-gint				sci_get_line_end_from_position	(ScintillaObject* sci, gint position);
+gint				sci_get_line_end_position	(ScintillaObject* sci, gint line);
 
 void 				sci_toggle_fold				(ScintillaObject* sci, gint line);
 gboolean			sci_get_line_is_visible		(ScintillaObject* sci, gint line);
@@ -147,6 +151,7 @@ void				sci_target_from_selection	(ScintillaObject * sci);
 void				sci_target_start			(ScintillaObject * sci, gint start);
 void				sci_target_end				(ScintillaObject * sci, gint end);
 gint				sci_target_replace			(ScintillaObject * sci, const gchar *text, gboolean regex);
+
 void				sci_set_keywords			(ScintillaObject * sci, gint k, gchar *text);
 gint				sci_get_lexer				(ScintillaObject * sci);
 void				sci_set_readonly			(ScintillaObject * sci, gboolean readonly);
@@ -162,10 +167,15 @@ void				sci_set_styling				(ScintillaObject * sci, gint len, gint style);
 void				sci_start_styling			(ScintillaObject * sci, gint pos, gint mask);
 void				sci_select_all				(ScintillaObject * sci);
 gint				sci_get_line_indent_position(ScintillaObject * sci, gint line);
+void				sci_set_line_indentation	(ScintillaObject * sci, gint line, gint indent);
+int					sci_get_line_indentation	(ScintillaObject * sci, gint line);
 void				sci_set_autoc_max_height	(ScintillaObject * sci, gint val);
 gint				sci_find_bracematch			(ScintillaObject * sci, gint pos);
 
 gint				sci_get_overtype			(ScintillaObject * sci);
 void				sci_set_tab_indents			(ScintillaObject * sci, gboolean set);
+void				sci_set_use_tabs			(ScintillaObject * sci, gboolean set);
+
+void				sci_set_scrollbar_mode		(ScintillaObject * sci, gboolean visible);
 
 #endif
