@@ -1,8 +1,8 @@
 /*
  *      search.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
- *      Copyright 2006 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,12 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: search.c 1252 2007-02-06 17:39:19Z ntrel $
+ * $Id: search.c 1541 2007-05-16 17:09:02Z eht16 $
+ */
+
+/*
+ * Find, Replace, Find in Files dialog related functions.
+ * Note that the basic text find functions are in document.c.
  */
 
 #include <gdk/gdkkeysyms.h>
@@ -56,23 +61,10 @@ enum {
 };
 
 
-typedef enum
-{
-	FIF_CASE_SENSITIVE	= 1 << 0,
-	FIF_WHOLE_WORD		= 1 << 1,
-	FIF_INVERT_MATCH	= 1 << 2
-} fif_options;
-
-typedef enum
-{
-	FIF_FGREP,
-	FIF_GREP,
-	FIF_EGREP
-} fif_match_type;
-
-
 GeanySearchData search_data;
 
+SearchPrefs search_prefs = {NULL};
+
 static struct
 {
 	GtkWidget	*find_dialog;
@@ -89,8 +81,6 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data);
 
 static gchar **search_get_argv(const gchar **argv_prefix, const gchar *dir);
 
-static GSList *search_get_file_list(const gchar *path, guint *length);
-
 
 static void
 on_find_replace_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user_data);
@@ -114,10 +104,7 @@ static void
 on_find_in_files_dialog_response(GtkDialog *dialog, gint response, gpointer user_data);
 
 static gboolean
-search_find_in_files(const gchar *search_text, const gchar *dir, fif_match_type regex_opt,
-		fif_options opts);
-
-static void on_open_dir_dialog_clicked(GtkButton *button, gpointer user_data);
+search_find_in_files(const gchar *search_text, const gchar *dir, const gchar *opts);
 
 
 void search_init()
@@ -233,19 +220,67 @@ static gchar *get_default_text(gint idx)
 	{	// use the word at current cursor position
 		gchar word[GEANY_MAX_WORD_LENGTH];
 
-		sci_cb_find_current_word(doc_list[idx].sci, -1, word, sizeof(word));
+		sci_cb_find_current_word(doc_list[idx].sci, -1, word, sizeof(word), NULL);
 		if (word[0] != '\0') s = g_strdup(word);
 	}
 	return s;
 }
 
 
+// store text, clear search flags so we can use Search->Find Next/Previous
+static void setup_find_next(const gchar *text)
+{
+	g_free(search_data.text);
+	search_data.text = g_strdup(text);
+	search_data.flags = 0;
+	search_data.backwards = FALSE;
+}
+
+
+/* Search for next match of the current "selection"
+ * For X11 based systems, this will try to use the system-wide
+ * x-selection first. If it doesn't find anything suitable in
+ * the x-selection (or if we are on Win32) it will try to use
+ * the scintilla selection or current token instead.
+ * Search flags are always zero.
+ */
+void search_find_selection(gint idx, gboolean search_backwards)
+{
+	gchar *s = NULL;
+#ifdef G_OS_UNIX
+	GtkClipboard *clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+#endif
+
+	g_return_if_fail(DOC_IDX_VALID(idx));
+
+#ifdef G_OS_UNIX
+	s=gtk_clipboard_wait_for_text(clipboard);
+	if (s)
+	{
+		if (strchr(s,'\n') || strchr(s, '\r'))
+		{
+			g_free(s);
+			s=NULL;
+		};
+	}
+#endif
+
+	if (!s)	{ s=get_default_text(idx); }
+	if (s)
+	{
+		setup_find_next(s);	// allow find next/prev
+		document_find_text(idx, s, 0, search_backwards, TRUE);
+		g_free(s);
+	}
+}
+
+
 void search_show_find_dialog()
 {
 	gint idx = document_get_cur_idx();
 	gchar *sel = NULL;
 
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
+	g_return_if_fail(DOC_IDX_VALID(idx));
 
 	sel = get_default_text(idx);
 
@@ -259,6 +294,7 @@ void search_show_find_dialog()
 			GTK_WINDOW(app->window), GTK_DIALOG_DESTROY_WITH_PARENT,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL, NULL);
 		vbox = ui_dialog_vbox_new(GTK_DIALOG(widgets.find_dialog));
+		gtk_widget_set_name(widgets.find_dialog, "GeanyDialogSearch");
 		gtk_box_set_spacing(GTK_BOX(vbox), 9);
 
 		button = ui_button_new_with_image(GTK_STOCK_GO_BACK, _("_Previous"));
@@ -300,12 +336,12 @@ void search_show_find_dialog()
 		exp = gtk_expander_new(_("Find All"));
 		bbox = gtk_hbutton_box_new();
 
-#if 0
 		button = gtk_button_new_with_mnemonic(_("_Mark"));
+		gtk_tooltips_set_tip(tooltips, button,
+				_("Mark all matches in the current document."), NULL);
 		gtk_container_add(GTK_CONTAINER(bbox), button);
 		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(send_find_dialog_response),
 			GINT_TO_POINTER(GEANY_RESPONSE_MARK));
-#endif	// not implemented yet
 
 		button = gtk_button_new_with_mnemonic(_("In Sessi_on"));
 		gtk_container_add(GTK_CONTAINER(bbox), button);
@@ -378,6 +414,7 @@ void search_show_replace_dialog()
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL, NULL);
 		vbox = ui_dialog_vbox_new(GTK_DIALOG(widgets.replace_dialog));
 		gtk_box_set_spacing(GTK_BOX(vbox), 9);
+		gtk_widget_set_name(widgets.replace_dialog, "GeanyDialogSearch");
 
 		button = gtk_button_new_from_stock(GTK_STOCK_FIND);
 		gtk_dialog_add_action_widget(GTK_DIALOG(widgets.replace_dialog), button,
@@ -491,12 +528,19 @@ void search_show_replace_dialog()
 }
 
 
+static void on_extra_options_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+{
+	// disable extra option entry when checkbutton not checked
+	gtk_widget_set_sensitive(GTK_WIDGET(user_data),
+		gtk_toggle_button_get_active(togglebutton));
+}
+
+
 void search_show_find_in_files_dialog()
 {
 	static GtkWidget *combo = NULL;
 	static GtkWidget *dir_combo;
 	GtkWidget *entry; // the child GtkEntry of combo (or dir_combo)
-	GtkWidget *dirbtn, *openimg;
 	gint idx = document_get_cur_idx();
 	gchar *sel = NULL;
 	gchar *cur_dir;
@@ -505,7 +549,8 @@ void search_show_find_in_files_dialog()
 
 	if (widgets.find_in_files_dialog == NULL)
 	{
-		GtkWidget *label, *label1, *checkbox1, *checkbox2, *check_wholeword;
+		GtkWidget *label, *label1, *checkbox1, *checkbox2, *check_wholeword,
+			*check_recursive, *check_extra, *entry_extra;
 		GtkWidget *dbox, *sbox, *cbox, *rbox, *rbtn, *hbox, *vbox;
 		GtkSizeGroup *size_group;
 		GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
@@ -515,6 +560,7 @@ void search_show_find_in_files_dialog()
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
 		vbox = ui_dialog_vbox_new(GTK_DIALOG(widgets.find_in_files_dialog));
 		gtk_box_set_spacing(GTK_BOX(vbox), 9);
+		gtk_widget_set_name(widgets.find_in_files_dialog, "GeanyDialogSearch");
 
 		gtk_dialog_add_button(GTK_DIALOG(widgets.find_in_files_dialog), "gtk-find", GTK_RESPONSE_ACCEPT);
 		gtk_dialog_set_default_response(GTK_DIALOG(widgets.find_in_files_dialog),
@@ -530,16 +576,9 @@ void search_show_find_in_files_dialog()
 		g_object_set_data_full(G_OBJECT(widgets.find_in_files_dialog), "dir_combo",
 						gtk_widget_ref(dir_combo), (GDestroyNotify)gtk_widget_unref);
 
-		dirbtn = gtk_button_new();
-		openimg = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON);
-		gtk_container_add(GTK_CONTAINER(dirbtn), openimg);
-		g_signal_connect(G_OBJECT(dirbtn), "clicked", G_CALLBACK(on_open_dir_dialog_clicked),
-			NULL);
-
-		dbox = gtk_hbox_new(FALSE, 6);
+		dbox = ui_path_box_new(NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+			GTK_ENTRY(entry));
 		gtk_box_pack_start(GTK_BOX(dbox), label1, FALSE, FALSE, 0);
-		gtk_box_pack_start(GTK_BOX(dbox), dir_combo, TRUE, TRUE, 0);
-		gtk_box_pack_start(GTK_BOX(dbox), dirbtn, FALSE, FALSE, 0);
 
 		label = gtk_label_new(_("Search for:"));
 		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
@@ -584,6 +623,11 @@ void search_show_find_in_files_dialog()
 		gtk_button_set_focus_on_click(GTK_BUTTON(rbtn), FALSE);
 		gtk_container_add(GTK_CONTAINER(rbox), rbtn);
 
+		check_recursive = gtk_check_button_new_with_mnemonic(_("_Recurse in subfolders"));
+		g_object_set_data_full(G_OBJECT(widgets.find_in_files_dialog), "check_recursive",
+						gtk_widget_ref(check_recursive), (GDestroyNotify)gtk_widget_unref);
+		gtk_button_set_focus_on_click(GTK_BUTTON(check_recursive), FALSE);
+
 		checkbox1 = gtk_check_button_new_with_mnemonic(_("_Case sensitive"));
 		g_object_set_data_full(G_OBJECT(widgets.find_in_files_dialog), "check_case",
 						gtk_widget_ref(checkbox1), (GDestroyNotify)gtk_widget_unref);
@@ -606,6 +650,7 @@ void search_show_find_in_files_dialog()
 		gtk_container_add(GTK_CONTAINER(cbox), checkbox1);
 		gtk_container_add(GTK_CONTAINER(cbox), check_wholeword);
 		gtk_container_add(GTK_CONTAINER(cbox), checkbox2);
+		gtk_container_add(GTK_CONTAINER(cbox), check_recursive);
 
 		hbox = gtk_hbox_new(FALSE, 6);
 		gtk_container_add(GTK_CONTAINER(hbox), rbox);
@@ -615,6 +660,29 @@ void search_show_find_in_files_dialog()
 		gtk_box_pack_start(GTK_BOX(vbox), sbox, TRUE, FALSE, 0);
 		gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
+		check_extra = gtk_check_button_new_with_mnemonic(_("E_xtra options:"));
+		g_object_set_data_full(G_OBJECT(widgets.find_in_files_dialog), "check_extra",
+						gtk_widget_ref(check_extra), (GDestroyNotify)gtk_widget_unref);
+		gtk_button_set_focus_on_click(GTK_BUTTON(check_extra), FALSE);
+
+		entry_extra = gtk_entry_new();
+		if (search_prefs.fif_extra_options)
+			gtk_entry_set_text(GTK_ENTRY(entry_extra), search_prefs.fif_extra_options);
+		gtk_widget_set_sensitive(entry_extra, FALSE);
+		g_object_set_data_full(G_OBJECT(widgets.find_in_files_dialog), "entry_extra",
+						gtk_widget_ref(entry_extra), (GDestroyNotify)gtk_widget_unref);
+		gtk_tooltips_set_tip(tooltips, entry_extra,
+				_("Other options to pass to Grep"), NULL);
+
+		// enable entry_extra when check_extra is checked
+		g_signal_connect(G_OBJECT(check_extra), "toggled",
+			G_CALLBACK(on_extra_options_toggled), entry_extra);
+
+		hbox = gtk_hbox_new(FALSE, 6);
+		gtk_box_pack_start(GTK_BOX(hbox), check_extra, FALSE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(hbox), entry_extra, TRUE, TRUE, 0);
+		gtk_container_add(GTK_CONTAINER(vbox), hbox);
+
 		g_signal_connect((gpointer) dir_combo, "key-press-event",
 				G_CALLBACK(on_widget_key_pressed_set_focus), combo);
 		g_signal_connect((gpointer) widgets.find_in_files_dialog, "response",
@@ -654,37 +722,6 @@ void search_show_find_in_files_dialog()
 }
 
 
-static void on_open_dir_dialog_clicked(GtkButton *button, gpointer user_data)
-{
-	GtkWidget *dialog = gtk_file_chooser_dialog_new(_("Select folder"),
-		GTK_WINDOW(app->window), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
-		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
-	GtkWidget *dir_combo =
-		lookup_widget(widgets.find_in_files_dialog, "dir_combo");
-	GtkWidget *entry_dir = gtk_bin_get_child(GTK_BIN(dir_combo));
-	gchar *dir_locale;
-	const gchar *entry_text;
-
-	entry_text = gtk_entry_get_text(GTK_ENTRY(entry_dir));
-	dir_locale = utils_get_locale_from_utf8(entry_text);
-	if (g_file_test(dir_locale, G_FILE_TEST_IS_DIR))
-		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), dir_locale);
-	g_free(dir_locale);
-
-	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
-	{
-		gchar *dir_utf8;
-		dir_locale = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(dialog));
-		dir_utf8 = utils_get_utf8_from_locale(dir_locale);
-		g_free(dir_locale);
-		gtk_entry_set_text(GTK_ENTRY(entry_dir), dir_utf8);
-		g_free(dir_utf8);
-	}
-	gtk_widget_destroy(dialog);
-}
-
-
 static void
 on_find_replace_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 {
@@ -726,6 +763,31 @@ on_find_replace_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user
 }
 
 
+static gint search_mark(gint idx, const gchar *search_text, gint flags)
+{
+	gint pos, line, count = 0;
+	struct TextToFind ttf;
+
+	g_return_val_if_fail(DOC_IDX_VALID(idx), 0);
+
+	ttf.chrg.cpMin = 0;
+	ttf.chrg.cpMax = sci_get_length(doc_list[idx].sci);
+	ttf.lpstrText = (gchar *)search_text;
+	while (1)
+	{
+		pos = sci_find_text(doc_list[idx].sci, flags, &ttf);
+		if (pos == -1) break;
+
+		line = sci_get_line_from_position(doc_list[idx].sci, pos);
+		sci_set_marker_at_line(doc_list[idx].sci, line, TRUE, 1);
+
+		ttf.chrg.cpMin = ttf.chrgText.cpMax + 1;
+		count++;
+	}
+	return count;
+}
+
+
 static void
 on_find_entry_activate(GtkEntry *entry, gpointer user_data)
 {
@@ -737,7 +799,7 @@ on_find_entry_activate(GtkEntry *entry, gpointer user_data)
 static void
 on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 {
-	if (response == GTK_RESPONSE_CANCEL)
+	if (response == GTK_RESPONSE_CANCEL || response == GTK_RESPONSE_DELETE_EVENT)
 		gtk_widget_hide(widgets.find_dialog);
 	else
 	{
@@ -780,26 +842,33 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		{
 			case GEANY_RESPONSE_FIND:
 			case GEANY_RESPONSE_FIND_PREVIOUS:
-			document_find_text(idx, search_data.text, search_data.flags,
-				(response == GEANY_RESPONSE_FIND_PREVIOUS), TRUE);
-			check_close = FALSE;
-			if (app->pref_main_suppress_search_dialogs)
-				check_close = TRUE;
-			break;
+				document_find_text(idx, search_data.text, search_data.flags,
+					(response == GEANY_RESPONSE_FIND_PREVIOUS), TRUE);
+				check_close = FALSE;
+				if (app->pref_main_suppress_search_dialogs)
+					check_close = TRUE;
+				break;
 
 			case GEANY_RESPONSE_FIND_IN_FILE:
-			search_find_usage(search_data.text, search_data.flags, FALSE);
-			break;
+				search_find_usage(search_data.text, search_data.flags, FALSE);
+				break;
 
 			case GEANY_RESPONSE_FIND_IN_SESSION:
-			search_find_usage(search_data.text, search_data.flags, TRUE);
-			break;
+				search_find_usage(search_data.text, search_data.flags, TRUE);
+				break;
 
 			case GEANY_RESPONSE_MARK:
-			// TODO
-			break;
+				if (DOC_IDX_VALID(idx))
+				{
+					gint count = search_mark(idx, search_data.text, search_data.flags);
 
-			// Note: we can get GTK_RESPONSE_DELETE_EVENT responses when ESC is pressed
+					if (count == 0)
+						ui_set_statusbar(_("No matches found for '%s'."), search_data.text);
+					else
+						ui_set_statusbar(_("Found %d matches for '%s'."), count,
+							search_data.text);
+				}
+				break;
 		}
 		if (check_close)
 			gtk_widget_hide(widgets.find_dialog);
@@ -826,7 +895,7 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 	gboolean close_window;
 	gchar *find, *replace;
 
-	if (response == GTK_RESPONSE_CANCEL)
+	if (response == GTK_RESPONSE_CANCEL || response == GTK_RESPONSE_DELETE_EVENT)
 	{
 		gtk_widget_hide(widgets.replace_dialog);
 		return;
@@ -849,7 +918,7 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 	find = g_strdup(gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_find)))));
 	replace = g_strdup(gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_replace)))));
 
-	if ((response != GEANY_RESPONSE_FIND) && (! fl1) && (strcasecmp(find, replace) == 0))
+	if ((response != GEANY_RESPONSE_FIND) && fl1 && (strcasecmp(find, replace) == 0))
 	{
 		utils_beep();
 		gtk_widget_grab_focus(GTK_WIDGET(GTK_BIN(lookup_widget(widgets.replace_dialog, "entry_find"))->child));
@@ -907,14 +976,16 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 			// replace in all documents following notebook tab order
 			for (n = 0; (gint) n < gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)); n++)
 			{
-				gint idx = document_get_n_idx(n);
+				gint ix = document_get_n_idx(n);
 
-				if (! doc_list[idx].is_valid) continue;
+				if (! doc_list[ix].is_valid) continue;
 
-				if (document_replace_all(idx, find, replace, search_flags_re,
+				if (document_replace_all(ix, find, replace, search_flags_re,
 					search_replace_escape_re)) count++;
 			}
 			ui_set_statusbar(_("Replaced text in %u files."), count);
+			// show which docs had replacements:
+			gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_STATUS);
 
 			ui_save_buttons_toggle(doc_list[idx].changed);	// update save all
 			if (close_window) gtk_widget_hide(widgets.replace_dialog);
@@ -926,7 +997,6 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 			if (close_window) gtk_widget_hide(widgets.replace_dialog);
 			break;
 		}
-		// Note: we can get GTK_RESPONSE_DELETE_EVENT responses when ESC is pressed
 	}
 	g_free(find);
 	g_free(replace);
@@ -947,6 +1017,55 @@ on_widget_key_pressed_set_focus(GtkWidget *widget, GdkEventKey *event, gpointer
 }
 
 
+static GString *get_grep_options()
+{
+	gboolean fgrep = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+					lookup_widget(widgets.find_in_files_dialog, "radio_fgrep")));
+	gboolean grep = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+					lookup_widget(widgets.find_in_files_dialog, "radio_grep")));
+	gboolean invert = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+					lookup_widget(widgets.find_in_files_dialog, "check_invert")));
+	gboolean case_sens = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+					lookup_widget(widgets.find_in_files_dialog, "check_case")));
+	gboolean whole_word = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+					lookup_widget(widgets.find_in_files_dialog, "check_wholeword")));
+	gboolean recursive = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+					lookup_widget(widgets.find_in_files_dialog, "check_recursive")));
+	gboolean extra = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+					lookup_widget(widgets.find_in_files_dialog, "check_extra")));
+	GString *gstr = g_string_new("-nHI");	// line numbers, filenames, ignore binaries
+
+	if (invert)
+		g_string_append_c(gstr, 'v');
+	if (! case_sens)
+		g_string_append_c(gstr, 'i');
+	if (whole_word)
+		g_string_append_c(gstr, 'w');
+	if (recursive)
+		g_string_append_c(gstr, 'r');
+
+	if (fgrep)
+		g_string_append_c(gstr, 'F');
+	else if (! grep)
+		g_string_append_c(gstr, 'E');
+
+	if (extra)
+	{
+		gchar *text = g_strdup(gtk_entry_get_text(GTK_ENTRY(
+					lookup_widget(widgets.find_in_files_dialog, "entry_extra"))));
+
+		text = g_strstrip(text);
+		if (*text != 0)
+		{
+			g_string_append_c(gstr, ' ');
+			g_string_append(gstr, text);
+		}
+		g_free(text);
+	}
+	return gstr;
+}
+
+
 static void
 on_find_in_files_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 {
@@ -959,35 +1078,28 @@ on_find_in_files_dialog_response(GtkDialog *dialog, gint response, gpointer user
 		const gchar *utf8_dir =
 			gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(dir_combo))));
 
+		// update extra options pref
+		g_free(search_prefs.fif_extra_options);
+		search_prefs.fif_extra_options = g_strdup(gtk_entry_get_text(GTK_ENTRY(
+					lookup_widget(widgets.find_in_files_dialog, "entry_extra"))));
+
 		if (utf8_dir == NULL || utils_str_equal(utf8_dir, ""))
 			ui_set_statusbar(_("Invalid directory for find in files."));
 		else if (search_text && *search_text)
 		{
 			gchar *locale_dir;
-			gboolean invert = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-							lookup_widget(widgets.find_in_files_dialog, "check_invert")));
-			gboolean case_sens = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-							lookup_widget(widgets.find_in_files_dialog, "check_case")));
-			gboolean whole_word = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-							lookup_widget(widgets.find_in_files_dialog, "check_wholeword")));
-			gint opts = (invert ? FIF_INVERT_MATCH : 0) |
-						(case_sens ? FIF_CASE_SENSITIVE : 0) |
-						(whole_word ? FIF_WHOLE_WORD : 0);
-			gboolean fgrep = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-							lookup_widget(widgets.find_in_files_dialog, "radio_fgrep")));
-			gboolean grep = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-							lookup_widget(widgets.find_in_files_dialog, "radio_grep")));
-			gint regex_opt = fgrep ? FIF_FGREP : (grep ? FIF_GREP : FIF_EGREP);
+			GString *opts = get_grep_options();
 
 			locale_dir = utils_get_locale_from_utf8(utf8_dir);
 
-			if (search_find_in_files(search_text, locale_dir, regex_opt, opts))
+			if (search_find_in_files(search_text, locale_dir, opts->str))
 			{
 				ui_combo_box_add_to_history(GTK_COMBO_BOX(user_data), search_text);
 				ui_combo_box_add_to_history(GTK_COMBO_BOX(dir_combo), utf8_dir);
 				gtk_widget_hide(widgets.find_in_files_dialog);
 			}
 			g_free(locale_dir);
+			g_string_free(opts, TRUE);
 		}
 		else
 			ui_set_statusbar(_("No text to find."));
@@ -998,67 +1110,63 @@ on_find_in_files_dialog_response(GtkDialog *dialog, gint response, gpointer user
 
 
 static gboolean
-search_find_in_files(const gchar *search_text, const gchar *dir, fif_match_type regex_opt,
-		fif_options opts)
+search_find_in_files(const gchar *search_text, const gchar *dir, const gchar *opts)
 {
-	gchar **argv_prefix;
-	gchar **grep_cmd_argv;
-	gchar **argv;
-	gchar grep_opts[] = "-nHI    ";
+	gchar **argv_prefix, **argv, **opts_argv;
+	guint opts_argv_len, i;
 	GPid child_pid;
-	gint stdout_fd, stdin_fd, grep_cmd_argv_len, i, grep_opts_len = 4;
+	gint stdout_fd, stdin_fd;
 	GError *error = NULL;
 	gboolean ret = FALSE;
 
 	if (! search_text || ! *search_text || ! dir) return TRUE;
 
-	// first process the grep command (need to split it in a argv because it can be "grep -I")
-	grep_cmd_argv = g_strsplit(app->tools_grep_cmd, " ", -1);
-	grep_cmd_argv_len = g_strv_length(grep_cmd_argv);
-
-	if (! g_file_test(grep_cmd_argv[0], G_FILE_TEST_IS_EXECUTABLE))
+	if (! g_file_test(app->tools_grep_cmd, G_FILE_TEST_IS_EXECUTABLE))
 	{
 		msgwin_status_add(_("Cannot execute grep tool '%s';"
 			" check the path setting in Preferences."), app->tools_grep_cmd);
 		return FALSE;
 	}
 
-	gtk_list_store_clear(msgwindow.store_msg);
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_MESSAGE);
-
-	switch (regex_opt)
-	{
-		default:
-		case FIF_FGREP:	grep_opts[grep_opts_len++] = 'F'; break;
-		case FIF_GREP:	break;
-		case FIF_EGREP:	grep_opts[grep_opts_len++] = 'E'; break;
-	}
-
-	if (! (opts & FIF_CASE_SENSITIVE))
-		grep_opts[grep_opts_len++] = 'i';
-	if (opts & FIF_WHOLE_WORD)
-		grep_opts[grep_opts_len++] = 'w';
-	if (opts & FIF_INVERT_MATCH)
-		grep_opts[grep_opts_len++] = 'v';
-	grep_opts[grep_opts_len] = '\0';
+	opts_argv = g_strsplit(opts, " ", -1);
+	opts_argv_len = g_strv_length(opts_argv);
 
 	// set grep command and options
-	argv_prefix = g_new0(gchar*, grep_cmd_argv_len + 4);
-	for (i = 0; i < grep_cmd_argv_len; i++)
+	argv_prefix = g_new0(gchar*, 1 + opts_argv_len + 3 + 1);	// last +1 for recursive arg
+
+	argv_prefix[0] = g_strdup(app->tools_grep_cmd);
+	for (i = 0; i < opts_argv_len; i++)
 	{
-		argv_prefix[i] = g_strdup(grep_cmd_argv[i]);
+		argv_prefix[i + 1] = g_strdup(opts_argv[i]);
 	}
-	argv_prefix[grep_cmd_argv_len] = g_strdup(grep_opts);
-	argv_prefix[grep_cmd_argv_len + 1] = g_strdup("--");
-	argv_prefix[grep_cmd_argv_len + 2] = g_strdup(search_text);
-	argv_prefix[grep_cmd_argv_len + 3] = NULL;
-	g_strfreev(grep_cmd_argv);
+	g_strfreev(opts_argv);
+
+	i++;	// correct for app->tools_grep_cmd
+	argv_prefix[i++] = g_strdup("--");
+	argv_prefix[i++] = g_strdup(search_text);
 
 	// finally add the arguments(files to be searched)
-	argv = search_get_argv((const gchar**)argv_prefix, dir);
-	g_strfreev(argv_prefix);
+	if (strstr(argv_prefix[1], "r"))	// recursive option set
+	{
+		argv_prefix[i++] = g_strdup(".");
+		argv_prefix[i++] = NULL;
+		argv = argv_prefix;
+	}
+	else
+	{
+		argv_prefix[i++] = NULL;
+		argv = search_get_argv((const gchar**)argv_prefix, dir);
+		g_strfreev(argv_prefix);
+	}
 
-	if (argv == NULL) return FALSE;
+	if (argv == NULL)	// no files
+	{
+		g_strfreev(argv);
+		return FALSE;
+	}
+
+	gtk_list_store_clear(msgwindow.store_msg);
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_MESSAGE);
 
 	if (! g_spawn_async_with_pipes(dir, (gchar**)argv, NULL,
 		G_SPAWN_STDERR_TO_DEV_NULL | G_SPAWN_DO_NOT_REAP_CHILD,
@@ -1072,11 +1180,19 @@ search_find_in_files(const gchar *search_text, const gchar *dir, fif_match_type
 	}
 	else
 	{
+		gchar *str, *utf8_str;
+
 		g_free(msgwindow.find_in_files_dir);
 		msgwindow.find_in_files_dir = g_strdup(dir);
 		utils_set_up_io_channel(stdout_fd, G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
-			search_read_io, NULL);
+			TRUE, search_read_io, NULL);
 		g_child_watch_add(child_pid, search_close_pid, NULL);
+
+		str = g_strdup_printf(_("%s %s -- %s (in directory: %s)"),
+			app->tools_grep_cmd, opts, search_text, dir);
+		utf8_str = utils_get_utf8_from_locale(str);
+		msgwin_msg_add(-1, -1, utf8_str);
+		utils_free_pointers(str, utf8_str, NULL);
 		ret = TRUE;
 	}
 	g_strfreev(argv);
@@ -1092,11 +1208,18 @@ static gchar **search_get_argv(const gchar **argv_prefix, const gchar *dir)
 	guint prefix_len, list_len, i, j;
 	gchar **argv;
 	GSList *list, *item;
+	GError *error = NULL;
 
 	g_return_val_if_fail(dir != NULL, NULL);
 
 	prefix_len = g_strv_length((gchar**)argv_prefix);
-	list = search_get_file_list(dir, &list_len);
+	list = utils_get_file_list(dir, &list_len, &error);
+	if (error)
+	{
+		msgwin_status_add(_("Could not open directory (%s)"), error->message);
+		g_error_free(error);
+		return NULL;
+	}
 	if (list == NULL) return NULL;
 
 	argv = g_new(gchar*, prefix_len + list_len + 1);
@@ -1117,43 +1240,6 @@ static gchar **search_get_argv(const gchar **argv_prefix, const gchar *dir)
 }
 
 
-/* Gets a sorted list of files in the current directory.
- * The list and the data in the list should be freed after use.
- * Returns: The list or NULL if no files found.
- * *length is set to the number of non-NULL data items in the list. */
-static GSList *search_get_file_list(const gchar *path, guint *length)
-{
-	GError *error = NULL;
-	GSList *list = NULL;
-	guint len = 0;
-	GDir *dir;
-
-	g_return_val_if_fail(path != NULL, NULL);
-
-	dir = g_dir_open(path, 0, &error);
-	if (error)
-	{
-		msgwin_status_add(_("Could not open directory (%s)"), error->message);
-		g_error_free(error);
-		*length = 0;
-		return NULL;
-	}
-
-	while (1)
-	{
-		const gchar *filename = g_dir_read_name(dir);
-		if (filename == NULL) break;
-
-		list = g_slist_insert_sorted(list, g_strdup(filename), (GCompareFunc) strcmp);
-		len++;
-	}
-	g_dir_close(dir);
-
-	*length = len;
-	return list;
-}
-
-
 static gboolean search_read_io              (GIOChannel *source,
                                              GIOCondition condition,
                                              gpointer data)
@@ -1178,20 +1264,31 @@ static gboolean search_read_io              (GIOChannel *source,
 static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 {
 #ifdef G_OS_UNIX
-	gchar *msg = _("Search failed.");
+	const gchar *msg = _("Search failed.");
 
 	if (WIFEXITED(status))
 	{
 		switch (WEXITSTATUS(status))
 		{
-			case 0: msg = _("Search completed."); break;
-			case 1: msg = _("No matches found."); break;
-			default: break;
+			case 0:
+			{
+				gint count = gtk_tree_model_iter_n_children(
+					GTK_TREE_MODEL(msgwindow.store_msg), NULL) - 1;
+
+				msgwin_msg_add_fmt(-1, -1, _("Search completed with %d matches."), count);
+				ui_set_statusbar(_("Search completed with %d matches."), count);
+				break;
+			}
+			case 1:
+				msg = _("No matches found.");
+			default:
+				msgwin_msg_add(-1, -1, msg);
+				ui_set_statusbar("%s", msg);
+				break;
 		}
 	}
-
-	msgwin_msg_add(-1, -1, msg);
 #endif
+
 	utils_beep();
 	g_spawn_close_pid(child_pid);
 }
@@ -1199,7 +1296,7 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 
 static gint find_document_usage(gint idx, const gchar *search_text, gint flags)
 {
-	gchar *buffer, *short_file_name, *string;
+	gchar *buffer, *short_file_name;
 	gint pos, line, count = 0;
 	struct TextToFind ttf;
 
@@ -1220,12 +1317,12 @@ static gint find_document_usage(gint idx, const gchar *search_text, gint flags)
 			short_file_name = g_strdup(GEANY_STRING_UNTITLED);
 		else
 			short_file_name = g_path_get_basename(doc_list[idx].file_name);
-		string = g_strdup_printf("%s:%d : %s", short_file_name, line + 1, g_strstrip(buffer));
-		msgwin_msg_add(line + 1, idx, string);
+
+		msgwin_msg_add_fmt(line + 1, idx,
+			"%s:%d : %s", short_file_name, line + 1, g_strstrip(buffer));
 
 		g_free(buffer);
 		g_free(short_file_name);
-		g_free(string);
 		ttf.chrg.cpMin = ttf.chrgText.cpMax + 1;
 		count++;
 	}
@@ -1260,9 +1357,16 @@ void search_find_usage(const gchar *search_text, gint flags, gboolean in_session
 
 	if (! found) // no matches were found
 	{
-		gchar *text = g_strdup_printf(_("No matches found for '%s'."), search_text);
-		ui_set_statusbar("%s", text);
-		msgwin_msg_add(-1, -1, text);
-		g_free(text);
+		ui_set_statusbar(_("No matches found for '%s'."), search_text);
+		msgwin_msg_add_fmt(-1, -1, _("No matches found for '%s'."), search_text);
+	}
+	else
+	{
+		gint count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(msgwindow.store_msg), NULL);
+
+		ui_set_statusbar(_("Found %d matches for '%s'."), count, search_text);
+		msgwin_msg_add_fmt(-1, -1, _("Found %d matches for '%s'."), count, search_text);
 	}
 }
+
+
