@@ -1,5 +1,5 @@
 /*
- *      socket.h - this file is part of Geany, a fast and lightweight IDE
+ *      socket.c - this file is part of Geany, a fast and lightweight IDE
  *
  *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
  *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
@@ -18,7 +18,28 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: socket.c 1270 2007-02-12 18:42:15Z eht16 $
+ * $Id: socket.c 1369 2007-03-06 16:57:09Z ntrel $
+ */
+
+/*
+ * Socket setup and messages handling.
+ * The socket allows detection and messages to be sent to the first running instance of Geany.
+ * Only the first instance loads session files at startup, and opens files from the command-line.
+ */
+
+/*
+ * Little dev doc:
+ * Each command which is sent between two instances (see send_open_command and
+ * socket_lock_input_cb) should have the following scheme:
+ * command name\n
+ * data\n
+ * data\n
+ * ...
+ * .\n
+ * The first thing should be the command name followed by the data belonging to the command and
+ * to mark the end of data send a single '.'. Each message should be ended with \n.
+ *
+ * At the moment the commands open, line and column are available.
  */
 
 
@@ -45,6 +66,7 @@
 #include "socket.h"
 #include "document.h"
 #include "support.h"
+#include "ui_utils.h"
 
 
 
@@ -95,14 +117,23 @@ void send_open_command(gint sock, gint argc, gchar **argv)
 		g_free(line);
 	}
 
+	if (cl_options.goto_column >= 0)
+	{
+		gchar *col = g_strdup_printf("%d\n", cl_options.goto_column);
+		socket_fd_write_all(sock, "column\n", 7);
+		socket_fd_write_all(sock, col, strlen(col));
+		socket_fd_write_all(sock, ".\n", 2);
+		g_free(col);
+	}
+
 	socket_fd_write_all(sock, "open\n", 5);
 
 	for(i = 1; i < argc && argv[i] != NULL; i++)
 	{
 		filename = get_argv_filename(argv[i]);
 
-		if (filename != NULL &&
-			g_file_test(filename, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
+		// if the filename is valid or if a new file should be opened is check on the other side
+		if (filename != NULL)
 		{
 			socket_fd_write_all(sock, filename, strlen(filename));
 			socket_fd_write_all(sock, "\n", 1);
@@ -378,6 +409,8 @@ gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpoint
 	{
 		if (strncmp(buf, "open", 4) == 0)
 		{
+			document_delay_colourise();
+
 			while (socket_fd_gets(sock, buf, sizeof(buf)) != -1 && *buf != '.')
 			{
 				g_strstrip(buf); // remove \n char
@@ -385,8 +418,18 @@ gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpoint
 				if (g_file_test(buf, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
 					document_open_file(-1, buf, 0, FALSE, NULL, NULL);
 				else
-					geany_debug("got data from socket, but it does not look like a filename");
+				{	// create new file if it doesn't exist
+					gint idx;
+
+					idx = document_new_file(buf, NULL);
+					if (DOC_IDX_VALID(idx))
+						ui_add_recent_file(doc_list[idx].file_name);
+					else
+						geany_debug("got data from socket, but it does not look like a filename");
+				}
 			}
+			document_colourise_new();
+
 			gtk_window_deiconify(GTK_WINDOW(app->window));
 #ifdef G_OS_WIN32
 			gtk_window_present(GTK_WINDOW(app->window));
@@ -397,10 +440,19 @@ gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpoint
 			while (socket_fd_gets(sock, buf, sizeof(buf)) != -1 && *buf != '.')
 			{
 				g_strstrip(buf); // remove \n char
-				// on any error we get 0 which should be save enough as fallback
+				// on any error we get 0 which should be safe enough as fallback
 				cl_options.goto_line = atoi(buf);
 			}
 		}
+		else if (strncmp(buf, "column", 6) == 0)
+		{
+			while (socket_fd_gets(sock, buf, sizeof(buf)) != -1 && *buf != '.')
+			{
+				g_strstrip(buf); // remove \n char
+				// on any error we get 0 which should be safe enough as fallback
+				cl_options.goto_column = atoi(buf);
+			}
+		}
 	}
 	socket_fd_close(sock);
 
