@@ -1,8 +1,8 @@
 /*
  *      symbols.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
- *      Copyright 2006 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,13 +18,15 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: symbols.h 1105 2006-12-16 17:18:53Z ntrel $
+ * $Id: symbols.h 1463 2007-04-18 15:21:33Z ntrel $
  */
 
 
 #ifndef GEANY_SYMBOLS_H
 #define GEANY_SYMBOLS_H 1
 
+#include "filetypes.h"
+
 extern const guint TM_GLOBAL_TYPE_MASK;
 
 
@@ -32,14 +34,22 @@ void symbols_global_tags_loaded(gint file_type_idx);
 
 GString *symbols_find_tags_as_string(GPtrArray *tags_array, guint tag_types);
 
+const gchar *symbols_get_context_separator(filetype_id ft_id);
+
 const GList *symbols_get_tag_list(gint idx, guint tag_types);
 
 GString *symbols_get_macro_list();
 
 TMTag *symbols_find_in_workspace(const gchar *tag_name, gint type);
 
-gchar **symbols_get_html_entities();
+const gchar **symbols_get_html_entities();
 
 void symbols_finalize();
 
+gboolean symbols_recreate_tag_list(gint idx);
+
+int symbols_generate_global_tags(int argc, char **argv);
+
+void symbols_show_load_tags_dialog();
+
 #endif
