@@ -1,8 +1,8 @@
 /*
  *      tools.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
- *      Copyright 2006 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: tools.h 1123 2006-12-18 22:55:27Z eht16 $
+ * $Id: tools.h 1334 2007-02-25 14:26:55Z eht16 $
  */
 
 
@@ -27,4 +27,8 @@
 
 void tools_show_dialog_insert_special_chars();
 
+void tools_create_insert_custom_command_menu_items();
+
+void tools_execute_custom_command(gint idx, const gchar *command);
+
 #endif
