@@ -18,9 +18,12 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: utils.c 1270 2007-02-12 18:42:15Z eht16 $
+ * $Id: utils.c 1528 2007-05-11 17:31:06Z ntrel $
  */
 
+/*
+ * General utility functions, non-GTK related.
+ */
 
 #include "SciLexer.h"
 #include "geany.h"
@@ -31,6 +34,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#include <stdarg.h>
 
 #ifdef HAVE_SYS_STAT_H
 # include <sys/stat.h>
@@ -39,6 +43,8 @@
 # include <sys/types.h>
 #endif
 
+#include <glib/gstdio.h>
+
 #include "support.h"
 #include "document.h"
 #include "sciwrappers.h"
@@ -142,18 +148,13 @@ gint utils_get_line_endings(gchar* buffer, glong size)
 
 gboolean utils_isbrace(gchar c)
 {
-	// match < and > only if desired, because I don't like it, but some people do
-	if (app->brace_match_ltgt)
-	{
-		switch (c)
-		{
-			case '<':
-			case '>': return TRUE;
-		}
-	}
-
 	switch (c)
 	{
+		// match < and > only if desired, because I don't like it, but some people do
+		case '<':
+		case '>':
+		return app->brace_match_ltgt;
+
 		case '(':
 		case ')':
 		case '{':
@@ -162,31 +163,22 @@ gboolean utils_isbrace(gchar c)
 		case ']': return TRUE;
 		default:  return FALSE;
 	}
-
-	return FALSE;
 }
 
 
 gboolean utils_is_opening_brace(gchar c)
 {
-	// match < only if desired, because I don't like it, but some people do
-	if (app->brace_match_ltgt)
-	{
-		switch (c)
-		{
-			case '<': return TRUE;
-		}
-	}
-
 	switch (c)
 	{
+		// match < only if desired, because I don't like it, but some people do
+		case '<':
+		return app->brace_match_ltgt;
+
 		case '(':
 		case '{':
 		case '[':  return TRUE;
 		default:  return FALSE;
 	}
-
-	return FALSE;
 }
 
 
@@ -340,8 +332,13 @@ gboolean utils_check_disk_status(gint idx, gboolean force)
 
 	if (! force && doc_list[idx].last_check > (t - GEANY_CHECK_FILE_DELAY)) return FALSE;
 
+#ifdef G_OS_WIN32
+	// don't try to convert the filename on Windows, it should be already in UTF8
+	locale_filename = g_strdup(doc_list[idx].file_name);
+#else
 	locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
-	if (stat(locale_filename, &st) != 0)
+#endif
+	if (g_stat(locale_filename, &st) != 0)
 	{
 		// TODO: warn user file on disk is missing
 	}
@@ -372,34 +369,54 @@ gboolean utils_check_disk_status(gint idx, gboolean force)
 }
 
 
+/* This could perhaps be improved to check for #if, class etc. */
+static gint get_function_fold_number(gint idx)
+{
+	// for Java the functions are always one fold level above the class scope
+	if (FILETYPE_ID(doc_list[idx].file_type) == GEANY_FILETYPES_JAVA)
+		return SC_FOLDLEVELBASE + 1;
+	else
+		return SC_FOLDLEVELBASE;
+}
+
+
 /* Should be used only with utils_get_current_function. */
 static gboolean current_function_changed(gint cur_idx, gint cur_line, gint fold_level)
 {
 	static gint old_line = -2;
 	static gint old_idx = -1;
-	static gint old_fold_level = -1;
+	static gint old_fold_num = -1;
+	const gint fold_num = fold_level & SC_FOLDLEVELNUMBERMASK;
 	gboolean ret;
 
 	// check if the cached line and file index have changed since last time:
-	if (cur_idx != old_idx)
+	if (cur_idx < 0 || cur_idx != old_idx)
 		ret = TRUE;
 	else
 	if (cur_line == old_line)
 		ret = FALSE;
 	else
 	{
-		// if the line has only changed by 1 and fold_level is the same, return unchanged.
+		// if the line has only changed by 1
 		if (abs(cur_line - old_line) == 1)
 		{
-			ret = (fold_level != old_fold_level);
+			const gint fn_fold =
+				get_function_fold_number(cur_idx);
+			/* It's the same function if the fold number hasn't changed, or both the new
+			 * and old fold numbers are above the function fold number. */
+			gboolean same =
+				fold_num == old_fold_num ||
+				(old_fold_num > fn_fold && fold_num > fn_fold);
+
+			ret = ! same;
 		}
 		else ret = TRUE;
 	}
 
-	//record current line and file index for next time
+	// record current line and file index for next time
 	old_line = cur_line;
 	old_idx = cur_idx;
-	old_fold_level = fold_level;
+	old_fold_num = fold_num;
 	return ret;
 }
 
@@ -479,13 +496,28 @@ static gchar *parse_cpp_function_at_line(ScintillaObject *sci, gint tag_line)
 }
 
 
+/* Sets *tagname to point at the current function or tag name.
+ * If idx is -1, reset the cached current tag data to ensure it will be reparsed on the next
+ * call to this function.
+ * Returns: line number of the current tag, or -1 if unknown. */
 gint utils_get_current_function(gint idx, const gchar **tagname)
 {
 	static gint tag_line = -1;
 	static gchar *cur_tag = NULL;
 	gint line;
 	gint fold_level;
-	TMWorkObject * tm_file = doc_list[idx].tm_file;
+	TMWorkObject *tm_file;
+
+	if (! DOC_IDX_VALID(idx))	// reset current function
+	{
+		current_function_changed(-1, -1, -1);
+		g_free(cur_tag);
+		cur_tag = g_strdup(_("unknown"));
+		if (tagname != NULL)
+			*tagname = cur_tag;
+		tag_line = -1;
+		return tag_line;
+	}
 
 	line = sci_get_current_line(doc_list[idx].sci, -1);
 	fold_level = sci_get_fold_level(doc_list[idx].sci, line);
@@ -506,11 +538,12 @@ gint utils_get_current_function(gint idx, const gchar **tagname)
 		tag_line = -1;
 		return tag_line;
 	}
+	tm_file = doc_list[idx].tm_file;
 
 	// if the document has no changes, get the previous function name from TM
 	if(! doc_list[idx].changed && tm_file != NULL && tm_file->tags_array != NULL)
 	{
-		TMTag *tag = TM_TAG(tm_get_current_function(tm_file->tags_array, line));
+		const TMTag *tag = (const TMTag*) tm_get_current_function(tm_file->tags_array, line);
 
 		if (tag != NULL)
 		{
@@ -528,9 +561,7 @@ gint utils_get_current_function(gint idx, const gchar **tagname)
 	if (doc_list[idx].file_type != NULL &&
 		doc_list[idx].file_type->id != GEANY_FILETYPES_ALL)
 	{
-		// for Java the functions are one fold level above the class scope
-		gint top_fold_level = (doc_list[idx].file_type->id == GEANY_FILETYPES_JAVA) ?
-			SC_FOLDLEVELBASE + 1 : SC_FOLDLEVELBASE;
+		const gint fn_fold = get_function_fold_number(idx);
 
 		tag_line = line;
 		do	// find the top level fold point
@@ -538,7 +569,7 @@ gint utils_get_current_function(gint idx, const gchar **tagname)
 			tag_line = sci_get_fold_parent(doc_list[idx].sci, tag_line);
 			fold_level = sci_get_fold_level(doc_list[idx].sci, tag_line);
 		} while (tag_line >= 0 &&
-			(fold_level & SC_FOLDLEVELNUMBERMASK) != top_fold_level);
+			(fold_level & SC_FOLDLEVELNUMBERMASK) != fn_fold);
 
 		if (tag_line >= 0)
 		{
@@ -705,20 +736,22 @@ gboolean utils_str_equal(const gchar *a, const gchar *b)
  * a newly allocated string */
 gchar *utils_remove_ext_from_filename(const gchar *filename)
 {
-	gchar *result = g_malloc0(strlen(filename));
 	gchar *last_dot = strrchr(filename, '.');
-	gint i = 0;
+	gchar *result;
+	gint i;
 
 	if (filename == NULL) return NULL;
 
 	if (! last_dot) return g_strdup(filename);
 
+	result = g_malloc(strlen(filename));	// assumes extension is small, so extra bytes don't matter
+	i = 0;
 	while ((filename + i) != last_dot)
 	{
 		result[i] = filename[i];
 		i++;
 	}
-
+	result[i] = 0;
 	return result;
 }
 
@@ -762,34 +795,24 @@ gchar *utils_get_hostname()
 gint utils_make_settings_dir(const gchar *dir, const gchar *data_dir, const gchar *doc_dir)
 {
 	gint error_nr = 0;
-	gchar *filetypes_readme = g_strconcat(
-					dir, G_DIR_SEPARATOR_S, "template.README", NULL);
+	gchar *conf_file = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "geany.conf", NULL);
 	gchar *filedefs_dir = g_strconcat(dir, G_DIR_SEPARATOR_S,
 					GEANY_FILEDEFS_SUBDIR, G_DIR_SEPARATOR_S, NULL);
-	gchar *filedefs_readme = g_strconcat(dir, G_DIR_SEPARATOR_S, GEANY_FILEDEFS_SUBDIR,
-					G_DIR_SEPARATOR_S, "filetypes.README", NULL);
+
+	gchar *templates_dir = g_strconcat(dir, G_DIR_SEPARATOR_S,
+					GEANY_TEMPLATES_SUBDIR, G_DIR_SEPARATOR_S, NULL);
 
 	if (! g_file_test(dir, G_FILE_TEST_EXISTS))
 	{
 		geany_debug("creating config directory %s", dir);
-#ifdef G_OS_WIN32
-		if (mkdir(dir) != 0) error_nr = errno;
-#else
-		if (mkdir(dir, 0700) != 0) error_nr = errno;
-#endif
+		error_nr = utils_mkdir(dir, FALSE);
 	}
 
-	if (error_nr == 0 && ! g_file_test(filetypes_readme, G_FILE_TEST_EXISTS))
-	{	// try to write template.README
-		gchar *text;
-		text = g_strconcat(
-"There are several template files in this directory. For these templates you can use wildcards.\n\
-For more information read the documentation (in ", doc_dir, "index.html or visit " GEANY_HOMEPAGE ").",
-					NULL);
-		error_nr = utils_write_file(filetypes_readme, text);
-		g_free(text);
+	if (error_nr == 0 && ! g_file_test(conf_file, G_FILE_TEST_EXISTS))
+	{	// try to write geany.conf
+		error_nr = utils_write_file(conf_file, "");
 
- 		if (error_nr == 0 && ! g_file_test(filetypes_readme, G_FILE_TEST_EXISTS))
+ 		if (error_nr == 0 && ! g_file_test(conf_file, G_FILE_TEST_EXISTS))
 		{ // check whether write test was successful, otherwise directory is not writable
 			geany_debug("The chosen configuration directory is not writable.");
 			errno = EPERM;
@@ -799,13 +822,12 @@ For more information read the documentation (in ", doc_dir, "index.html or visit
 	// make subdir for filetype definitions
 	if (error_nr == 0)
 	{
+		gchar *filedefs_readme = g_strconcat(dir, G_DIR_SEPARATOR_S, GEANY_FILEDEFS_SUBDIR,
+					G_DIR_SEPARATOR_S, "filetypes.README", NULL);
+
 		if (! g_file_test(filedefs_dir, G_FILE_TEST_EXISTS))
 		{
-#ifdef G_OS_WIN32
-			if (mkdir(filedefs_dir) != 0) error_nr = errno;
-#else
-			if (mkdir(filedefs_dir, 0700) != 0) error_nr = errno;
-#endif
+			error_nr = utils_mkdir(filedefs_dir, FALSE);
 		}
 		if (error_nr == 0 && ! g_file_test(filedefs_readme, G_FILE_TEST_EXISTS))
 		{
@@ -816,11 +838,34 @@ For more information read the documentation (in ", doc_dir, "index.html or visit
 			utils_write_file(filedefs_readme, text);
 			g_free(text);
 		}
+		g_free(filedefs_readme);
+	}
+
+	// make subdir for template files
+	if (error_nr == 0)
+	{
+		gchar *templates_readme = g_strconcat(dir, G_DIR_SEPARATOR_S, GEANY_TEMPLATES_SUBDIR,
+					G_DIR_SEPARATOR_S, "templates.README", NULL);
+
+		if (! g_file_test(templates_dir, G_FILE_TEST_EXISTS))
+		{
+			error_nr = utils_mkdir(templates_dir, FALSE);
+		}
+		if (error_nr == 0 && ! g_file_test(templates_readme, G_FILE_TEST_EXISTS))
+		{
+			gchar *text = g_strconcat(
+"There are several template files in this directory. For these templates you can use wildcards.\n\
+For more information read the documentation (in ", doc_dir, "index.html or visit " GEANY_HOMEPAGE ").",
+					NULL);
+			utils_write_file(templates_readme, text);
+			g_free(text);
+		}
+		g_free(templates_readme);
 	}
 
-	g_free(filetypes_readme);
 	g_free(filedefs_dir);
-	g_free(filedefs_readme);
+	g_free(templates_dir);
+	g_free(conf_file);
 
 	return error_nr;
 }
@@ -993,13 +1038,19 @@ void utils_switch_document(gint direction)
 	gint page_count = gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook));
 	gint cur_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(app->notebook));
 
-	if (direction == LEFT && cur_page > 0)
+	if (direction == LEFT)
 	{
-		gtk_notebook_set_current_page(GTK_NOTEBOOK(app->notebook), cur_page - 1);
+		if (cur_page > 0)
+			gtk_notebook_set_current_page(GTK_NOTEBOOK(app->notebook), cur_page - 1);
+		else
+			gtk_notebook_set_current_page(GTK_NOTEBOOK(app->notebook), page_count - 1);
 	}
-	else if (direction == RIGHT && cur_page < page_count)
+	else if (direction == RIGHT)
 	{
-		gtk_notebook_set_current_page(GTK_NOTEBOOK(app->notebook), cur_page + 1);
+		if (cur_page < page_count - 1)
+			gtk_notebook_set_current_page(GTK_NOTEBOOK(app->notebook), cur_page + 1);
+		else
+			gtk_notebook_set_current_page(GTK_NOTEBOOK(app->notebook), 0);
 	}
 }
 
@@ -1032,15 +1083,6 @@ void utils_replace_filename(gint idx)
 }
 
 
-/* wrapper function to let strcmp work with GeanySymbol struct */
-gint utils_compare_symbol(const GeanySymbol *a, const GeanySymbol *b)
-{
-	if (a == NULL || b == NULL) return 0;
-
-	return strcmp(a->str, b->str);
-}
-
-
 gchar *utils_get_hex_from_color(GdkColor *color)
 {
 	gchar *buffer = g_malloc0(9);
@@ -1193,17 +1235,22 @@ gchar *utils_get_current_time_string()
 }
 
 
-GIOChannel *utils_set_up_io_channel(gint fd, GIOCondition cond, GIOFunc func, gpointer data)
+GIOChannel *utils_set_up_io_channel(
+				gint fd, GIOCondition cond, gboolean nblock, GIOFunc func, gpointer data)
 {
 	GIOChannel *ioc;
-	GError *error = NULL;
-	const gchar *encoding;
+	//const gchar *encoding;
 
 	ioc = g_io_channel_unix_new(fd);
 
-	g_io_channel_set_flags(ioc, G_IO_FLAG_NONBLOCK, NULL);
+	if (nblock)
+		g_io_channel_set_flags(ioc, G_IO_FLAG_NONBLOCK, NULL);
+
+	g_io_channel_set_encoding(ioc, NULL, NULL);
+/*
 	if (! g_get_charset(&encoding))
 	{	// hope this works reliably
+		GError *error = NULL;
 		g_io_channel_set_encoding(ioc, encoding, &error);
 		if (error)
 		{
@@ -1212,6 +1259,7 @@ GIOChannel *utils_set_up_io_channel(gint fd, GIOCondition cond, GIOFunc func, gp
 			return ioc;
 		}
 	}
+*/
 	// "auto-close" ;-)
 	g_io_channel_set_close_on_unref(ioc, TRUE);
 
@@ -1395,21 +1443,66 @@ gboolean utils_wrap_string(gchar *string, gint wrapstart)
 
 gchar *utils_get_locale_from_utf8(const gchar *utf8_text)
 {
+#ifdef G_OS_WIN32
+	// just do nothing on Windows platforms, this ifdef is just to prevent unwanted conversions
+	// which would result in wrongly converted strings
+	return g_strdup(utf8_text);
+#else
 	gchar *locale_text = g_locale_from_utf8(utf8_text, -1, NULL, NULL, NULL);
 	if (locale_text == NULL) locale_text = g_strdup(utf8_text);
 	return locale_text;
+#endif
 }
 
 
 gchar *utils_get_utf8_from_locale(const gchar *locale_text)
 {
+#ifdef G_OS_WIN32
+	// just do nothing on Windows platforms, this ifdef is just to prevent unwanted conversions
+	// which would result in wrongly converted strings
+	return g_strdup(locale_text);
+#else
 	gchar *utf8_text = g_locale_to_utf8(locale_text, -1, NULL, NULL, NULL);
 	if (utf8_text == NULL) utf8_text = g_strdup(locale_text);
 	return utf8_text;
+#endif
 }
 
 
-/* frees all passed pointers if they are non-NULL, the first argument is nothing special,
+/* Returns a string containing whitespace of the amount a according to the
+ * setting app->pref_editor_use_tabs filled with simple space characters or with the right amount
+ * of tabulator characters (a is filled with tabulators *and* spaces if a isn't a multiple of
+ * app->pref_editor_tab_width).
+ * If alternative is set to TRUE, it returns the opposite of app->pref_editor_use_tabs. */
+gchar *utils_get_whitespace(gint a, gboolean alternative)
+{
+	gchar *str;
+	gboolean use_tabs;
+
+	g_return_val_if_fail(a > 0, NULL);
+
+	use_tabs = (alternative) ? ! app->pref_editor_use_tabs : app->pref_editor_use_tabs;
+
+	if (use_tabs)
+	{	// first fill text with tabluators and fill the rest with spaces
+		gint tabs = a / app->pref_editor_tab_width;
+		gint spaces = a % app->pref_editor_tab_width;
+		gint len = tabs + spaces;
+
+		str = g_malloc(len + 1);
+
+		memset(str, '\t', tabs);
+		memset(str + tabs, ' ', spaces);
+		str[len] = '\0';
+ 	}
+	else
+		str = g_strnfill(a, ' ');
+
+	return str;
+}
+
+
+/* Frees all passed pointers if they are non-NULL, the first argument is nothing special,
  * it will also be freed, the list should be ended with NULL */
 void utils_free_pointers(gpointer first, ...)
 {
@@ -1428,4 +1521,167 @@ void utils_free_pointers(gpointer first, ...)
 }
 
 
+/* Creates a string array deep copy of a series of non-NULL strings.
+ * The first argument is nothing special.
+ * The list must be ended with NULL.
+ * If first is NULL, NULL is returned. */
+gchar **utils_strv_new(gchar *first, ...)
+{
+	gsize strvlen, i;
+	va_list args;
+	gchar *str;
+	gchar **strv;
+
+	if (first == NULL)
+		return NULL;
+
+	strvlen = 1;	// for first argument
+
+    // count other arguments
+    va_start(args, first);
+    for (; va_arg(args, gchar*) != NULL; strvlen++);
+	va_end(args);
+
+	strv = g_new(gchar*, strvlen + 1);	// +1 for NULL terminator
+	strv[0] = g_strdup(first);
+
+    va_start(args, first);
+    for (i = 1; str = va_arg(args, gchar*), str != NULL; i++)
+    {
+		strv[i] = g_strdup(str);
+	}
+	va_end(args);
+
+	strv[i] = NULL;
+	return strv;
+}
+
+
+#if ! GLIB_CHECK_VERSION(2, 8, 0)
+// Taken from GLib SVN, 2007-03-10
+/**
+ * g_mkdir_with_parents:
+ * @pathname: a pathname in the GLib file name encoding
+ * @mode: permissions to use for newly created directories
+ *
+ * Create a directory if it doesn't already exist. Create intermediate
+ * parent directories as needed, too.
+ *
+ * Returns: 0 if the directory already exists, or was successfully
+ * created. Returns -1 if an error occurred, with errno set.
+ *
+ * Since: 2.8
+ */
+int
+g_mkdir_with_parents (const gchar *pathname,
+		      int          mode)
+{
+  gchar *fn, *p;
+
+  if (pathname == NULL || *pathname == '\0')
+    {
+      errno = EINVAL;
+      return -1;
+    }
+
+  fn = g_strdup (pathname);
+
+  if (g_path_is_absolute (fn))
+    p = (gchar *) g_path_skip_root (fn);
+  else
+    p = fn;
+
+  do
+    {
+      while (*p && !G_IS_DIR_SEPARATOR (*p))
+	p++;
+
+      if (!*p)
+	p = NULL;
+      else
+	*p = '\0';
+
+      if (!g_file_test (fn, G_FILE_TEST_EXISTS))
+	{
+	  if (g_mkdir (fn, mode) == -1)
+	    {
+	      int errno_save = errno;
+	      g_free (fn);
+	      errno = errno_save;
+	      return -1;
+	    }
+	}
+      else if (!g_file_test (fn, G_FILE_TEST_IS_DIR))
+	{
+	  g_free (fn);
+	  errno = ENOTDIR;
+	  return -1;
+	}
+      if (p)
+	{
+	  *p++ = G_DIR_SEPARATOR;
+	  while (*p && G_IS_DIR_SEPARATOR (*p))
+	    p++;
+	}
+    }
+  while (p);
+
+  g_free (fn);
+
+  return 0;
+}
+#endif
+
+
+gint utils_mkdir(const gchar *path, gboolean create_parent_dirs)
+{
+	gint mode = 0700;
+	gint result;
+
+	if (path == NULL || strlen(path) == 0)
+		return EFAULT;
+
+	result = (create_parent_dirs) ? g_mkdir_with_parents(path, mode) : g_mkdir(path, mode);
+	if (result != 0)
+		return errno;
+	return 0;
+}
+
+
+/* Gets a sorted list of files in the specified directory.
+ * The list and the data in the list should be freed after use.
+ * Returns: The list or NULL if no files found.
+ * length will point to the number of non-NULL data items in the list, unless NULL.
+ * error is the location for storing a possible error, or NULL. */
+GSList *utils_get_file_list(const gchar *path, guint *length, GError **error)
+{
+	GSList *list = NULL;
+	guint len = 0;
+	GDir *dir;
+
+	if (error)
+		*error = NULL;
+	if (length)
+		*length = 0;
+	g_return_val_if_fail(path != NULL, NULL);
+
+	dir = g_dir_open(path, 0, error);
+	if (dir == NULL)
+		return NULL;
+
+	while (1)
+	{
+		const gchar *filename = g_dir_read_name(dir);
+		if (filename == NULL) break;
+
+		list = g_slist_insert_sorted(list, g_strdup(filename), (GCompareFunc) strcmp);
+		len++;
+	}
+	g_dir_close(dir);
+
+	if (length)
+		*length = len;
+	return list;
+}
+
 
