@@ -1,7 +1,8 @@
 /*
  *      vte.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,20 +18,21 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: vte.h 774 2006-08-30 18:53:45Z eht16 $
+ * $Id: vte.h 1188 2007-01-14 17:36:42Z eht16 $
  */
 
 
 #ifndef GEANY_VTE_H
 #define GEANY_VTE_H 1
 
+#ifdef HAVE_VTE
+
 /* include stdlib.h AND unistd.h, because on GNU/Linux pid_t seems to be
  * in stdlib.h, on FreeBSD in unistd.h */
 #include <stdlib.h>
 #include <unistd.h>
 
 
-#ifdef HAVE_VTE
 typedef struct
 {
 	gboolean load_vte;
@@ -51,6 +53,7 @@ typedef struct
 	gboolean scroll_on_out;
 	gboolean ignore_menu_bar_accel;
 	gboolean follow_path;
+	gboolean run_in_vte;
 	gint scrollback_lines;
 	gchar *emulation;
 	gchar *shell;
@@ -59,7 +62,6 @@ typedef struct
 	GdkColor *colour_back;
 } VteConfig;
 VteConfig *vc;
-#endif
 
 
 void vte_init(void);
@@ -72,7 +74,9 @@ void vte_send_cmd(const gchar *cmd);
 
 const gchar* vte_get_working_directory(void);
 
-void vte_cwd(const gchar *filename);
+void vte_cwd(const gchar *filename, gboolean force);
+
+void vte_append_preferences_tab();
 
 /*
 void vte_drag_data_received(GtkWidget *widget, GdkDragContext  *drag_context, gint x, gint y,
@@ -129,5 +133,6 @@ struct VteFunctions
 	void (*vte_terminal_im_append_menuitems) (VteTerminal *terminal, GtkMenuShell *menushell);
 };
 
+#endif
 
 #endif
