@@ -33,6 +33,7 @@
 #include "tm_workspace.h"
 #include "tm_project.h"
 
+
 static TMWorkspace *theWorkspace = NULL;
 guint workspace_class_id = 0;
 
@@ -133,11 +134,16 @@ gboolean tm_workspace_remove_object(TMWorkObject *w, gboolean free)
 	return FALSE;
 }
 
+static TMTagAttrType global_tags_sort_attrs[] =
+{
+	tm_tag_attr_name_t, tm_tag_attr_scope_t,
+	tm_tag_attr_type_t, tm_tag_attr_arglist_t, 0
+};
+
 gboolean tm_workspace_load_global_tags(const char *tags_file, gint mode)
 {
 	FILE *fp;
 	TMTag *tag;
-	TMTagAttrType attr[] = { tm_tag_attr_name_t, 0 };
 
 	if (NULL == (fp = fopen(tags_file, "r")))
 		return FALSE;
@@ -151,13 +157,98 @@ gboolean tm_workspace_load_global_tags(const char *tags_file, gint mode)
 
 	// resort the whole array, because tm_tags_find expects a sorted array and it is not sorted
 	// when global.tags, php.tags and latex.tags are loaded at the same time
-	tm_tags_sort(theWorkspace->global_tags, attr, TRUE);
+	tm_tags_sort(theWorkspace->global_tags, global_tags_sort_attrs, TRUE);
 
 	return TRUE;
 }
 
+static guint tm_file_inode_hash(gconstpointer key)
+{
+	struct stat file_stat;
+	const char *filename = (const char*)key;
+	if (stat(filename, &file_stat) == 0)
+	{
+#ifdef TM_DEBUG
+		g_message ("Hash for '%s' is '%d'\n", filename, file_stat.st_ino);
+#endif
+		return g_direct_hash (GUINT_TO_POINTER (file_stat.st_ino));
+	} else {
+		return 0;
+	}
+}
+
+static void tm_move_entries_to_g_list(gpointer key, gpointer value, gpointer user_data)
+{
+	GList **pp_list = (GList**)user_data;
+
+	if (user_data == NULL)
+		return;
+
+	*pp_list = g_list_prepend(*pp_list, value);
+}
+
+static void write_includes_file(FILE *fp, GList *includes_files)
+{
+	GList *node;
+
+	node = includes_files;
+	while (node)
+	{
+		char *str = g_strdup_printf("#include \"%s\"\n", (char*)node->data);
+		int str_len = strlen(str);
+
+		fwrite(str, str_len, 1, fp);
+		free(str);
+		node = g_list_next (node);
+	}
+}
+
+
+static void append_to_temp_file(FILE *fp, GList *file_list)
+{
+	GList *node;
+
+	node = file_list;
+	while (node)
+	{
+		const char *fname = node->data;
+		char *contents;
+		size_t length;
+		GError *err = NULL;
+
+		if (! g_file_get_contents(fname, &contents, &length, &err))
+		{
+			fprintf(stderr, "Unable to read file: %s\n", err->message);
+			g_error_free(err);
+		}
+		else
+		{
+			fwrite(contents, length, 1, fp);
+			fwrite("\n", 1, 1, fp);	// in case file doesn't end in newline (e.g. windows).
+			g_free(contents);
+		}
+		node = g_list_next (node);
+	}
+}
+
+static gint get_global_tag_type_mask(gint lang)
+{
+	switch (lang)
+	{
+		case 0:
+		case 1:
+			// C/C++
+			return tm_tag_class_t | tm_tag_typedef_t | tm_tag_enum_t | tm_tag_enumerator_t |
+				tm_tag_prototype_t |
+				tm_tag_function_t | tm_tag_method_t |	// for inline functions
+				tm_tag_macro_t | tm_tag_macro_with_arg_t;
+		default:
+			return tm_tag_max_t;
+	}
+}
+
 gboolean tm_workspace_create_global_tags(const char *pre_process, const char **includes
-  , int includes_count, const char *tags_file)
+  , int includes_count, const char *tags_file, int lang)
 {
 #ifdef HAVE_GLOB_H
 	glob_t globbuf;
@@ -169,11 +260,8 @@ gboolean tm_workspace_create_global_tags(const char *pre_process, const char **i
 	FILE *fp;
 	TMWorkObject *source_file;
 	GPtrArray *tags_array;
+	GHashTable *includes_files_hash;
 	GList *includes_files = NULL;
-	guint list_len;
-	guint idx_main;
-	guint idx_sub;
-	int remove_count = 0;
 #ifdef G_OS_WIN32
 	char *temp_file = g_strdup_printf("%s_%d_%ld_1.cpp", P_tmpdir, getpid(), time(NULL));
 	char *temp_file2 = g_strdup_printf("%s_%d_%ld_2.cpp", P_tmpdir, getpid(), time(NULL));
@@ -181,13 +269,18 @@ gboolean tm_workspace_create_global_tags(const char *pre_process, const char **i
 	char *temp_file = g_strdup_printf("%s/%d_%ld_1.cpp", P_tmpdir, getpid(), time(NULL));
 	char *temp_file2 = g_strdup_printf("%s/%d_%ld_2.cpp", P_tmpdir, getpid(), time(NULL));
 #endif
-	TMTagAttrType sort_attrs[] = { tm_tag_attr_name_t, tm_tag_attr_scope_t
-		, tm_tag_attr_type_t, 0};
+
 	if (NULL == (fp = fopen(temp_file, "w")))
 		return FALSE;
 
+	includes_files_hash = g_hash_table_new_full (tm_file_inode_hash,
+												 g_direct_equal,
+												 NULL, g_free);
+
 #ifdef HAVE_GLOB_H
 	globbuf.gl_offs = 0;
+
+	if (includes[0][0] == '"')	// leading \" char for glob matching
 	for(idx_inc = 0; idx_inc < includes_count; idx_inc++)
 	{
  		int dirty_len = strlen(includes[idx_inc]);
@@ -195,96 +288,94 @@ gboolean tm_workspace_create_global_tags(const char *pre_process, const char **i
 		strncpy(clean_path, includes[idx_inc] + 1, dirty_len - 1);
 		clean_path[dirty_len - 2] = 0;
 
-		//printf("[o][%s]\n", clean_path);
+#ifdef TM_DEBUG
+		g_message ("[o][%s]\n", clean_path);
+#endif
 		glob(clean_path, 0, NULL, &globbuf);
-		//printf("matches: %d\n", globbuf.gl_pathc);
+
+#ifdef TM_DEBUG
+		g_message ("matches: %d\n", globbuf.gl_pathc);
+#endif
+
 		for(idx_glob = 0; idx_glob < globbuf.gl_pathc; idx_glob++)
 		{
-			includes_files = g_list_append(includes_files, strdup(globbuf.gl_pathv[idx_glob]));
-			//printf(">>> %s\n", globbuf.gl_pathv[idx_glob]);
+#ifdef TM_DEBUG
+			g_message (">>> %s\n", globbuf.gl_pathv[idx_glob]);
+#endif
+			if (!g_hash_table_lookup(includes_files_hash,
+									globbuf.gl_pathv[idx_glob]))
+			{
+				char* file_name_copy = strdup(globbuf.gl_pathv[idx_glob]);
+				g_hash_table_insert(includes_files_hash, file_name_copy,
+									file_name_copy);
+#ifdef TM_DEBUG
+				g_message ("Added ...\n");
+#endif
+			}
 		}
 		globfree(&globbuf);
 		free(clean_path);
   	}
-#else
+  	else
+#endif
+	// no glob support or globbing not wanted
 	for(idx_inc = 0; idx_inc < includes_count; idx_inc++)
 	{
- 		includes_files = g_list_append(includes_files, strdup(includes[idx_inc]));
+		if (!g_hash_table_lookup(includes_files_hash,
+									includes[idx_inc]))
+		{
+			char* file_name_copy = strdup(includes[idx_inc]);
+			g_hash_table_insert(includes_files_hash, file_name_copy,
+								file_name_copy);
+		}
   	}
-#endif
-
 
 	/* Checks for duplicate file entries which would case trouble */
-	{
-		struct stat main_stat;
-		struct stat sub_stat;
+	g_hash_table_foreach(includes_files_hash, tm_move_entries_to_g_list,
+						 &includes_files);
 
-		remove_count = 0;
+	includes_files = g_list_reverse (includes_files);
 
-		list_len = g_list_length(includes_files);
-
-		/* We look for files with the same inode */
-		for(idx_main = 0; idx_main < list_len; idx_main++)
-		{
-//			printf("%d / %d\n", idx_main, list_len - 1);
-			stat(g_list_nth_data(includes_files, idx_main), &main_stat);
-			for(idx_sub = idx_main + 1; idx_sub < list_len; idx_sub++)
-			{
-				GList *element = NULL;
-
-				stat(g_list_nth_data(includes_files, idx_sub), &sub_stat);
-
-
-				if(main_stat.st_ino != sub_stat.st_ino)
-					continue;
-
-				/* Inodes match */
-
-				element = g_list_nth(includes_files, idx_sub);
-
-/*				printf("%s == %s\n", g_list_nth_data(includes_files, idx_main),
-										 g_list_nth_data(includes_files, idx_sub)); */
-
-				/* We delete the duplicate entry from the list */
-				includes_files = g_list_remove_link(includes_files, element);
-				remove_count++;
-
-				/* Don't forget to free the mallocs (we duplicated every string earlier!) */
-				free(element->data);
-
-				idx_sub--; /* Cause the inner loop not to move; good since we removed
-							   an element at the current position; we don't have to worry
-							   about the outer loop because the inner loop always starts
-							   after the outer loop's index */
-
-				list_len = g_list_length(includes_files);
-			}
-		}
-	}
-
-
-	printf("writing out files to %s\n", temp_file);
-	for(idx_main = 0; idx_main < g_list_length(includes_files); idx_main++)
-	{
-		char *str = g_strdup_printf("#include \"%s\"\n",
-									(char*)g_list_nth_data(includes_files,
-														   idx_main));
-		int str_len = strlen(str);
-
-		fwrite(str, str_len, 1, fp);
-
-		free(str);
-		free(g_list_nth(includes_files, idx_main) -> data);
-	}
+#ifdef TM_DEBUG
+	g_message ("writing out files to %s\n", temp_file);
+#endif
+	if (pre_process != NULL)
+		write_includes_file(fp, includes_files);
+	else
+		append_to_temp_file(fp, includes_files);
 
+	g_list_free (includes_files);
+	g_hash_table_destroy(includes_files_hash);
+	includes_files_hash = NULL;
+	includes_files = NULL;
 	fclose(fp);
 
-	command = g_strdup_printf("%s %s >%s", pre_process, temp_file, temp_file2);
-	system(command);
-	g_free(command);
-	unlink(temp_file);
-	g_free(temp_file);
-	source_file = tm_source_file_new(temp_file2, TRUE, NULL);
+	/* FIXME: The following grep command removes the lines
+	 * G_BEGIN_DECLS and G_END_DECLS from the header files. The reason is
+	 * that in tagmanager, the files are not correctly parsed and the typedefs
+	 * following these lines are incorrectly parsed. The real fix should,
+	 * of course be in tagmanager (c) parser. This is just a temporary fix.
+	 */
+	if (pre_process != NULL)
+	{
+		command = g_strdup_printf("%s %s | grep -v -E '^\\s*(G_BEGIN_DECLS|G_END_DECLS)\\s*$' > %s",
+							  pre_process, temp_file, temp_file2);
+#ifdef TM_DEBUG
+		g_message("Executing: %s", command);
+#endif
+		system(command);
+		g_free(command);
+		unlink(temp_file);
+		g_free(temp_file);
+	}
+	else
+	{
+		// no pre-processing needed, so temp_file2 = temp_file
+		g_free(temp_file2);
+		temp_file2 = temp_file;
+		temp_file = NULL;
+	}
+	source_file = tm_source_file_new(temp_file2, TRUE, tm_source_file_get_lang_name(lang));
 	if (NULL == source_file)
 	{
 		unlink(temp_file2);
@@ -297,8 +388,7 @@ gboolean tm_workspace_create_global_tags(const char *pre_process, const char **i
 		tm_source_file_free(source_file);
 		return FALSE;
 	}
-	tags_array = tm_tags_extract(source_file->tags_array, tm_tag_class_t |
-	  tm_tag_typedef_t | tm_tag_prototype_t | tm_tag_enum_t | tm_tag_macro_with_arg_t);
+	tags_array = tm_tags_extract(source_file->tags_array, get_global_tag_type_mask(lang));
 	if ((NULL == tags_array) || (0 == tags_array->len))
 	{
 		if (tags_array)
@@ -306,7 +396,7 @@ gboolean tm_workspace_create_global_tags(const char *pre_process, const char **i
 		tm_source_file_free(source_file);
 		return FALSE;
 	}
-	if (FALSE == tm_tags_sort(tags_array, sort_attrs, TRUE))
+	if (FALSE == tm_tags_sort(tags_array, global_tags_sort_attrs, TRUE))
 	{
 		tm_source_file_free(source_file);
 		return FALSE;
@@ -316,10 +406,11 @@ gboolean tm_workspace_create_global_tags(const char *pre_process, const char **i
 		tm_source_file_free(source_file);
 		return FALSE;
 	}
-	for (i=0; i < tags_array->len; ++i)
+	for (i = 0; i < tags_array->len; ++i)
 	{
 		tm_tag_write(TM_TAG(tags_array->pdata[i]), fp, tm_tag_attr_type_t
-		  | tm_tag_attr_scope_t | tm_tag_attr_arglist_t | tm_tag_attr_vartype_t);
+		  | tm_tag_attr_scope_t | tm_tag_attr_arglist_t | tm_tag_attr_vartype_t
+		  | tm_tag_attr_pointer_t);
 	}
 	fclose(fp);
 	tm_source_file_free(source_file);
@@ -352,7 +443,7 @@ void tm_workspace_recreate_tags_array(void)
 	guint i, j;
 	TMWorkObject *w;
 	TMTagAttrType sort_attrs[] = { tm_tag_attr_name_t, tm_tag_attr_file_t
-		, tm_tag_attr_scope_t, tm_tag_attr_type_t, 0};
+		, tm_tag_attr_scope_t, tm_tag_attr_type_t, tm_tag_attr_arglist_t, 0};
 
 #ifdef TM_DEBUG
 	g_message("Recreating workspace tags array");
@@ -521,7 +612,86 @@ const GPtrArray *tm_workspace_find(const char *name, int type, TMTagAttrType *at
 		}
 	}
 
-	tm_tags_sort(tags, attrs, TRUE);
+	if (attrs)
+		tm_tags_sort(tags, attrs, TRUE);
+	return tags;
+}
+
+static gboolean match_langs(gint lang, const TMTag *tag)
+{
+	if (tag->atts.entry.file)
+	{	// workspace tag
+		if (lang == tag->atts.entry.file->lang)
+			return TRUE;
+	}
+	else
+	{	// global tag
+		if (lang == tag->atts.file.lang)
+			return TRUE;
+	}
+	return FALSE;
+}
+
+/* scope can be NULL.
+ * lang can be -1 */
+static int
+fill_find_tags_array (GPtrArray *dst, const GPtrArray *src,
+					  const char *name, const char *scope, int type, gboolean partial,
+					  gint lang, gboolean first)
+{
+	TMTag **match;
+	int tagIter, count;
+
+	if ((!src) || (!dst) || (!name) || (!*name))
+		return 0;
+
+	match = tm_tags_find (src, name, partial, &count);
+	if (count && match && *match)
+	{
+		for (tagIter = 0; tagIter < count; ++tagIter)
+		{
+			if (! scope || (match[tagIter]->atts.entry.scope &&
+				0 == strcmp(match[tagIter]->atts.entry.scope, scope)))
+			{
+				if (type & match[tagIter]->type)
+				if (lang == -1 || match_langs(lang, match[tagIter]))
+				{
+					g_ptr_array_add (dst, match[tagIter]);
+					if (first)
+						break;
+				}
+			}
+		}
+	}
+	return dst->len;
+}
+
+
+// adapted from tm_workspace_find, Anjuta 2.02
+const GPtrArray *
+tm_workspace_find_scoped (const char *name, const char *scope, gint type,
+		TMTagAttrType *attrs, gboolean partial, langType lang, gboolean global_search)
+{
+	static GPtrArray *tags = NULL;
+
+	if ((!theWorkspace))
+		return NULL;
+
+	if (tags)
+		g_ptr_array_set_size (tags, 0);
+	else
+		tags = g_ptr_array_new ();
+
+	fill_find_tags_array (tags, theWorkspace->work_object.tags_array,
+						  name, scope, type, partial, lang, FALSE);
+	if (global_search)
+	{
+		// for a scoped tag, I think we always want the same language
+		fill_find_tags_array (tags, theWorkspace->global_tags,
+							  name, scope, type, partial, lang, FALSE);
+	}
+	if (attrs)
+		tm_tags_sort (tags, attrs, TRUE);
 	return tags;
 }
 
